; Custom Bounce Block Sprites
; By Kaijyuu
;
; See readme if you don't know how to use

HEADER
LOROM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Defines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!FreeSpace 	= $138000	; point to frickin freespace

	!RAM_CUSTBLOCK	= $7EC275	; 10 byte table that holds the map16 number for custom block changes
					; bytes 1-8 correspond to the bounce block sprite numbers. 
					; 9-10 are for other map16 changes


ORG $00BFBC			; routine that changes block's map16 in smw. Used for all block changes
	JML BLOCK_STUFF		; hijack it so bounce block sprites can change into any map16 number

ORG $029052			; Hijack bounce block sprite routines
	JML MAIN


ORG $029347			;\ Fixes the bug where hitting a block under a coin
	JSR COIN_FIX		; |will leave an invisible solid block in place of the coin
				; |
ORG $02FFE2			; |(random freespace in bank 02)
				; |
COIN_FIX:			; |This is not neccessary for this patch to run, 
	LDA #$02		; |so delete it if for some reason you don't want it
	JMP $91BA		;/

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;New main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ORG !FreeSpace

db "STAR"				;\
dw MainEnd-MainStart			; |Write RATS tag #1 to protect main code
dw MainEnd-MainStart^#$FFFF		;/

MainStart:

POINTERS:
dw Bounce8
dw Bounce9
dw BounceA
dw BounceB
dw BounceC
dw BounceD
dw BounceE		; pointers
dw BounceF		
dw Bounce10
dw Bounce11		; feel free to add more if needed
dw Bounce12		; max possible would be 86
dw Bounce13
dw Bounce14
dw Bounce15
dw Bounce16
dw Bounce17
dw Bounce18
dw Bounce19
dw Bounce1A
dw Bounce1B
dw Bounce1C
dw Bounce1D
dw Bounce1E
dw Bounce1F

BOUNCE_SPEED_Y:			; Generic bounce sprite speed modifiers for y speed
	db $F0,$00,$00,$10	; these are added to the y speed every frame for sprites that use it
BOUNCE_SPEED_X:			; Generic bounce sprite speed modifiers for x speed
	db $00,$F0,$10,$00	;
BOUNCE_OAM: 			; OAM addresses for bounce extended sprites
	db $10,$14,$18,$1C

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite code starts here
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Bounce8:	; SMB3 brick
	; graphics

	JSR SUB_OFFSCREEN	; check if offscreen
				; $00 holds the x position relative to screen border
				; $01 holds the y
				; carry set if offscreen
	BCS SMB3_SKIP_GFX	; skip graphics if offscreen

	LDA BOUNCE_OAM,y	; get OAM offset
	TAX			; stick in x

	LDA $00			; x pos
	STA $0200,X		; store to OAM
	
	LDA $01			; y pos
	STA $0201,x		; store to OAM

	LDA #$2A		; tile number
	STA $0202,x 		; store to OAM

	LDA $1901,y		; properties
	ORA $64			; add in prority bits from level settings
	STA $0203,x		; store to OAM

	TXA			; set tile size
	LSR			; 
	LSR			;
	TAX			;
	LDA #$02		; #$02 = 16x16
	STA $0420,x		; #$00 = 8x8

SMB3_SKIP_GFX:

	LDA $9D			; check if sprites are locked
	BNE SMB3_RETURN		; return if so
	LDA $169D,y		; check if init should be run
	BNE SMB3_NO_INIT	; branch if not
	LDA #$01		; stop init from running again
	STA $169D,y
	; Init would go here if there was one
SMB3_NO_INIT:

	; speeds

	JSR SUB_SPEEDS		; regular bounce block speed routines

	LDA $16C9,y		; $16C9 = LxxxxxBB
				; L = layer. 0 = layer 1, 1 = layer 2
				; x = unused extra bits
				; BB = speed modifiers
	AND #$03		; get speed modifiers only
	TAX			; stick in x
	LDA $16B1,y		; sprite y speed
	CLC
	ADC BOUNCE_SPEED_Y,x	; add to speed based on table
	STA $16B1,y		; set y speed
	LDA $16B5,y		; sprite x speed
	CLC
	ADC BOUNCE_SPEED_X,x	; add to speed based on table
	STA $16B5,y		; set x speed

	; check if timer's ended

	LDA $16C5,y		; timer
	BNE SMB3_RETURN		; return if not yet 0
				; this timer is automatically decremented every frame until it hits 0
	
	TYX		; stick block index in x
	PHK
	PER $0006	; Replace block with block set in $16C1
	PEA $94F3	; 
	JML $02919F
	TXY		; get it back into y

	LDA #$00
	STA $1699,y	; kill bounce block sprite

SMB3_RETURN:
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Put your own bounce block sprites here


Bounce9:	; SMB3 brick
	; graphics

	JSR SUB_OFFSCREEN	; check if offscreen
				; $00 holds the x position relative to screen border
				; $01 holds the y
				; carry set if offscreen
	BCS SKIP_GFX	; skip graphics if offscreen

	LDA BOUNCE_OAM,y	; get OAM offset
	TAX			; stick in x

	LDA $00			; x pos
	STA $0200,X		; store to OAM
	
	LDA $01			; y pos
	STA $0201,x		; store to OAM

	LDA #$40		; tile number
	STA $0202,x 		; store to OAM

	LDA $1901,y		; properties
	ORA $64			; add in prority bits from level settings
	STA $0203,x		; store to OAM

	TXA			; set tile size
	LSR			; 
	LSR			;
	TAX			;
	LDA #$02		; #$02 = 16x16
	STA $0420,x		; #$00 = 8x8

SKIP_GFX:

	LDA $9D			; check if sprites are locked
	BNE RETURN		; return if so
	LDA $169D,y		; check if init should be run
	BNE NO_INIT	; branch if not
	LDA #$01		; stop init from running again
	STA $169D,y
	; Init would go here if there was one
NO_INIT:

	; speeds

	JSR SUB_SPEEDS		; regular bounce block speed routines

	LDA $16C9,y		; $16C9 = LxxxxxBB
				; L = layer. 0 = layer 1, 1 = layer 2
				; x = unused extra bits
				; BB = speed modifiers
	AND #$03		; get speed modifiers only
	TAX			; stick in x
	LDA $16B1,y		; sprite y speed
	CLC
	ADC BOUNCE_SPEED_Y,x	; add to speed based on table
	STA $16B1,y		; set y speed
	LDA $16B5,y		; sprite x speed
	CLC
	ADC BOUNCE_SPEED_X,x	; add to speed based on table
	STA $16B5,y		; set x speed

	; check if timer's ended

	LDA $16C5,y		; timer
	BNE RETURN		; return if not yet 0
				; this timer is automatically decremented every frame until it hits 0
	
	TYX		; stick block index in x
	PHK
	PER $0006	; Replace block with block set in $16C1
	PEA $94F3	; 
	JML $02919F
	TXY		; get it back into y

	LDA #$00
	STA $1699,y	; kill bounce block sprite

RETURN:
	RTS
BounceA:	; SMB3 brick
	; graphics

	JSR SUB_OFFSCREEN	; check if offscreen
				; $00 holds the x position relative to screen border
				; $01 holds the y
				; carry set if offscreen
	BCS SMB3SKIP_GFX	; skip graphics if offscreen

	LDA BOUNCE_OAM,y	; get OAM offset
	TAX			; stick in x

	LDA $00			; x pos
	STA $0200,X		; store to OAM
	
	LDA $01			; y pos
	STA $0201,x		; store to OAM

	LDA #$8A		; tile number
	STA $0202,x 		; store to OAM

	LDA $1901,y		; properties
	ORA $64			; add in prority bits from level settings
        ORA #$01
	STA $0203,x		; store to OAM

	TXA			; set tile size
	LSR			; 
	LSR			;
	TAX			;
	LDA #$02		; #$02 = 16x16
	STA $0420,x		; #$00 = 8x8

SMB3SKIP_GFX:

	LDA $9D			; check if sprites are locked
	BNE SMB3RETURN		; return if so
	LDA $169D,y		; check if init should be run
	BNE SMB3NO_INIT	; branch if not
	LDA #$01		; stop init from running again
	STA $169D,y
	; Init would go here if there was one
SMB3NO_INIT:

	; speeds

	JSR SUB_SPEEDS		; regular bounce block speed routines

	LDA $16C9,y		; $16C9 = LxxxxxBB
				; L = layer. 0 = layer 1, 1 = layer 2
				; x = unused extra bits
				; BB = speed modifiers
	AND #$03		; get speed modifiers only
	TAX			; stick in x
	LDA $16B1,y		; sprite y speed
	CLC
	ADC BOUNCE_SPEED_Y,x	; add to speed based on table
	STA $16B1,y		; set y speed
	LDA $16B5,y		; sprite x speed
	CLC
	ADC BOUNCE_SPEED_X,x	; add to speed based on table
	STA $16B5,y		; set x speed

	; check if timer's ended

	LDA $16C5,y		; timer
	BNE SMB3RETURN		; return if not yet 0
				; this timer is automatically decremented every frame until it hits 0
	
	TYX		; stick block index in x
	PHK
	PER $0006	; Replace block with block set in $16C1
	PEA $94F3	; 
	JML $02919F
	TXY		; get it back into y

	LDA #$00
	STA $1699,y	; kill bounce block sprite

SMB3RETURN:
	RTS
BounceB:	; SMB3 brick
	; graphics

	JSR SUB_OFFSCREEN	; check if offscreen
				; $00 holds the x position relative to screen border
				; $01 holds the y
				; carry set if offscreen
	BCS SMBSKIP_GFX	; skip graphics if offscreen

	LDA BOUNCE_OAM,y	; get OAM offset
	TAX			; stick in x

	LDA $00			; x pos
	STA $0200,X		; store to OAM
	
	LDA $01			; y pos
	STA $0201,x		; store to OAM

	LDA #$EA		; tile number
	STA $0202,x 		; store to OAM

	LDA $1901,y		; properties
	ORA $64			; add in prority bits from level settings
        ORA #$01
	STA $0203,x		; store to OAM

	TXA			; set tile size
	LSR			; 
	LSR			;
	TAX			;
	LDA #$02		; #$02 = 16x16
	STA $0420,x		; #$00 = 8x8

SMBSKIP_GFX:

	LDA $9D			; check if sprites are locked
	BNE SMBRETURN		; return if so
	LDA $169D,y		; check if init should be run
	BNE SMBNO_INIT	; branch if not
	LDA #$01		; stop init from running again
	STA $169D,y
	; Init would go here if there was one
SMBNO_INIT:

	; speeds

	JSR SUB_SPEEDS		; regular bounce block speed routines

	LDA $16C9,y		; $16C9 = LxxxxxBB
				; L = layer. 0 = layer 1, 1 = layer 2
				; x = unused extra bits
				; BB = speed modifiers
	AND #$03		; get speed modifiers only
	TAX			; stick in x
	LDA $16B1,y		; sprite y speed
	CLC
	ADC BOUNCE_SPEED_Y,x	; add to speed based on table
	STA $16B1,y		; set y speed
	LDA $16B5,y		; sprite x speed
	CLC
	ADC BOUNCE_SPEED_X,x	; add to speed based on table
	STA $16B5,y		; set x speed

	; check if timer's ended

	LDA $16C5,y		; timer
	BNE SMB3RETURN		; return if not yet 0
				; this timer is automatically decremented every frame until it hits 0
	
	TYX		; stick block index in x
	PHK
	PER $0006	; Replace block with block set in $16C1
	PEA $94F3	; 
	JML $02919F
	TXY		; get it back into y

	LDA #$00
	STA $1699,y	; kill bounce block sprite

SMBRETURN:
	RTS
BounceC:
	;RTS
BounceD:
	;RTS
BounceE:
	;RTS
BounceF:
	;RTS
Bounce10:
	;RTS
Bounce11:
	;RTS
Bounce12:
	;RTS
Bounce13:
	;RTS
Bounce14:
	;RTS
Bounce15:
	;RTS
Bounce16:
	;RTS
Bounce17:
	;RTS
Bounce18:
	;RTS
Bounce19:
	;RTS
Bounce1A:
	;RTS
Bounce1B:
	;RTS
Bounce1C:
	;RTS
Bounce1D:
	;RTS
Bounce1E:
	;RTS
Bounce1F:
	RTS


; Various other routines below
; You should never need to modify them

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main hijack routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MAIN:
	LDY $9D			;
	BNE CODE_RESTORE	;
	LDY $16C5,x		;
	BEQ CODE_RESTORE	;
	DEC $16C5,x		; Restore old code
CODE_RESTORE:			;

	CMP #$08		; if bounce sprite number 8 or above...
	BCS CUST_BOUNCE_SPRITES	; do custom bounce block sprite code
	JML $02905E		; otherwise return to normal code

CUST_BOUNCE_SPRITES:
	PHB		;\
	PHK		; |wrapper
	PLB		; |
	PHX		; |
	PHY		;/
	TXY		; stick sprite table offset in y 
	SEC
	SBC #$08	; subtract 8 for jump offset
	ASL 		; x2 for two byte pointers 
	TAX
	JSR (POINTERS,x); jump to appropriate subroutine
	PLY		;\
	PLX		; |wrapper
	PLB		;/
	JML $02919C	; return


;;;;;;;;;;;;;;;;;;;;;;;;;
; Generic speed routine
;;;;;;;;;;;;;;;;;;;;;;;;;

			; this sets the sprite's position based on speed values
SUB_SPEEDS:		; taken from $02B51A & $02B526
	LDA $16B1,y	; sprite y speed
	ASL
	ASL
	ASL
	ASL
	CLC
	ADC $16B9,y	; accumulating fraction bits for y speed
	STA $16B9,y	; set it
	PHP
	LDA $16B1,y	; sprite y speed
	LSR
	LSR
	LSR
	LSR
	CMP #$08
	LDX #$00
	BCC SPEED_1
	ORA #$F0
	DEX
SPEED_1:
	PLP
	ADC $16A1,y	; sprite y (low)
	STA $16A1,y	; set it
	TXA
	ADC $16A9,y	; sprite y (high)
	STA $16A9,y	; set it

	; do it all again for x speeds

	LDA $16B5,y	; sprite x speed
	ASL
	ASL
	ASL
	ASL
	CLC
	ADC $16BD,y	; accumulating fraction bits for x speed
	STA $16BD,y	; set it
	PHP
	LDA $16B5,y	; sprite x speed
	LSR
	LSR
	LSR
	LSR
	CMP #$08
	LDX #$00
	BCC SPEED_2
	ORA #$F0
	DEX
SPEED_2:
	PLP
	ADC $16A5,y	; sprite x (low)
	STA $16A5,y	; set it
	TXA
	ADC $16AD,y	; sprite x (high)
	STA $16AD,y	; set it

	RTS		; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Subroutine that checks if offscreen. Taken from $0291F8
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_OFFSCREEN:
	LDX #$00	; check if on layer 2
	LDA $16C9,y	; highest bit of this number is set if so
	BPL OFFSCREEN_1
	LDX #$04	; all this is to check if offscreen
OFFSCREEN_1:		;
	LDA $16A1,y	; bounce sprite y (low)
	SEC
	SBC $1C,x	; layer 1 y (low). 
	STA $01		; store to scratch
	LDA $16A9,y	; bounce sprite y (high)
	SBC $1D,x	; layer 1 y (high)
	BNE OFFSCREEN_END; offscreen if not 0 
	LDA $16A5,y	; sprite x (low)
	SEC
	SBC $1A,x	; layer 1 x (low)
	STA $00		; store to scratch
	LDA $16AD,y	; sprite x (high)
	SBC $1B,x	; layer 1 x (high)
	BNE OFFSCREEN_END; branch if offscreen
	CLC		; clear carry (on screen)
	RTS

OFFSCREEN_END:
	SEC	; set carry (offscreen)
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Code for custom bounce block map16 changes
;
; Probably not a good idea to touch this
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BLOCK_STUFF:
	SEP #$30		; 8 bit A X Y
				; old code

	LDA $9C			; block number
	CMP #$1C		; 1b is the last part of the normal table
	BCS CUSTOM_BLOCKS	; do normal stuff if below 1c
				; jump to custom block code otherwise
	JML $00BFC0		; return


CUSTOM_BLOCKS:
	PHB		; small wrapper
	PHK
	PLB		
			; If the code reaches here, we have a custom map16 number to turn the block into
	LDA $04,s	; Get x index for custom block map16 number 
	ASL		; x2 for two byte tables
	REP #$30        ; Index (16 bit) Accum (16 bit) 
	AND #$00FF	; clear high byte
	TAX		; stick in x
	LDA $98		; block x       
	AND #$01F0              
	STA $04                   
	LDA $9A		; block y        
	LSR                       
	LSR    		; bunch of code from all.log that I really don't know the exact purpose of
	LSR                       
	LSR                       
	AND #$000F              
	ORA $04                   
	TAY                       
	SEP #$20	; 8 bit A
	LDA !RAM_CUSTBLOCK+1,x	; get high byte of map16 number
	STA [$6E],Y		; set it
	LDA !RAM_CUSTBLOCK,x  	; get low map16 number 
	STA [$6B],Y		; set low byte
	REP #$20		; 16 bit A
	LDA !RAM_CUSTBLOCK,x	; get full map16 number
	ASL A			; more code I'm uncertain the purpose of
	TAY			;
	PLB			; get old bank back
	JML $00C0FB		; finish up with regular SMW code
;;;;;;;;;;;;;;;;;;;;;;;;;
MainEnd:
