;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Bonus Roulette
;;   by leod
;;
;;
;; Causes a little roulette to show up whenever Mario collects 100 coins instead of
;; giving him a life, that will end up applying one of 4 (or more/less if you want) effects
;; when the player presses X (or whatever button you assign it).
;; Might make big 10 coins or even 50 coins worthwhile hidden pickups now!
;;
;; The patch can be applied entirely as it is, however you are free to modify practically
;; anything you want about it, given that you have the ASM knowledge to put your ideas
;; into the slot ASM. If you don't, asking on the forums might help you.
;; (http://www.smwcentral.net/?p=viewforum)
;;
;; The included GFX00/01 is meant to be used in conjunction with the Mario GFX DMAer patch,
;; found here: http://www.smwcentral.net/?p=section&a=details&id=4897
;; It also replaces the flopping cheep-cheep. Sorry! But those things are dumb anyway.
;; The tile using the berry can be turned off below at !Backdrop.
;; If you already use this patch and already filled the freed up graphic slots with
;; something or for some reason don't want to use it, then sorry...
;; You'll have to make room for these elsewhere. The graphic locations are editable
;; at the very bottom of the file, under .BorderTiles.
;;
;;
;; Note: If you don't want the roulette to start at 100 coins, but rather only have it
;;       function manually (eg through some kind of pickup, or special roulette block),
;;       simply delete the two $0180B5 hijacks near the top of the code.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



; = Defines = ;

!Slots = $04
;how many results the roulette has
;must be at least 02 (because having 1 entry in a roulette makes no sense and 0 breaks)
;to adjust every slot entry's display graphics and palettes, scroll to the tables at the bottom
;to adjust what code a slot executes, ctrl-f "= Cases ="

!Button = !X
;which button the player has to press to make the roulette slow down
;and to activate the result's effect afterwards
;possible values: !X, !A, !Y, !B, !L, !R and !Select
;!Start isn't an option because that pauses the game, fix that if you plan on using Start
;when using !X or !Y, I recommend disabling that button's all other functions,
;since the cape spin and fire sfx like to overwrite all of this thing's sfx, which sounds bad

!WaitForPlayer = $D0
;how many frames to wait for player input, until the roulette starts slowing down by itself

!StayIdle = $E0
;how many frames*2 to wait before the powerup disappears after it has settled

!WaitBeforeLeaving = $38
;how many frames to wait after the item has been used before the roulette retreats

!StarTimer = $C0
;how long the star effect lasts (should you not change the effect of the slot of course)
;vanilla starman sets it to FF


!Drumroll = !Yes
;whether or not to play the drumroll while the roulette is slowing down
;set to !No if you don't want this

!Backdrop = !Yes
;whether or not there's an additional tile behind the item
;eats up yet another 16x16 gfx space, set to !No if you don't
;want that and can live with a clear background


!BGTile = $88
;what tile to use, if enabled. berry tile by default

!BGProps = $30
;YXPPCCCT format, http://www.smwiki.net/wiki/YXPPCCCT_format



!BeepSFX = $23
!BeepPort = $1DFC
;which sfx to play in which port, title screen menu select beep by default
;sfx values can be found here: http://smwc.me/96604

!ButtonSFX = $2A
!ButtonPort = $1DF9
;which sfx to play when the player presses a button to make the slots stop rolling
;default is the peach emerging from clown car effect, because it sounds "what am i gonna get?"y
;don't use a sfx on channel 1DFC here if you also use the drumroll, otherwise they will overwrite

!SnapSFX = $29
!SnapPort = $1DFC
;which sfx to play when the rolled item snaps to the middle
;default is the correct! sound effect, sounds kinda celebratory

!StartSFX = $1E
!StartPort = $1DF9
;which sfx to play when the slots pop up
;default is the p-ballon sound, sounds like something appearing

!PoofSFX = $26
!PoofPort = $1DFC
;which sfx to play when the roulette disappears unused
;default is the swooper sound, sounds like something disappearing a little. it fits ok



!XPos = $D8
;leftmost pixel of roulette, change to move the roulette around on the screen horizontally
;the y position is in the "Don't touch these" section below, because moving it will screw up the
;hiding and appearing animations



; = FreeRAM addresses = ;

!Current = $1473
;freeram! default is empty in vanilla
;decided which slot to display right now, also used for the rng

!State = $1415
;freeram! default is empty in vanilla, cleared on overworld and titlescreen load (important)
;which state the roulette is in right now
;00 = not active
;01 = scrolling onscreen while rolling fast
;02 = rolling fast, waiting for timer or button press
;03 = slowing down
;04 = snapping to next target
;05 = standing still, waiting for player to claim
;06 = scrolling offscreen

!YSlots = $0DA1
;freeram! default is empty in vanilla
;the entire roulette's offset from the preferred screen position

!YItem = $0DC3
;freeram! default is empty in vanilla, part of the 4 bytes at 0DC3
;the item's Y offset inside the roulette

!Timer = $0DC4
;freeram! default is empty in vanilla, part of the 4 bytes at 0DC3
;timer used in a few states

!RollSpeed = $0DC5
;freeram! default is empty in vanilla, part of the 4 bytes at 0DC3
;how fast the item rolls (if RollItem is called)

!Flash = $0DC6
;freeram! default is empty in vanilla, part of the 4 bytes at 0DC3
;whether or not to flash the roulette, is a timer

!RNGTop = $0F
;scratch ram, don't edit
;top of the next RNG result+1



; = Don't touch these = ;
; =   Magic Numbers   = ;

!OAM = $0200
;oam table
!Size = !OAM&$1FF/4+$420
;size table

!YPos = $B8
;top pixel of roulette at highest point
!YPosItem = !YPos-$08
;top pixel of item at highest point
!MaxYSlots = $28
;the max value for the Y offset

!FlashTime = $60
;how many frames the flashing lasts

!ItemTop = $00
;offset from current !YSlot that would be the top
!ItemBottom = $20
;when the item is considered at the bottom and should loop back
!ItemCenter = $10
;where the center is for the item, used for the beep sounds and the snapping

!A = $80      ;and on $18
!X = $40      ;and on $18
!L = $20      ;and on $18
!R = $10      ;and on $18
!B = $81      ;and on $16
!Select = $21 ;and on $16
!Y = $0F      ;special code, and 40 on $16 and on $18

!Yes = $01
!No = $00



; = Macros = ;

;finds free oam slots, starting from where x is currently at
;16bit x/y
macro getOAM()
    ?slotLoop:
  CPY #$0200
  BEQ ?foundSlot

  LDA !OAM+$1,y
  CMP #$F0
  BEQ ?foundSlot

  INY : INY : INY : INY
  BRA ?slotLoop
  
    ?foundSlot:
endmacro

;check if the button defined is set
macro checkbutton()
  if !Button == !X || !Button == !L || !Button == !R || !Button == !A

    LDA $18
    AND #!Button

  endif
  if !Button == !B || !Button == !Select

    LDA $16
    AND #!Button-1

  endif
  if !Button == !Y

    LDA $18
    AND #$40
    BEQ +
 
    LDA #$00
    BRA .SkipCheck
  
    +
    LDA $16
    AND #$40

    .SkipCheck

  endif
endmacro

macro jump()
  LDA !YSlots
  SEC
  SBC #$05
  STA !YSlots
endmacro


;;;;;
; call the roulette at 100 coins
;;;;;

org $008F2F         ;kill 1up-when->100-coins handler
NOP #12

org $008F2F         ;replace with own code
autoclean JSL Invoke
BRA $06             ;skip a few NOPs



;;;;;
; run the handler every frame
;;;;;

org $0180B5         ;hijack right after sprites have run
NOP #9

org $0180B5
JSL HandleRoulette
BRA $03



;;;;;
; invoking routine
;;;;;

freecode
  Invoke:
LDA $0DBF
SEC
SBC #$64          ;-100 coins (this code only runs when they're >100)
STA $0DBF

print "Invoke Roulette Routine is at: $",pc
print "JSL to that location in custom blocks or sprites (preserve X) to start the roulette."
print "If one is already running, it'll just flash."
print ""

LDA !State
BEQ .GoStart      ;if no roulette there, do naught
CMP #$06          ;if roulette is just leaving (after claiming), come right back up
BCS .GoStartNoReset
LDA #!FlashTime   ;if roulette still has a prize to claim, just flash to remind
STA !Flash
  .NoFlash
RTL


  .GoStart
LDA #!MaxYSlots
STA !YSlots

  .GoStartNoReset
;play defined starting sound
LDA #!StartSFX
STA !StartPort

LDA #!Slots
STA !RNGTop
JSR RNG
STA !Current        ;store the result into the current item

LDA #$01            ;set roulette state to alive
STA !State
RTL


;;;;;
; the routine that actually handles the roulette
;;;;;

SlotStates:
dw FastRoll,FastRoll,SlowRoll,Snap,WaitForInput,WaitABit,Leave
;01 = scrolling onscreen while rolling fast
;02 = rolling fast, waiting for timer or button press
;03 = slowing down
;04 = snapping to next target
;05 = standing still, waiting for player to claim
;06 = waiting a tad
;07 = scrolling offscreen

GFXOnly:
LDA !Flash
AND #$10
BNE .NoRender

PHB
PHK
PLB
JSR GFX             ;run graphics routine
PLB
  .NoRender
JMP HandleRoulette_Restore


HandleRoulette:
;restore hijack
LDA $18B8
BEQ .NoCluster
JSL $02F808
  .NoCluster

;only run any code if this is invoked
LDA !State          ;if state is 0, no code is actually executed
BNE +
RTL
+

;if froze, dont tick down timers or run behavior
LDA $9D
ORA $1493
BNE GFXOnly

;decrease timer
LDA !Timer
BEQ .NoTimerDec     ;unless it's 0
DEC
STA !Timer
  .NoTimerDec
  
;decrease flash timer
LDA !Flash
BEQ .NoFlashDec     ;unless it's 0
DEC
STA !Flash

AND #$10
BNE .NoRender
  .NoFlashDec

PHB
PHK
PLB
JSR GFX             ;run graphics routine
PLB

  .NoRender
;move roulette to 00
LDA #!YPos
CLC
ADC !YSlots
CMP #!YPos          ;if top height + offset = top height
BEQ .DoneMove
LDA !YSlots
BCS .MoveUp

INC
BEQ .SkipInc
INC
  .SkipInc
STA !YSlots
BRA .DoneMove

  .MoveUp
DEC
BEQ .SkipDec
DEC
  .SkipDec
STA !YSlots

  .DoneMove
LDA !State
DEC
ASL                 ;state machine
TAX

PHB
PHK
PLB
JSR (SlotStates,x)
PLB

  .Restore
RTL



;;;;;;;;;;; ROUTINES USED IN STATES
RNG:
;gets a random number between 0 and !RNGTop
JSL $01ACF9         ;vanilla rng routine

LDA !Current        ;use previous rng result for multiplication
STA $4202
LDA $94 	          ;use player pos for multiplication
STA $4203
NOP #4              ;wait

REP #$20
LDA $4216           ;read the result
CLC
ADC $148D           ;add vanilla rng output for division
STA $4204
SEP #$20

LDA !RNGTop         ;divide by the rng top, making the remainder be between 0 and this
STA $4206
NOP #8              ;wait
LDA $4216           ;use low byte of remainder as rng result
RTS





Score:
;displays the score you give it at mario's position
PHY
PHA
JSL $02AD34    ;get next slot for score sprite

PLA
STA $16E1,y    ;set score amount

LDA $96        ;get mario y lo
SEC
SBC #$08       ;move up a bit
STA $16E7,y    ;set score sprite y lo

LDA $97        ;get mario y high
SBC #$00
STA $16F9,x    ;set score sprite y hi

LDA $94        ;get mario x lo
STA $16ED,y    ;set score sprite x lo
LDA $95        ;get mario x hi
STA $16F3,y    ;set score sprite x hi

LDA #$30
STA $16FF,y    ;set score sprite speed
PLY
RTS




SpinStarFX:
;displays a spinjump star at mario's position
LDY #$07
  .SlotLoop
LDA $170B,y       ;loop for an empty slot
BEQ .FoundSlot
DEY
BPL .SlotLoop
RTS

  .FoundSlot
LDA #$10           ;star sprite number
STA $170B,y 

LDA $94            ;mario x lo
CLC
ADC #$04
STA $171F,y        ;ex sprite x lo
LDA $95            ;mario x hi
ADC #$00
STA $1733,y        ;ex sprite x hi

LDA $96            ;mario y lo
CLC
ADC #$0F
STA $1715,y        ;ex sprite y lo
LDA $97            ;mario y hi
ADC #$00
STA $1729,y        ;ex sprite y hi

LDA .StarX,x
STA $1747,y        ;ex sprite x speed
LDA .StarY,x
STA $173D,Y        ;ex sprite y speed

LDA #$17
STA $176F,y        ;how many frames effect lasts
RTS

  .StarX
db $E8,$18,$E8,$18

  .StarY
db $E8,$E8,$18,$18




RollItem:
;make the item roll
LDA !RollSpeed
STA $0B
BNE .NormalRoll

;if speed is 0, it'll move at 1 pixel every other frame
LDA $14
AND #$01
BNE +
RTS
+

LDA #$01
STA $0B

  .NormalRoll
STZ $0D             ;init 0D
LDA !YItem

;check if eligible for beep (has to be below middle before)
CMP #!ItemCenter-$2
BCC .NoBeep1
INC $0D

  .NoBeep1
CLC
ADC $0B
STA !YItem

;check if eligible for beep (has to be above middle after)
CMP #!ItemCenter-$2
BCC .NoBeep2
LDA $0D
BNE .NoBeep2

;play defined sound
LDA #!BeepSFX
STA !BeepPort

  .NoBeep2
LDA !YItem
CMP #!ItemBottom
BCC .NoWrap

SEC
SBC.b #!ItemBottom
STA !YItem

;if it wraps, change displayed item
LDA !Current
INC
STA !Current
CMP #!Slots
BCC .NoWrap

;if we would overflow slot amount, return to 0
STZ !Current

  .NoWrap
RTS





;;;;;;;;;;; STATES
FastRoll:
;roll pretty fast
LDA #$04
STA !RollSpeed
JSR RollItem

;decide how to enter next state
LDA !State
CMP #$02
BEQ .ButtonOrTime

;move roulette up, and enter next if it hits top
LDA !YSlots
BEQ .next1
RTS


.next1
STZ !YSlots

INC !State

LDA #!WaitForPlayer
STA !Timer
RTS


  .ButtonOrTime
;when button is pressed or timer hits 0, switch to next state
%checkbutton()
BNE .next2

LDA !Timer
BEQ .next3
RTS


.next2
;only jump up if the button is hit
%jump()
;play defined sfx
LDA #!ButtonSFX
STA !ButtonPort

.next3
if !Drumroll = !Yes

  LDA #$11      ;enable drumroll
  STA $1DFC

endif

INC !State

;how long to wait to decrease speed once more
LDA #$09
STA !RNGTop
JSR RNG
CLC
ADC #$11
STA !Timer
RTS



SlowRoll:
JSR RollItem

LDA !Timer
BEQ .nextSpeed
RTS

  .nextSpeed
LDA !RollSpeed
DEC               ;decrease rolling speed
BMI .next         ;if new speed would make it negative, enter snap state
                  ;(speed of 0 means it's half a pixel)
STA !RollSpeed

LDA #$13
STA !RNGTop
JSR RNG
STA $0B
LDA !RollSpeed
TAX
LDA .SpeedTimers,x
CLC
ADC $0B
STA !Timer        ;store new time between 11 and 1F
RTS

  .SpeedTimers
db $4E,$32,$1E,$0E


.next
if !Drumroll = !Yes

  LDA #$12      ;disable drumroll
  STA $1DFC

endif

INC !State
RTS



Snap:
LDA !YItem
CMP #!ItemCenter
BEQ .next          ;check if it's closer to move back or forth
BCS .DecYI

INC
CMP #!ItemCenter
BEQ .StoreNewYI    ;if item is above the middle, move it down
INC
BRA .StoreNewYI

  .DecYI
DEC
CMP #!ItemCenter
BEQ .StoreNewYI    ;if item is below the middle, move it up
DEC

  .StoreNewYI
STA !YItem         ;store new value
RTS

.next
;play defined sound
LDA #!SnapSFX
STA !SnapPort

INC !State
LDA #!StayIdle
STA !Timer

;display stars
LDX #$03
  .StarLoop

JSR SpinStarFX

DEX
BPL .StarLoop
RTS



WaitForInput:
;vaporize after a while
LDA $14
AND #$01
BNE +
INC !Timer      ;slow down timer countdown by half
+

LDA !Timer
BEQ .poof
CMP #$40
BNE .NoFlash

LDA #$80        ;make flash
STA !Flash

  .NoFlash

%checkbutton()
BNE .RunCode
RTS

.poof
LDA #!MaxYSlots ;put roulette back on the bottom fine
STA !YSlots

;play defined sfx
LDA #!PoofSFX
STA !PoofPort

STZ !State
STZ !Timer      ;re-init state and timer to 0
RTS




;this is where the powerups' codes are executed
  .RunCode
%jump()

STZ !Flash
INC !State
LDA #!WaitBeforeLeaving ;wait this long before scrolling offscreen
STA !Timer

LDA !Current
ASL
TAX
JMP (.SlotResults,x)

  .SlotResults
dw Mushroom,Star,FireFlower,Feather
;= Cases =;
;add/remove entries in the list above to add/remove additional slot results
;so if you, say, wanted a fifth entry that runs code to turn on the silver p,
;then you'd just add ",SilverP" or something behind "Feather", or whatever you named
;the existing labels.
;after that, create a label with the same name below (eg SilverP), then put your code
;into it and end it with an RTS, like the other labels already do.
;order doesn't matter.


Star:
;enables star power
LDA #$0A        ;power-up sound effect
STA $1DF9       ;values at http://smwc.me/96604

LDA #$0D        ;set star music
STA $1DFB

LDA #!StarTimer ;length of star power
STA $1490       ;store to star timer
RTS





Mushroom:
;score
LDA #$09
JSR Score

;put current item into box
LDA $19
BEQ .Grow

LDA #$01        ;mushroom is obnoxious and always puts itself into the box unless small
STA $0DC2

LDA #$0B
STA $1DFC       ;play reserve box sound
RTS

  .Grow
;play animation
LDA #$02
STA $71

LDA #$2F
STA $1496       ;animation timer
STA $9D         ;and sprite lock timer

;sound
LDA #$0A
STA $1DF9
RTS







FireFlower:
;score
LDA #$09
JSR Score

;put current item into box
LDA $19
BEQ .NoInventory ;if small, dont mess with reserve item
TAX
LDA .Inventory,x
STA $0DC2
  .NoInventory

LDA $19
CMP #$03
BNE .GiveFire

;if already firey, play reserve box sound
LDA #$0B
STA $1DFC       ;play reserve box sound
RTS


  .Inventory
db $00,$01,$04,$02

  .GiveFire
;play animation
LDA #$20
STA $149B       ;palette flash for mario
STA $9D         ;lock all sprites

LDA #$04
STA $71         ;set the "get powerup" animation
LDA #$03
STA $19         ;store fire powerup

;sound
LDA #$0A
STA $1DF9
RTS






Feather:
;score
LDA #$09
JSR Score

;put current item into box
LDA $19
BEQ .NoInventory ;if small, dont mess with reserve item
TAX
LDA .Inventory,x
STA $0DC2
  .NoInventory

LDA $19
CMP #$02
BNE .GiveCape

;if already cape, play reserve box sound
LDA #$0B
STA $1DFC       ;play reserve box sound
RTS


  .Inventory
db $00,$01,$04,$02

  .GiveCape
;play animation
INC $9D

LDA $81
ORA $7F         ;check if mario is on the screen
BNE .NoAnimation


LDA #$18
STA $1496       ;make him invincible a bit

LDA #$03
STA $71         ;set the "get cape" animation
LDA #$02
STA $19         ;store cape powerup


LDY #$03
  .SmokeLoop
LDA $17C0,y     ;look for empty smoke sprite slot
BEQ .FoundSlot

DEY
BPL .SmokeLoop

LDY #$02        ;we are lazy and will just use any old slot if none is free

  .FoundSlot
LDA #$81
STA $17C0,y     ;cape smoke effect
LDA #$1B
STA $17CC,y     ;this long

LDA $94         ;mario x
STA $17C8,y     ;smoke x
LDA $96         ;mario y
CLC
ADC #$08
STA $17C4,y     ;smoke y


  .NoAnimation
;sound
LDA #$0D
STA $1DF9

;score
LDA #$09
JSR Score
RTS





WaitABit:
;wait until timer runs out
LDA !Timer
BNE .stayhere

INC !State

  .stayhere
RTS




Leave:
LDA !YSlots
CLC
ADC #$03             ;move roulette down by 2 every frame
CMP #!MaxYSlots
BCS .endroulette
STA !YSlots
RTS

  .endroulette
LDA #!MaxYSlots ;put roulette back on the bottom fine
STA !YSlots

STZ !State
STZ !Timer      ;re-init state and timer to 0
RTS





;;;;;;;;;;;;;;;;;;;;;;
;; graphics routine
;;;;;;;;;;;;;;;;;;;;;;

  GFX:
REP #$10
LDY #$0000

LDX #$0003
  .BorderLoop

%getOAM()          ;put free oam slot index into y

LDA #!XPos         ;get roulette screen x position
CLC
ADC .BorderXOffs,x
STA !OAM,y         ;store to x


LDA .BorderYOffs,x
CLC
ADC !YSlots
STA $0D

LDA #!YPos         ;get roulette screen y position
CLC
ADC $0D            ;add loop offset + actual y offset
STA.w !OAM+$1,y    ;store to y

LDA .BorderTiles,x
STA.w !OAM+$2,y    ;store to oam tile

LDA .BorderProps,x
STA.w !OAM+$3,y    ;store to oam properties



REP #$20
TYA
LSR A
LSR A
PHY
TAY
SEP #$20
LDA #$02            ;16x16
STA !Size,y         ;store to size table
PLY

DEX
BPL .BorderLoop

;item tiles
LDA !State
CMP #$06            ;no item is drawn in the scrolling-offscreen state
BCS .NoItem


LDX #$0003
  .ItemLoop

%getOAM()          ;put free oam slot index into y

LDA !YSlots
CLC
ADC #!YPos
STA $0D             ;if tile is above this, hide it
CLC
ADC #$18            ;add border and 10
STA $0C             ;if tile is below this, hide it

LDA #!YPosItem      ;get roulette screen position
CLC
ADC !YItem
CLC
ADC !YSlots
CLC
ADC .ItemYOffs,x
CMP $0C
BCS .NoTile
CMP $0D
BCS .ShowTile
  .NoTile
LDA #$F0
  .ShowTile
STA.w !OAM+$1,y     ;store to y

LDA.b #!XPos+$08    ;store absolute screen pos to oam x
CLC
ADC .ItemXOffs,x
STA !OAM,y          ;store to x

PHX
LDX !Current
LDA .ItemPalettes,x
STA.w !OAM+$3,y     ;store to oam properties

LDA .ItemTiles,x
PLX
CLC
ADC .ItemTilesOffs,x
STA.w !OAM+$2,y     ;store to oam tile

REP #$20
TYA
LSR A
LSR A
PHY
TAY
SEP #$20
LDA #$00            ;8x8
STA !Size,y         ;store to size table
PLY

DEX
BPL .ItemLoop
  .NoItem


;background tile
if !Backdrop == !Yes

  %getOAM()          ;put free oam slot index into y

  LDA.b #!XPos+$08    ;get roulette screen x position + 8
  STA !OAM,y          ;store to x

  LDA.b #!YPos+$08    ;get roulette screen y position + 8
  CLC
  ADC !YSlots         ;and add slot offset
  STA.w !OAM+$1,y     ;store to y

  LDA #!BGTile
  STA.w !OAM+$2,y     ;store to oam tile

  LDA #!BGProps
  STA.w !OAM+$3,y     ;store to oam properties
  
  REP #$20
  TYA
  LSR A
  LSR A
  PHY
  TAY
  SEP #$20
  LDA #$02            ;16x16
  STA !Size,y         ;store to size table
  PLY

endif

SEP #$10
RTS


  .ItemTiles
db $24,$48,$26,$0E
;the top left 8x8 for your slot's 16x16
;default: mushroom, star, fire flower, feather
;note: these do not have to be powerups, with the right code you could theoretically
;make a slot spawn a shell for mario or kill all enemies currently alive, akin to a POW,
;or activate the blue/silver P timer etc.

  .ItemPalettes
db $0C-$08<<1|$30,$0A-$08<<1|$30,$0D-$08<<1|$30,$0A-$08<<1|$30
;stored in the YXPPCCCT format
;more info here: http://www.smwiki.net/wiki/YXPPCCCT_format
;the default values above are stored in this format:
;  $xx-$08<<1|$30|$yy
;just copy and paste that over one of the table's slots
;(or add more for more slots), then fill in $xx and $yy, where:
;$xx is the sprite's palette (eg palette 09 would be $09-$08<<1|$30|$yy)
;$yy is 00 if the tile is meant to use SP1/2 and 01 if it is meant to use SP3/4



  .BorderTiles
db $0A,$20,$CE,$22
;top left, top right, bottom left, bottom right
;tiles used for the border
;you can totally use a single tile for all of these and just flip it
;using the props below, just add 40 to x flip it, 80 to y flip it and C0 for both

  .BorderProps
db $30,$30,$30,$30
;top left, top right, bottom left, bottom right
;YXPPCCCT format, http://www.smwiki.net/wiki/YXPPCCCT_format

  .BorderXOffs
db $00,$10,$00,$10
  .BorderYOffs
db $00,$00,$10,$10
;dont edit

  .ItemXOffs
db $00,$08,$00,$08
  .ItemYOffs
db $00,$00,$08,$08
  .ItemTilesOffs
db $00,$01,$10,$11
;dont edit either