

; ###############################################
; #                                             #
; #       16-BIT TIMER (xkas version)           #
; #    made in 2011-2012 by WhiteYoshiEgg       #
; #                                             #
; #  refer to readme.txt for more information.  #
; #                                             #
; ###############################################


header
lorom







; ###############
; # Definitions #
; ###############


	; Note  that  long  RAM  addresses (24 bits / 6 digits  long)
	; are not supported by this patch!
	; In case  you absolutely need to change  the addresses here,
	; use 2- or 4-digit addresses instead.



	!FreeSpace	= $1D8000
				; At   least   799   bytes  of   free 
				; ROM space.
				; Make sure NOT to rely on this value
				; without checking  if  it really  is
				; free space first!
				; Use  a tool like  Free Space Logger
				; ("slogger") or "FreezePace" to find
				; free ROM space.


	!TimerRAM	= $0F31	; The   address   to  use   for   the
				; timer itself (2 consecutive bytes).
				; Chancing  this   is   not   needed,
				; since $0F31-2 are freed up anyway.

	!TimerSpeed	= $28	; How  fast  the  timer  counts  down
				; (how  many frames it takes for  the
				; timer to decrement by one).
				; The  default  value  is  $28 -- one
				; real-life  second  would  be  ~$60.

	!TimerSpeedRAM	= $0F30	; The  address  to  store  the  above
				; value in.
				; Again, it's not necessary to change
				; this,  since $0F30 was  always used
				; for this very purpose anyway.

	!SpeedUpTime	= 99	; At which value the "Time is running
				; out" sound  effect  starts  playing
				; and the music speeds up.
				; (Yes, it's decimal, no $ needed.)

	!BerryTime	= 55	; How many seconds should be added to
				; the   timer  when   Yoshi  eats   a
				; green berry.
				; (Again,  it's  decimal.   It's  not
				; limited to multiples of 10 though -
				; everything  between   0  and  65535
				; is fine to use.)
					
	!ChangeSB	= 0	; Whether or not to  change the tiles
				; in the status bar to make  room for
				; the new timer.
				; It'll  probably only look right  if
				; you  haven't  severely changed  the
				; status bar with other tools yet, so
				; unless  you're  still  using  SMW's
				; original  status  bar, it might  be
				; wise to turn this off.
				; -1  =  don't,              0  =  do
				; (note:  the  Asar version  requires
				; different values).

	!TimerPos	= $0F0A	; The  position of  the timer in  the
				; status  bar (of the  10000s  digit,
				; to be exact - 5 consecutive bytes).
				; If you choose  to change the status
				; bar (see above),  the default value
				; ($0F0A) works best here.

	!OmitZeroes	= 0	; Whether  or  not  to  omit  leading
				; zeroes in the status bar.
				; -1  =  don't,              0  =  do
				; (note:  the  Asar version  requires
				; different values).

	!Align		= left	; If the  above value is 1,  this one
				; determines  if the number should be
				; aligned to the left or to the right
				; in the status bar  when it has less
				; than 5 digits.
				; left = left,  right = right  (duh).
				; No ! or anything  required -- don't
				; use  anything  other than  left  or
				; right though.

	!FreeRAM	= $0F33	; The  address to use  as a new  flag
				; that indicates  whether or not  the
				; level starts with a time limit of 0
				; (needed  for  fixing the  "Time Up"
				; message).
				; Once  again,  there's  no  need  to
				; change  this, since $0F33  is freed
				; up by this patch anyway.





; ###########
; # Hijacks #
; ###########


org $008D8A
	LDA #!TimerSpeed	; \  Apply new
	STA !TimerSpeedRAM	; /  timer speed


org $0091BD
	REP #$20		; \  
	STZ !TimerRAM		;  | Change the "reset timer" code
	SEP #$20		;  | (before bonus game?)
	BRA $00			; /  


org $009A91			; \  Change the "reset timer" code (title screen)
	BRA $01 : NOP		;  | (disable the original STZ $0F31,
org $009A97			;  | then hijack a more convenient place 
	JML ResetTimer		; /  with no JSRs around it)


org $05DAE5			; \  Disable level 24's
	db $FF			; /  special behavior


org $058583			;    Change the level loading routine (?)
	JML LoadTimer


org $00D0E8			;    Change the death message routine (?)
	JML TimerFix

	
org $008E1A			;    Change the main timer routine
	JML CustomTimer


org $0DF160			;    Change LM's "Bypass the time limit" code
	JML LMBypass


org $01F102			;    Change the "green berry eaten" routine
	JML GreenBerry


org $05CE4C			;    Change the "Course Clear" Scorecard routine
	JML Scorecard


org $05CCA0			;    Change the scorecard tilemap
	JML ScorecardTilemap


org $05CC93			;    New scorecard tilemap size
	LDY.w #$004A


org $05CCC8			;    Don't write the time to the scorecard twice
	JML $05CCDF


org $05CC66			;    Change the "give 1Up on level end" routine
	JML Goal1Up



; ######################
; # Status Bar Hijacks #
; ######################


macro sbhij()

org $008CA9
	db $76,$38,$FC,$3C	; \
	db $FC,$3C,$FC,$3C	;  | Move the timer to the top line of the status bar
	db $FC,$3C,$FC,$3C	;  | and put a clock symbol next to it
	db $FC,$3C,$2E,$3C	;  |
	db $26,$38		; /
org $008CDF
	db $FC,$38,$FC,$38	; \  Erase the tiles on the bottom line
	db $FC,$38,$FC,$38	; /  (where the timer used to be)
	db $FC,$38

endmacro

	rep !ChangeSB : %sbhij();    Run the above only if the define is zero






org !FreeSpace

; ##################################
; # Reset the timer (title screen) #
; ##################################

ResetTimer:

	PHP : REP #$20		; \
	STZ !TimerRAM		;  | Reset the timer, you know the deal
	PLP			; /
	
	LDA #$03 : STA $12	;    Hijacked code
	JML $009A9B		;    Return





; #################################################
; # Initialize the timer (level loading routine?) #
; #################################################


LoadTimer:
	
	PHX
	STZ !FreeRAM		;    Clear "initial time is zero" flag
		
	LDA $13BF		; \
	ASL			;  |
	TAX			;  | Get timer value for this level
	PHP : REP #$20		;  | and store it to the timer address
	LDA.l TimerTable,x	;  |
	STA !TimerRAM		; /

	CMP.w #$0000		; \
	BNE .notZero		;  | If the time limit for this level is 0,
	SEP #$20		;  | set "initial time is zero" flag
	INC !FreeRAM		; /  (= level has no time limit)

.notZero
	PLP
	PLX
	JML $058590		;    Return




; ############################
; # Handle "Time up" message #
; ############################


TimerFix:

	REP #$20
	LDA !TimerRAM		; \  If the timer isn't 0 when dying,
	BNE .noTimeUp		; /  don't show "Time up" message
	SEP #$20

	LDA !FreeRAM		; \  If the timer was 0 all along,
	BNE .noTimeUp		; /  don't show it either

	JML $00D0F3		;    Otherwise, set "Time up" message

.noTimeUp
	SEP #$20
	JML $00D104		;    Skip "Time up" message




; ########################
; # Custom timer routine #
; ########################
	

CustomTimer:

	LDA $1493		; \  If the level is ending
	ORA $9D			;  | (or sprites/etc. are locked),
	BNE .noDecrement	; /  don't decrement the timer

	LDA $0D9B		; \  Don't decrement during
	CMP #$C1		;  | the Bowser battle either
	BEQ .noDecrement	; /

	DEC !TimerSpeedRAM	; \  And when the timer's timer isn't 0,
	LDA !TimerSpeedRAM	;  | don't do it either
	BNE .noDecrement	; /

	LDA #!TimerSpeed	; \  If it is 0, however,
	STA !TimerSpeedRAM	; /  reset it

	REP #$20		; \  If the timer itself
	LDA !TimerRAM		;  | is already at 0,
	BEQ .noDecrement	; /  don't decrement either

.decrement
	DEC !TimerRAM		;    Otherwise, (finally) decrement the timer

	LDA !TimerRAM		; \ 
	CMP.w #!SpeedUpTime	;  | 
	BNE .dontSpeedUp	;  | If the time is low enough,
	SEP #$20		;  | play "Time is running out" SFX
	LDA #$FF		;  | and speed up the music
	STA $1DF9		;  |
	REP #$20		; /

.dontSpeedUp
	LDA !TimerRAM		; \
	BNE .dontKill		;  | If the timer is 0
	SEP #$20		;  | (after decrementing),
	JSL $00F606		; /  kill Mario

.dontKill
.noDecrement

	REP #$20		; \
	LDA !TimerRAM		;  | Convert the timer to decimal
				;  | and draw it to the status bar
	JSR Hex2Dec16Bit	; /





; ###################
; # Alignment stuff #
; ###################


macro align()			;    Macro: choose which alignment to use
	%align!Align(x)		;    (this is executed later, and only if !OmitZeroes is zero)
endmacro

macro alignleft(x)		;    Macro: align the number to the left
	PHX			; \
	LDX #$00		;  |
.loop				;  |
	LDA !TimerPos		;  | If the leftmost tile is zero,
	BNE .end		;  | erase it and move the other tiles
	LDA !TimerPos+1		;  | to the left by one
	STA !TimerPos		;  |
	LDA #$FC		;  | (repeat this three times)
	STA !TimerPos+1		;  |
	CPX #$03		;  |
	BEQ .end		;  |
	LDA !TimerPos+2		;  |
	STA !TimerPos+1		;  |
	LDA #$FC		;  |
	STA !TimerPos+2		;  |
	CPX #$02		;  |
	BEQ .end		;  |
	LDA !TimerPos+3		;  |
	STA !TimerPos+2		;  |
	LDA #$FC		;  |
	STA !TimerPos+3		;  |
	CPX #$01		;  |
	BEQ .end		;  |
	LDA !TimerPos+4		;  |
	STA !TimerPos+3		;  |
	LDA #$FC		;  |
	STA !TimerPos+4		;  |
.end				;  |
	INX			;  |
	CPX #$04		;  |
	BNE .loop		;  |
	PLX			; /
endmacro

macro alignright(x)		;    Macro: align the number to the right
	LDA !TimerPos		; \
	BNE .end		;  |
	LDA #$FC		;  |
	STA !TimerPos		;  | Erase a tile if it's zero
	LDA !TimerPos+1		;  | and continue with the tile to its right
	BNE .end		;  | (if it's non-zero, break)
	LDA #$FC		;  |
	STA !TimerPos+1		;  |
	LDA !TimerPos+2		;  |
	BNE .end		;  |
	LDA #$FC		;  |
	STA !TimerPos+2		;  |
	LDA !TimerPos+3		;  |
	BNE .end		;  |
	LDA #$FC		;  |
	STA !TimerPos+3		;  |
.end				; /
endmacro

rep !OmitZeroes : %align()	;    Run the "align" macro if desired


	JML $008E95		;    Return







; ##########################################
; # Custom "Bypass the time limit" routine #
; ##########################################

	; Technical  info  I feel like  sharing:

	; LM stores bypassed time limit settings
	; in  $57 and $59,  with the highest bit
	; of $59  serving  as  the  "force  time 
	; limit to reset on  entering the level"
	; flag.
	; Example:  Entering  12F in  LM  stores
	; #$2F to $57 and #$01 to $59.
	; (If the aforementioned  flag was  set,
	; the latter would be #$81 instead.)


LMBypass:

	PHP : SEP #$20

	LDA $59			; \  If the "force time limit to reset"
	AND #$80		;  | flag is set, go to the .forceReset label
	BNE .forceReset		; /  no matter what $141A says

	LDA $141A		; \  If it isn't set, and you didn't come here
	BNE .end		; /  directly from the OW, return

.forceReset
	LDA $59			; \  
	AND #$0F		;  |
	XBA			;  | Store the custom timer value
	LDA $57			;  | to the timer RAM
	REP #$20		;  |
	STA !TimerRAM		; /

.end
	PLP
	JML $0DF182		;    Return






; ######################################
; # Custom "Green berry eaten" routine #
; ######################################

GreenBerry:

	PHP : REP #$20		; \
	LDA !TimerRAM		;  | Add the specified amount
	CLC : ADC.w #!BerryTime	;  | of seconds to the timer
	BCC .noOverflow		;  | (if that results in the time being greater
	LDA #$FFFF		;  | than FFFF though - hey, that could happen -
				;  | set it to FFFF instead)
.noOverflow			;  |
	STA !TimerRAM		; /

	PLP
	JML $01F12D		;    Return
	
	


; ###############################################
; # Custom "Couse Clear Time Countdown" routine #
; ###############################################


	; I    don't    think   the    multiplication   registers
	; support    results   greater   than   FFFF,  and  since 
	; those  are  possible   with   the   new  timer  system,
	; I'm afraid we'll have to do without multiplication.
	; What we'll do instead  is  add  the  timer value itself
	; to the score - unmultiplied.


Scorecard:

	REP #$20
	LDA !TimerRAM
	PHA

.loop
	STA $4204		; \
	LDY #$0A		;  | Make the number a multiple of 10
	STY $4206		;  | so the countdown doesn't screw up   
	JSR .waste24Cycles	;  |
	LDA $4216		;  | (add one and divide by ten
	BEQ .isMultipleOf10	;  | until the remainder is zero)
	PLA : INC : PHA		;  |
	BRA .loop		; /

.isMultipleOf10
	PLA			; \
	STA $02			;  | Store to $02 and let SMW handle the rest
	SEP #$20		; /
	
	JML $05CEA2		;    Return


.waste24Cycles			; \  Division takes time,
	JSR +			;  | so we need a piece of code
+	RTS			; /  to waste it






; ###############################################
; # Custom "Course Clear Time Coundown" tilemap #
; ###############################################

ScorecardTilemap:

	PHX
	TYX
	LDA.l .newTilemap,x
	PLX
	STA.l $7F837D,x

	JML $05CCA4		;    Return


.newTilemap

	; "COURSE CLEAR!"
	db $51,$0D,$00,$09 ; header
	db $30,$28,$31,$28,$32,$28,$33,$28
	db $34,$28,$51,$49,$00,$19,$0C,$38
	db $18,$38,$1E,$38,$1B,$38,$1C,$38
	db $0E,$38,$FC,$38,$0C,$38,$15,$38
	db $0E,$38,$0A,$38,$1B,$38,$28,$38
	
	; "o LEFT:" (o = clock symbol)
	db $51,$A9,$00,$19 ; header
	db $76,$38,$FC,$38,$15,$38,$0E,$38
	db $0F,$38,$1D,$38,$78,$38,$FC,$38
	db $FC,$38,$FC,$38,$FC,$38,$FC,$38
	db $FC,$38
	db $FF




; ##########################################
; # Custom "Give 1Up on level end" routine #
; ##########################################

	; In the original SMW, you would receive a 1Up
	; at the goal if the  tens digit of your bonus
	; stars  and  the tens and ones  digit of  the
	; time left were the same.
	; (I don't think too many have noticed that)


Goal1Up:


	LDY #$00		; \
	LDX $0DB3		;  | Load the number of bonus stars to get
	LDA $0F48,x		; /
.loop
	CMP #$0A		; \
	BCC .break		;  | Convert to decimal,
	SBC #$0A		;  | Y now holds the tens digit
	INY			;  |
	BRA .loop		; /
.break

	LDA #$FC		; \
	CMP !TimerPos+4		;  |
	BNE .cmp1		;  | Get the position of the ones digit
	CMP !TimerPos+3		;  | and branch to the corresponding code
	BNE .cmp2		;  | 
	CMP !TimerPos+2		;  | (return if there's only
	BNE .cmp3		;  | one digit left)
	CMP !TimerPos+1		;  |
	BNE .cmp4		; /
	BRA .return

.cmp1				;    If the ones digit is in the rightmost position:
	CPY !TimerPos+4		; \
	BNE .return		;  |
	CPY !TimerPos+3		;  | Compare the two last digits with Y
	BNE .return		;  | and give a 1Up if they're equal
	INC $18E4		;  |
	BRA .return		; /

.cmp2				;    If the ones digit is in the second-to-rightmost position:
	CPY !TimerPos+3		; \
	BNE .return		;  |
	CPY !TimerPos+2		;  | (see above)
	BNE .return		;  |
	INC $18E4		;  |
	BRA .return		; /

.cmp3				;    If the ones digit is in the third-to-rightmost position:
	CPY !TimerPos+2		; \
	BNE .return		;  |
	CPY !TimerPos+1		;  | (see above)
	BNE .return		;  |
	INC $18E4		;  |
	BRA .return		; /

.cmp4				;    If the ones digit is in the second-to-leftmost position:
	CPY !TimerPos+1		; \
	BNE .return		;  |
	CPY !TimerPos		;  | (see above)
	BNE .return		;  |
	INC $18E4		; /

.return
	JML $05CC84		;    Return







; #############################
; # 16-bit Hex -> Dec routine #
; #############################

Hex2Dec16Bit:

	SEP #$20
	STZ !TimerPos		; \  Reset the output RAM
	STZ !TimerPos+1		;  | (each address corresponds to one decimal digit) -
	STZ !TimerPos+2		;  | that's a usable format already,
	STZ !TimerPos+3		;  | so we can use the status bar RAM directly
	STZ !TimerPos+4		; /  instead of having to use free RAM
	REP #$20

.loop1
	CMP.w #10000		;    subtract 10000 until the value is less than 10000,
	BCC .loop2		;    and increase the output RAM address with each subtraction
	SBC.w #10000
	SEP #$20 : INC !TimerPos : REP #$20
	BRA .loop1

.loop2
	CMP.w #1000		;    same for the other loops,
	BCC .loop3		;    execpt with 1000, 100 and 10
	SBC.w #1000
	SEP #$20 : INC !TimerPos+1 : REP #$20
	BRA .loop2

.loop3
	CMP.w #100
	BCC .loop4
	SBC.w #100
	SEP #$20 : INC !TimerPos+2 : REP #$20
	BRA .loop3

.loop4
	CMP.w #10
	BCC .loopReturn
	SBC.w #10
	SEP #$20 : INC !TimerPos+3 : REP #$20
	BRA .loop4

.loopReturn
	SEP #$20
	STA !TimerPos+4

	RTS







; ###########################################
; # Table for initial (non-bypassed) timers #
; ###########################################


	; This table contains the initial timer values for every overworld level ("translevel"),
	; each  number corresponding to one level.  (It's  in decimal - $ signs aren't  required
	; unless you want to use Hex for some reason, in which case you'll need them.)
	; They don't all have to be 5 digits either - feel free to omit leading zeroes - it just
	; looks nicer that way.
	; Note  that these values won't have  any effect on a level if you bypass the time limit
	; for them in Lunar Magic. (Read the Readme for more information.)



TimerTable:

	; Levels 000 - 00F
	dw 00010,00300,00300,00000,00300,00300,00300,00400,00200,00400,00300,00300,00300,00300,00300,00300

	; Levels 010 - 01F
	dw 00300,00300,00200,00400,00200,00400,00400,00400,00400,00200,00300,00300,00300,00300,00200,00300

	; Levels 020 - 024
	dw 00300,00300,00300,00300,00300

	; Levels 101 - 10B
	dw 00300,00300,00300,00000,12345,00400,00400,00200,00300,00400,00300,00400,00400,00300

	; Levels 10C - 11B
	dw 00200,00400,00400,00300,00300,00300,00200,00300,00300,00400,00400,00300,00300,00300,00400,00200

	; Levels 11C - 12B
	dw 00400,00400,00300,00300,00300,00200,00300,00300,00200,00200,00300,00300,00300,00200,00300,00300

	; Levels 12C - 13B
	dw 00300,00300,00200,00200,00300,00200,00200,00200,00300,00300,00300,00200,00200,00200,00200,00200

