;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Custom Score Sprites (advanced version), by imamelia
;;
;; This patch enables you to insert custom score sprites into your ROM (the little
;; point notations that appear when you get mushrooms and stuff).  SMW uses
;; score sprites 01-0F, 10-15 are unused score sprites that have glitched graphics,
;; and 16-FF are completely unused slots that can be used for custom score sprites.
;; You can even use $02ACEF, SMW's built-in score sprite spawning routine, to spawn
;; the custom score sprites.  This version of the patch does not use the original code
;; ($02ADC9) at all; some of it is rewritten, and all of it is handled by the patch.  This
;; patch:
;; 	1) enables you to use score sprite numbers 16-FF for custom code;
;;	2) makes it possible to use score sprites 10-15 (5-Up and the coin ones)
;;		without glitched graphics;
;;	3) enables you to define new "types" for score sprites (what they give you, such
;;		as points and lives);
;;	4) lets score sprites use either one, two, or three tiles instead of always two;
;;	5) enables you to set the palette, GFX page, and tilemap of any score sprite;
;;	6) has an optional feature to disable all of the score sprites that give you points
;;		only (useful for users of the 6-digit coin counter, for example).
;; There is an example code for score sprite 16 that gives Mario 10 bonus stars and uses
;; 3 tiles.
;;
;; Score sprites:
;;
;; 00 - empty slot
;; 01 - 10 pts.
;; 02 - 20 pts.
;; 03 - 40 pts.
;; 04 - 80 pts.
;; 05 - 100 pts.
;; 06 - 200 pts.
;; 07 - 400 pts.
;; 08 - 800 pts.
;; 09 - 1000 pts.
;; 0A - 2000 pts.
;; 0B - 4000 pts.
;; 0C - 8000 pts.
;; 0D - 1-Up
;; 0E - 2-Up
;; 0F - 3-Up
;; 10 - 5-Up (unused and glitched)
;; 11 - 5 coins (unused and glitched)
;; 12 - 10 coins (unused and glitched)
;; 13 - 15 coins (unused and glitched)
;; 14 - 20 coins (unused and glitched)
;; 15 - 25 coins (unused and glitched)
;; 16-FF - Unused, as far as I know
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

!Freespace = $188000

org $02ADC9
JML ScoreSpriteHack

org !Freespace
db "STAR"
dw EndCode-StartCode-$01
dw EndCode-StartCode-$01^$FFFF

StartCode:

TilesUsed:			; how many tiles each score sprite uses (should be 00, 01, or 02 for one, two, or three tiles)
db $00,$00,$00,$00,$00,$01,$01,$01
db $01,$01,$01,$01,$01,$01,$01,$01
db $01,$01,$01,$01,$01,$01
; Custom score sprites start here at score sprite 16.
db $02,$01
db $01,$01,$01,$01,$01,$01,$01,$01

ScoreSprTile1:		; the left tile of a score sprite (or the only one, if it uses only one tile)
db $00,$44,$54,$46,$47,$44,$54,$46
db $47,$44,$54,$46,$47,$56,$29,$39
db $38,$5E,$5E,$5E,$5E,$5E
; Custom score sprites start here at score sprite 16.
db $5C,$FF
db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF

ScoreSprTile2:		; the right tile of a score sprite (or the middle tile, if it uses three tiles)
db $FF,$FF,$FF,$FF,$FF,$45,$45,$45
db $45,$55,$55,$55,$55,$57,$57,$57
db $57,$4E,$44,$4F,$54,$5D
; Custom score sprites start here at score sprite 16.
db $6C,$FF
db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF

ScoreSprTile3:		; the right tile of a score sprite, if it uses three tiles
db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
db $FF,$FF,$FF,$FF,$FF,$FF
; Custom score sprites start here at score sprite 16.
db $6D,$FF
db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF

ScoreSpritePalette:		; the palette and GFX page of each score sprite
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$08,$04,$06
db $0A,$04,$04,$04,$04,$04
; Custom score sprites start here at score sprite 16.
db $02,$00
db $00,$00,$00,$00,$00,$00,$00,$00

PointsToGive:		; how many points each score sprite gives, divided by 0A
dw $0000,$0001,$0002,$0004,$0008,$000A,$0014,$0028
dw $0050,$0064,$00C8,$0190,$0320,$0000,$0000,$0000
dw $0000,$0000,$0000,$0000,$0000,$0000

LivesToGive:		; how many lives each score sprite gives
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$01,$02,$03
db $05,$00,$00,$00,$00,$00

CoinsToGive:		; how many coins each score sprite gives
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$05,$0A,$0F,$14,$19

ScoreSpriteType:
; This indicates what each score sprite gives you.
; 00 - nothing
; 01 - points only
; 02 - lives only
; 03 - points and lives
; 04 - coins only
; 05 - coins and points
; 06 - coins and lives
; 07 - coins, points, and lives
; 08-FF - use custom code (preferable for custom score sprites, even ones that give you points, lives, or coins)
db $00,$01,$01,$01,$01,$01,$01,$01
db $01,$01,$01,$01,$01,$02,$02,$02
db $02,$05,$05,$05,$05,$05
; Custom score sprites start here at score sprite 16.
db $08,$00
db $00,$00,$00,$00,$00,$00,$00,$00

InitialOffsets:		;
db $08,$00,$F8		;

ScoreSprYSpeed:
db $03,$01,$00,$00

CustomPtrs:		; pointers for the custom score sprite types (NOT score sprite numbers!)
dw ScoreSpriteType08	; type 08
dw ScoreSpriteType09	; type 09
dw ScoreSpriteType0A	; type 0A
dw ScoreSpriteType0B	; type 0B
;dw ScoreSpriteType0C	; add as many extra pointers as necessary
;...
;dw ScoreSpriteTypeFF	; type FF

ScoreSpriteReturn:	;
PLB				;
JML $02ADC8		;

ScoreSpriteHack:

PHB				;
PHK				;
PLB				;

LDY $16E1,x		; all tables are indexed by the score sprite number
;CPY #$0D			; uncomment these lines if you wish
;BCC .EraseSprite		; to disable all score sprites below 1-Up
STY $07			; we'll be using this a lot, so put it into scratch RAM
LDA.w ScoreSpriteType,y	;
STA $08			; $08 will hold the score sprite type
CMP #$08			; if this is 08 or greater...
BCS .CustomScoreSprite	; then the score sprite is a custom one

LDA $9D			; if sprites are locked...
BEQ .NotLocked		; jump to the GFX routine
JSR ScoreSpriteGFX		;
BRA ScoreSpriteReturn	;

.NotLocked			;

LDA $16FF,x		;
BNE .NoErase		; if the score sprite speed is 0...
.EraseSprite			;
STZ $16E1,x		; erase the sprite
BRA ScoreSpriteReturn	;

.CustomScoreSprite			;
SEC						;
SBC #$08					;
REP #$30					;
AND.w #$00FF				;
ASL						;
TAY						;
LDA CustomPtrs,y				;
STA $00					;
SEP #$30					;
PEA.w ScoreSpriteReturn-$01	;
JMP ($0000)				;

.NoErase			;

DEC $16FF,x		;
CMP #$2A			; unless the sprite Y speed is exactly 2A...
BNE .NoBonus		; skip the part of code that gives the player points, lives, etc.

LDA $08			;
AND #$01			; if bit 0 is set, give the player points
BEQ .NoPoints		; (yes, I know I could use LSR : BCC here, but it saves only 1 byte and no cycles)

REP #$30			;
TYA				;
ASL				;
TAY				;
LDA $0DB3			;
ASL				;
CLC				;
ADC $0DB3			; player * 3
AND.w #$00FF		;
TAX				;
LDA $0F34,x		;
CLC				;
ADC.w PointsToGive,y	; first two bytes of the points to give
STA $0F34,x		;
SEP #$30			;
LDA $0F36,x		;
ADC #$00			; third byte
STA $0F36,x		;
LDX $15E9			;
LDY $07			;

.NoPoints			;
LDA $08			;
AND #$02			; if bit 1 is set, give the player lives
BEQ .NoLives		;

LDA.w LivesToGive,y		;
CLC					;
ADC $18E4				; set the number of lives to give ($18E4)
STA $18E4				;
STZ $18E5				;

.NoLives				;

LDA $08				;
AND #$04				; if bit 2 is set, give the player coins
BEQ .NoCoins			;

PHX					;
PHY					;
LDA.w CoinsToGive,y		; set the number of coins to give
JSL $05B329				; call the coin-giving subroutine
PLY					;
PLX					;

.NoCoins				;
LDX $15E9				;

.NoBonus				;

JSR ScoreSpriteSpeed		;
JSR ScoreSpriteGFX			;
JMP ScoreSpriteReturn		;

ScoreSpriteSpeed:			;

LDA $16FF,x			;
LSR #4				;
TAY					;
LDA $13				;
AND.w ScoreSprYSpeed,y	; the score sprite moves at different Y speeds depending on how long it has been on the screen
BNE .EndSpeed			;
LDA $16E7,x			;
TAY					;
SEC					;
SBC $1C				;
CMP #$04				; unless the score sprite is about to go offscreen...
BCC .EndSpeed			;
DEC $16E7,x			; move it upward 1 pixel
TYA					;
BNE .EndSpeed			;
DEC $16F9,x			; if necessary, decrement the high byte of the sprite's Y position
.EndSpeed				;
RTS					;

ScoreSpriteGFX:

LDA $1705,x			;
ASL #2				;
TAY					;
REP #$20				;
LDA $001C,y			;
STA $02				;
LDA $001A,y			;
STA $04				;
SEP #$20				;
LDA $16ED,x			;
CLC					;
ADC #$0C				;
PHP					;
SEC					;
SBC $04				;
LDA $16F3,x			;
SBC $05				;
PLP					;
ADC #$00				;
BNE .Return2			;

LDA $16ED,x			;
CMP $04				;
LDA $16F3,x			;
SBC $05				;
BNE .Return2			;
LDA $16E7,x			;
CMP $02				;
LDA $16F9,x			;
SBC $03				;
BEQ .Continue			;

.Return2				;
JMP .EndGFX			;

.Continue				;

JSR GetOAMIndex			; get an OAM index for the sprite
CPY #$FF				; if there are none free...
BEQ .EndGFX			; then don't draw the sprite at all

LDA $16ED,x			;
SEC					;
SBC $04				;
PHY					;
PHA					;
LDY $07				;
LDA.w TilesUsed,y			;
TAY					;
PLA					;
CLC					;
ADC InitialOffsets,y		;
PLY					;
STA $0200,y			;
CLC					;
ADC #$08				;
STA $0204,y			;
CLC					;
ADC #$08				;
STA $0208,y			;

LDA $16E7,x			;
SEC					;
SBC $02				;
STA $0201,y			;
STA $0205,y			;
STA $0209,y			;

LDX $07				;
LDA.w ScoreSprTile1,x		;
STA $0202,y			;
LDA.w ScoreSprTile2,x		;
STA $0206,y			;
LDA.w ScoreSprTile3,x		;
STA $020A,y			;

LDA.w ScoreSpritePalette,x	;
ORA #$30				;
STA $0203,y			;
STA $0207,y			;
STA $020B,y			;

PHY					;
TYA					;
LSR #2				;
TAY					;
LDA #$00				;
STA $0420,y			;
STA $0421,y			;
STA $0422,y			;
PLY					;

LDX $07				;
LDA.w TilesUsed,x		;
TAX					;
LDA #$F0				;
CPX #$02				;
BEQ .EndErase			; if the sprite uses 3 tiles, don't erase any of them
STA $0209,y			;
CPX #$01				; if the sprite uses 2 tiles, erase the last one
BEQ .EndErase			;
STA $0205,y			;
.EndErase				;
LDX $15E9				;
.EndGFX				;
RTS					;

GetOAMIndex:			;
LDY #$B0				; start the index at B0
.Loop					;
LDA $0201,y			;
CMP #$F0				; if this slot is empty...
BEQ .FoundSlot			; use this index
INY #4				; if not, increment Y until it reaches E0
CPY #$E0				;
BCC .Loop				; and all possible slots are exhausted
LDY #$FF				; if no slots are free, load FF
.FoundSlot				;
RTS					;

ScoreSpriteType08:	;

LDA $9D			; if sprites are locked...
BEQ $03			; jump to the GFX routine
JMP ScoreSpriteGFX	;

LDA $16FF,x		;
BNE .NoErase		; if the score sprite speed is 0...
STZ $16E1,x		; erase the sprite
RTS				;

.NoErase			;

DEC $16FF,x		;
CMP #$2A			; unless the sprite Y speed is exactly 2A...
BNE .NoBonus		; skip the part of code that gives the player points, lives, etc.

LDA $0F48			;
CLC				;
ADC #$0A			;
CMP #$64			;
BCC .NoMax			;
LDA #$63			;
.NoMax			;
STA $0F48			;

.NoBonus			;
JSR ScoreSpriteSpeed	;
JMP ScoreSpriteGFX	;

ScoreSpriteType09:		;
ScoreSpriteType0A:		;
ScoreSpriteType0B:		;

EndCode:









