;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Custom Score Sprites (simple version), by imamelia
;;
;; This patch enables you to insert custom score sprites into your ROM (the little
;; point notations that appear when you get mushrooms and stuff).  SMW uses
;; score sprites 01-0F, 10-15 are unused score sprites that have glitched graphics,
;; and 16-FF are completely unused slots that can be used for custom score sprites.
;; You can even use $02ACEF, SMW's built-in score sprite spawning routine, to spawn
;; the custom score sprites.  $02ADC9 is the code for all the score sprites in the
;; original SMW; you can use it as an example.  There is also small sample code that
;; score sprite 16 instantly give the first player 99 bonus stars.
;;
;; Score sprites:
;;
;; 00 - empty slot
;; 01 - 10 pts.
;; 02 - 20 pts.
;; 03 - 40 pts.
;; 04 - 80 pts.
;; 05 - 100 pts.
;; 06 - 200 pts.
;; 07 - 400 pts.
;; 08 - 800 pts.
;; 09 - 1000 pts.
;; 0A - 2000 pts.
;; 0B - 4000 pts.
;; 0C - 8000 pts.
;; 0D - 1-Up
;; 0E - 2-Up
;; 0F - 3-Up
;; 10 - 5-Up (unused and glitched)
;; 11 - 5 coins (unused and glitched)
;; 12 - 10 coins (unused and glitched)
;; 13 - 15 coins (unused and glitched)
;; 14 - 20 coins (unused and glitched)
;; 15 - 25 coins (unused and glitched)
;; 16-FF - Unused, as far as I know
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

!Freespace = $188000

org $02ADBD
JML ScoreSpriteHack
NOP

org !Freespace
db "STAR"
dw EndCode-StartCode-$01
dw EndCode-StartCode-$01^$FFFF

StartCode:

ScoreSpriteHack:

LDA $16E1,x		; if the score sprite number is 00...
BEQ .Return			; then the slot is blank
CMP #$16			; if the score sprite number is 16 or greater...
BCS .CustomScoreSprites	; then it is a custom score sprite
JML $02ADC2		; else, jump back to the normal score sprite codes

.CustomScoreSprites	;

PHB				;
PHK				;
PLB				; change data bank
SEC				;
SBC #$16			; score sprite number - $16
REP #$30			; set 16-bit AXY
AND.w #$00FF		; clear the high byte of A
ASL				; multiply the score sprite number by 2, since the pointers are 16-bit
TAY				; result into Y
LDA .Pointers,y		; score sprite pointers
STA $00			; into scratch RAM
SEP #$30			; set 8-bit AXY
PEA.w .Return2-$01	; push a return address
JMP ($0000)		;
.Return2			;
PLB				;
.Return			;
JML $02ADC5		; if the slot is blank, skip over it

.Pointers
dw CustomScoreSprite16		; 16 - 
dw CustomScoreSprite17		; 17 - 
dw CustomScoreSprite18		; 18 - 
dw CustomScoreSprite19		; 19 - 
dw CustomScoreSprite1A	; 1A - 
dw CustomScoreSprite1B		; 1B - 
dw CustomScoreSprite1C	; 1C - 
dw CustomScoreSprite1D	; 1D - 
dw CustomScoreSprite1E		; 1E - 
dw CustomScoreSprite1F		; 1F - 
;dw CustomScoreSprite20	; add as many more pointers as you need
;...
;dw CustomScoreSpriteFF	;

CustomScoreSprite16:

LDA #$63
STA $0F48
STZ $16E1,x
RTS

CustomScoreSprite17:
CustomScoreSprite18:
CustomScoreSprite19:
CustomScoreSprite1A:
CustomScoreSprite1B:
CustomScoreSprite1C:
CustomScoreSprite1D:
CustomScoreSprite1E:
CustomScoreSprite1F:
;CustomScoreSprite20:
;...
;CustomScoreSpriteFF:



EndCode:






