;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Extended Object +14, by imamelia
;;
;; This patch enables you to have 14 more extended objects in your ROM.
;; Extended objects are basically codes that run at level load that are usually used to
;; place Map16 tiles in certain places (such as at the position of the object itself).
;; Often, the Map16 tiles that are created depend on a certain RAM address.  SMW uses
;; the switch block flags to determine whether switch block objects should be outlined
;; or filled in.  You could, for example, make an object that is a door only after 40 levels
;; have been beaten, but just appears as blank space otherwise.
;;
;; Notes:
;;
;; - This patch is fully compatible with 1024's ObjecTool.  If you want more than 14
;; extra extended objects, use that.
;; - There are a couple of things you should know when writing the code for extended
;; objects.  Firstly, [$6E],y holds the high byte of the Map16 number, and [$6B],y holds the
;; low byte.  For instance, LDA #$01 STA [$6E],y LDA #$30 STA [$6B],y will create a cement
;; block.  Secondly, Y should normally contain the index pointing to your Map16 tile, which
;; is in $57.  Placing LDY $57 at the beginning of every code will work.  However, if you want
;; to shift the position of a tile, you have to change Y.  If you increase Y by 1 (i.e. $01), the
;; Map16 tile will be shifted one tile to the right.  If you increase Y by 16 (i.e. $10), the
;; Map16 tile will be shifted one tile down.  You can also decrease Y to shift the tile left or up.
;; Thirdly, you should probably make all codes branch or jump to "Return:" or copy the code I put ;; there; it returns in a..."special" way.  There is a sample code included, which will create
;; a 32x32 block of black space indiscriminately of any RAM address.
;; - BIG NOTE: To use the objects in Lunar Magic, you must uncheck "Correct Fatal Errors".
;; If you do not do this, then when you try to insert the object, Lunar Magic will see it as an
;; error and automatically change it to extended object 10.  No, the objects won't display
;; correctly in Lunar Magic, and yes, you do have to insert them manually.  Put 0 in for the
;; object number, then put the extended object number in for "Size/Type/Ext".
;;
;; Have fun!
;;
;; -imamelia
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

!Freespace = $1E8000

org $0DA115	; hijack the original pointers for extended objects 02-0F

dl CustObj02	; Pointers...
dl CustObj03	; Don't touch these.
dl CustObj04
dl CustObj05
dl CustObj06
dl CustObj07
dl CustObj08
dl CustObj09
dl CustObj0A
dl CustObj0B
dl CustObj0C
dl CustObj0D
dl CustObj0E
dl CustObj0F

org !Freespace

db "STAR"
dw CodeEnd-CodeStart-$01
dw CodeEnd-CodeStart-$01^$FFFF

CodeStart:	; This is where the actual code goes.

CustObj02:
CustObj03:
CustObj04:
CustObj05:
CustObj06:
CustObj07:
CustObj08:
CustObj09:
CustObj0A:
CustObj0B:
CustObj0C:
CustObj0D:
CustObj0E:
CustObj0F:

Return:

JML $0DA53C

Blank32:

LDY $57
LDA #$00 ; High byte of Map16 entry for first block
STA [$6E],y
LDA #$25 ; Low byte of Map16 entry for first block
STA [$6B],y
TYA
CLC
ADC #$01 ; Amount of tiles to move forward (#$10=one tile downwards, #$01=one tile to the right)
TAY
LDA #$00 ; High byte of Map16 entry for second block
STA [$6E],y
LDA #$25 ; Low byte of Map16 entry for second block
STA [$6B],y
TYA
CLC
ADC #$10 ; Amount of tiles to move forward (#$10=one tile downwards, #$01=one tile to the right)
TAY
LDA #$00 ; High byte of Map16 entry for third block
STA [$6E],y
LDA #$25 ; Low byte of Map16 entry for third block
STA [$6B],y
TYA
SEC
SBC #$01 ; Amount of tiles to move forward (#$10=one tile downwards, #$01=one tile to the right)
TAY
LDA #$00 ; High byte of Map16 entry for fourth block
STA [$6E],y
LDA #$25 ; Low byte of Map16 entry for fourth block
STA [$6B],y
RTS

CodeEnd:



