;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;HDMA Vertical Layer 1 Waves
;by KilloZapit, modified by Ice Man
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	lorom				;\ ROM is LoRom
	header				;| and has a header
	org $228000			;/ Address to insert code

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;RATS Tag Macro
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

macro RATS_start(id)
	db "STAR"
	dw RATS_Endcode<id>-RATS_Startcode<id>
	dw RATS_Endcode<id>-RATS_Startcode<id>^#$FFFF
RATS_Startcode<id>:
endmacro

macro RATS_end(id)
RATS_Endcode<id>:
endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;HDMA Channel Settings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%RATS_start(0)
	REP #$20			;\ Set processor 16 bit
	LDA #$0E02			;| $4330 = $210E
	STA $4330			;| $4331 = Mode 02
	LDA #$9E00			;| 
	STA $4332			;| Destination: $7F9E00
	LDY #$7F			;| (low and high byte)
	STY $4334			;| 
	SEP #$20			;/ Set processor 8 bit

	LDA #$08			;\ Enable HDMA
	TSB $0D9F			;/ on channel 3

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;HDMA Main Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	LDX #$00			;\ Init of
	LDY #$00			;/ X and Y

	SEP #$20			;\ Set processor 8 bit
	LDA $13				;| Set speed of waves
	LSR A				;| Adding more LSR A
	LSR A				;| will make it slower
	STA $00				;/ Store in scratch RAM

	PHB					;\ Push data bank
	PHK					;| Push program bank
	PLB					;/ Pull data bank

Wave_Loop:
	LDA #$06			;\ Set scanline height
	STA $7F9E00,x		;| for each wave
	TYA					;| Transfer Y to A
	ADC $00				;| Add in scratch RAM
	AND #$0F			;| and transfer #$0F bytes
	PHY					;| Push Y
	TAY					;/ Transfer A to Y

	LDA.w Wave_Table,y	;\ Load in wave values
	LSR A				;| half of waves only
	CLC					;| Clear carry flag
	ADC $1464			;| Apply to layer 1
	STA $7F9E01,x		;| Y position low byte
	LDA $1465			;| And add nothing to
	ADC #$00			;| layer 1 Y position
	STA $7F9E02,x		;/ high byte

	PLY					;\ Pull Y
	CPY #$25			;| Compare with #$25 scanlines
	BPL End_Wave		;| If bigger, end HDMA
	INX					;| Increase X
	INX					;| Increase X
	INX					;| Increase X
	INY					;| Increase Y
	BRA Wave_Loop		;/ Do the loop

End_Wave:
	PLB					;\ Pull data bank
	LDA #$00			;| End HMDA by writing
	STA $7F9E03,x		;| #$00 here
	RTL					;/ Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Table Settings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Wave_Table:
	db $00
	db $01
	db $02
	db $03
	db $04
	db $05
	db $06
	db $07
	db $07
	db $06
	db $05
	db $04
	db $03
	db $02
	db $01
	db $00
%RATS_end(0)