;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; LC_LZ2 Optimization Patch v1.2
;
; By:
; - Ersanio
; - edit1754
; - 33953YoShI
; - Min
; - smkdan
;
; In addition to being much faster than the original
; this one also stores the decompressed size to $8D,
; which may be useful in patches that upload ExGFX files
;
; NEW in v1.1:
; - rearranged some branches so they branch in the less-likely case rather than
;   the more-likely case (Ex: BEQ .end rather than BEQ .notEnd)
; - rewrote the MVN $7E7E / MVN $7F7F portion of code because it destroyed
;   the ability to decompress to SRAM.
;
; NEW in v1.2
; - should have no issues with interrupts.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HEADER
LOROM

!Freespace = $178000|$800000

!S = $D8			; 0x10 bytes in $7E00XX that aren't touched during interrupts (NMI or IRQ)

macro ReadByte()
	STX $8A
	LDA [$8A]
	INX
	BNE +
	LDX #$8000
	INC.b $03+!S
	INC $8C
+
endmacro

org $80B8E3
	JSL CodeStart		;was JML before
	RTS
	
org !Freespace
	reset bytes
	db "STAR"
	dw CodeEnd-CodeStart-$01
	dw CodeEnd-CodeStart-$01^$FFFF

CodeStart:
	PHB
	PEI ($00+!S)
	PEI ($02+!S)
	PEI ($04+!S)
	PEI ($06+!S)
	PEI ($08+!S)
	PEI ($0A+!S)
	PEI ($0C+!S)
	PEI ($0E+!S)
	PEI ($8A)
	SEP #$20
	REP #$10
	LDA $02
	PHA
	PLB
	STA.b $02+!S	; dest_bank (direct-copy)
	STA.b $0B+!S	; dest_bank (indirect-copy)
	STA.b $0C+!S	; src_bank (indirect-copy)
	INC
	STA.b $00+!S	; dest_bank [plus or minus]
	LDA #$54
	STA.b $01+!S	; mvn (direct-copy)
	STA.b $0A+!S	; mvn (indirect-copy)
	LDA #$4C
	STA.b $04+!S	; jmp (direct-copy)
	STA.b $0D+!S	; jmp (indirect-copy)
	LDA $8C
	STA.b $03+!S	; src_bank (direct-copy)
	LDX.w #.back
	STX.b $05+!S	; jmp to (direct-copy)
	LDX.w #.back2
	STX.b $0E+!S	; jmp to (indirect-copy)
	
	LDY $00		; dest_low
	LDX $8A		; src_low
	STZ $8A
	STZ $8B
	BRA .main
	
.end
	PLX : STX $8A
	PLX : STX.b $0E+!S
	PLX : STX.b $0C+!S
	PLX : STX.b $0A+!S
	PLX : STX.b $08+!S
	PLX : STX.b $06+!S
	PLX : STX.b $04+!S
	PLX : STX.b $02+!S
	PLX : STX.b $00+!S
	REP #$20
	TYA
	SEC
	SBC $00
	STA $8D		; size!!!
	SEP #$30
	PLB
	RTL			;JML $80B8EA

.case_e0_or_end
	LDA $8D
	CMP #$1F
	BEQ .end

	AND #$03
	STA $8E
	EOR $8D
	ASL
	ASL
	ASL
	XBA
	%ReadByte()
	STA $8D
	XBA
	BRA .type

.case_80_or_else
	BMI .case_e0_or_end

	%ReadByte()
	XBA
	%ReadByte()
	
	STX.b $08+!S
	REP #$21
	ADC $00
	TAX
	LDA $8D
	SEP #$20
	JMP $000A+!S

.back2
	LDX.b $08+!S
	
.main
	%ReadByte()
	STA $8D
	STZ $8E
	AND #$E0
	TRB $8D

.type
	ASL
	BCS .case_80_or_else
	BMI .case_40_or_60
	ASL
	BMI .case_20

.case_00
	REP #$20
	LDA $8D
	STX $8D
	
-	SEP #$20
	JMP $0001+!S
	
.back
	CPX $8D
	BCS .main
	
	INC.b $03+!S
	INC $8C
	CPX #$0000
	BEQ ++
	
	DEX
	STX.b $08+!S
	REP #$21
	LDX #$8000
	STX $8D
	TYA
	SBC.b $08+!S
	TAY
	LDA.b $08+!S
	BRA -
	
++	LDX #$8000
	BRA .main

.case_20
	%ReadByte()
	STX.b $08+!S
	PHA
	PHA
	REP #$20
	
.case_20_main
	LDA $8D
	INC
	LSR
	TAX
	PLA
	
-	STA $0000,Y
	INY
	INY
	DEX
	BNE -
	
	SEP #$20
	BCC +
	STA $0000,Y
	INY
+	LDX.b $08+!S
	BRA .main
	
.case_40_or_60
	ASL
	BMI .case_60
	%ReadByte()
	XBA
	%ReadByte()

	XBA
	STX.b $08+!S
	REP #$20
	PHA
;Replace BRA .case_20_main the code itself
	LDA $8D
	INC
	LSR
	TAX
	PLA
-	STA $0000,Y
	INY
	INY
	DEX
	BNE -
	SEP #$20
	BCC +
	STA $0000,Y
	INY
+	LDX.b $08+!S
	JMP .main
	
.case_60
	%ReadByte()
	STX.b $08+!S
	LDX $8D
-	STA $0000,Y
	INC
	INY
	DEX
	BPL -
	LDX.b $08+!S
	JMP .main

CodeEnd:
print "Insert Size: ",bytes," bytes"