
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Notes:
; In the original Layer 3-handling routines, the formula for figuring out the image
; and behavior setting went something like this:
; - Take the tileset number and multiply by 3.
; - Use that as an index to the Layer 3 settings table.
; - Check the Layer 3 settings table at $009F88.
; - If the value here is positive...
;	- Store it to the tide number RAM ($1403).
;	- If the value was zero, set the initial Layer 3 Y position to 0070; else, set it
;		to 0040.
;	- During the main routine, make the tide move 1 pixel to the left each frame.
;	- If the tide rises and falls, modify its Y position as well, unless $1B9D is set.
;		($1B9D is used to freeze the tide temporarily when it is at the top or bottom.)
; - If the value here is negative...
;	- Check bit 6.  If bit 6 is also set, then use the settings for the cage; set the
;		Y position to 00D0, disable CGADSUB on Layer 3, and set $13D5 to make
;		Layer 3 follow Layer 1 in the main routine.
;	- If bit 6 is clear, then check if any other bits were set.
;	- If no other bits were set...
;		- Load the smasher palette.
;		- Use the same settings that were used for the cage.
;	- If at least one other bit was set...
;		- Check the tileset number.
;		- If the tileset is 1 or 3...
;			- Set the Layer 3 X position to half the Layer 1 X position, both in
;			- the init and main routines.
;		- If the tileset is not 1 or 3...
;			- Skip to the end of the init routine.
;			- In the main routine, make the image scroll slowly automatically.
; - In all cases...
; - Set the Layer 3 pointer depending on $1BE3 (Layer 3 setting) in the init routine.
; - Run the stripe image DMA routine with data from said pointer.
; 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

; most of this is adapted from the code starting at $009FCC

L3Init01_InitTide1:

LDA #$02			; tide image number = 02
LDY #$40				; Layer 3 Y position = 0040
BRA L3TideInitSub		;

L3Init02_InitTide2:

LDA #$01			; tide image number = 01
LDY #$70				; Layer 3 Y position = 0070

L3TideInitSub:			;

STA $1403			; set the tide image number
STY $24				; set the Layer 3 Y position
STZ $25				; (seems to have an odd format...)
LDA #$04			;
TRB $40				; disable Layer 3 from being affected by CGADSUB

JSL $05BC72			; set up $26-$29

REP #$30				; start of the water tile loop
LDX #$0100			;
.Loop1				;
LDY #$0058			;
LDA #$0000			; low byte of the Map16 tile that the tide will act like
.Loop2				;
STA $7EE300,x		;
INX #2				;
DEY					;
BNE .Loop2			;
TXA					;
CLC					;
ADC #$0100			;
TAX					;
CPX #$1B00			;
BCC .Loop1			;
SEP #$30				;
LDA #$80			;
TSB $5B				; set the "interact with layer 2/3" flag
RTL					;

L3Init03_InitSmashers:

LDX #$07				;
.Loop				;
LDA $00B66C,x		; overwrite part of the palette
STA $071B,x			; with the colors for the smasher and cage
DEX					;
BPL .Loop				;

L3Init04_InitCage:

LDA #$D0			;
STA $24				; set the initial Y position of Layer 3
STZ $25				;
LDA #$04			;
TRB $40				; disable Layer 3 from being affected by CGADSUB

L3Init05_InitFog:

RTL					;

L3Init06_InitWindows:

REP #$20				;
LDA $1A				; Layer 1 X position
LSR					; divided by 2
STA $22				; equals the initial X position for Layer 3
SEP #$20				;
LDA #$C0			;
STA $24				; set the initial Y position of Layer 3
STZ $25				;
LDA #$04			;
TRB $40				; disable Layer 3 from being affected by CGADSUB
RTL					;

; most of this is adapted from the code starting at $05C40C

TideIncTbl:
db $FF,$01

TideTbl2:
db $FC,$04

TideEndHeight:
db $30,$A0

Data05CBBB:
db $04,$00,$FC,$FF,$04,$00,$FC,$FF

L3Main02_Tide2:

LDA $9D				;
BNE TideMainShared	;

LDY $1460			; Layer 3 scrolling direction
LDA $14				;
AND #$03			;
BNE .NoUpdate		;
LDA $145A			;
BNE .Label1			;
DEC $1B9D			;
BNE TideMainShared	;
.Label1				;
CMP TideTbl2,y		;
BEQ .Label2			;
CLC					;
ADC TideIncTbl,y		;
STA $145A			;
.Label2				;
LDA #$4B				;
STA $1B9D			;
.NoUpdate			;
LDA $24				;
CMP TideEndHeight,y	;
BNE .NoReverse		;
TYA					;
EOR #$01				;
STA $1460			;
.NoReverse			;
LDA $145A			;
ASL #4				;
CLC					;
ADC $145C			;
STA $145C			;
LDA $145A			;
PHP					;
LSR #4				;
PLP					;
BPL $02				;
ORA #$F0			;
ADC $24				;
STA $24				;

L3Main01_Tide1:

TideMainShared:		;

LDA $22				;
SEC					;
ADC $17BD			;
STA $22				;
LDA #$01			;
STA $23				;
RTL					;            

L3Main06_Windows:

REP #$20				;
LDA $1A				; Layer 1 X position
LSR					; divided by 2
STA $22				; equals the X position for Layer 3
SEP #$20				;

L3Main03_Smashers:
L3Main04_Cage:

RTL

L3Main05_Fog:

REP #$20				;
LDY $9D				;
BNE .Skip				;

LDA $1460			;
AND #$00FF			;
TAY					;
LDA #$0040			;
CPY #$01				;
BEQ .NoFlip			;
LDA #$FFC0			;
.NoFlip				;
LDY #$00				;
CMP $1458			;
BEQ .Label1			;
BPL .Label2			;
LDY #$02				;
.Label2				;
LDA $1458			;
CLC					;
ADC Data05CBBB,y		;
STA $1458			;
.Label1				;
LDA $145C			;
AND #$00FF			;
CLC					;
ADC $1458			;
STA $145C			;
AND #$FF00			;
BPL $03				;
ORA #$00FF			;
XBA					;
CLC					;
ADC $22				;
STA $22				;
LDA $17BD			;
AND #$00FF			;
CMP #$0080			;
BCC $03				;
ORA #$FF00			;
STA $00				;
LDA $22				;
CLC					;
ADC $00				;
STA $22				;
.Skip				;
LDA $1C				;
STA $24				;
SEP #$20				;
RTL					;








