;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Level Depending On RAM v0.8
;made by Deflaktor
;
;bug reports and questions go to deflaktor@oleco.net
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

;--------------------------
;---------defines----------
;--------------------------

!FreeSpace = $10EF00	;if already used, relocate

; You can change the table at the end of this file

;--------------------------
;---------replaces---------
;--------------------------

org $05DCE2		;hijack level loading routine
JSL Main
NOP #4
;BCC .skip		\ replaced opcodes
;SBC #$24		|
;INY			|
;.skip			|
;STA $17BB		/

;--------------------------
;----overworld routines----
;--------------------------

org !FreeSpace		;the SNES adress the code will be inserted to
!CodeSize = CodeEnd-CodeStart
db "STAR"
dw !CodeSize-$01
dw !CodeSize-$01^$FFFF

CodeStart:
Main:	PHY
	PHX
	PHP
	LDA $13BF	; load the level number
	PHA
	LDA $141D	; load the "Disable Mario Start! Flag"
	BEQ GoOn	; if the flag is 1 (because it is only 0 while loading the overworld level)
	PLA
	BRA Ret2	; return
GoOn:	PLA
	REP #$30	; let A,X and Y go 16 bit, since we deal with large numbers > FF
	JSR LevelToRoom	; convert the level number to a room number
	LDX #$0000	; load zero to X (the index for the table)

Loop:	PHA
	LDA Table,x	
	TAY
	PLA
	CPY #$FFFF	; end of table reached?
	BEQ Return	; return
	CMP Table,x
	BEQ RAM		; we found an entry, check RAM address
	INX #6		; increase index by 6
	BRA Loop	; loop

RAM:	INX #4		; increase X by 4 to get the RAM address
	LDA Table,x	; load RAM address
	STA $0E		; store the pointer into the scratch ram
	LDA ($0E)	; load RAM address from the pointer
	BNE SetLvl	; if the address is not zero, set the new level number

	; else load the level number and jump back to the loop
	SEP #$20	; let A go back to 8 bit
	LDA $13BF	; load the level number
	JSR LevelToRoom ; convert the level number to a room number
	INX #2		; increase X by 2 to get the next row of the table
	BRA Loop

	; now that we know the event is set, change the level number

SetLvl: DEX #2		; decrease index X by 2 to get the new level number
	LDA Table,x
Return:	JSR RoomToLevel	; convert the room number to the level number
	SEP #$20	; let A go back to 8 bit

Ret2:	PLP
	PLX
	PLY
	BCC .skip
	SBC #$24
	INY
.skip	STA $17BB
	RTL

; Subroutine that converts overworld level numbers to room numbers
LevelToRoom: 	REP #$20	; let A go 16 bit, since we deal with large numbers > FF
		AND #$00FF	; mask the high byte
		CMP #$0025
		BCC ReturnLR	; if level number is greater than #$025
		CLC
		ADC #$00DC	; subtract #$0DC from it
ReturnLR:	RTS

; Subroutine that converts room numbers to overworld level numbers
RoomToLevel: 	CMP #$0025
		BCC ReturnRL	; if level number is greater than #$025
		SEC
		SBC #$00DC	; add #$0DC to it
ReturnRL:	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 	edit this table: use only levels 000-024 and 101-13B
;	DO NOT MIX THE HIGH BIT OF THE LEVELS (0XX and 1XX)
;		e.g.	024 -> 105   WRONG
;			127 -> 005   WRONG
;			020 -> 023   RIGHT
;			107 -> 130   RIGHT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Table:
dw #$105, #$13B, #$0019		;level 105 becomes level 13B when RAM address $0019 is not #$00 (Player is not small)
dw #$101, #$114, #$13C7		;level 101 becomes level 114 when RAM address $13C7 is not #$00 (Player has yoshi)
dw #$FFFF			;table end (necessary)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CodeEnd:
