;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;LevelASM v1.4. 
;Made by Ersanio with help from certain people.
;To be applied with Xkas
;You can access another level's code by modifying RAM $010B and $010C
;To do so, just store the new level number in RAM $010B when you are in 16-bit mode.
;An example code *(For use with Custom Blocks)*
;	REP #$30
;	LDA #$0105	;Access the levelASM code of level 105
;	STA $010B
;	SEP #$30
;When you touch this block, LevelASM will execute the codes that level 105.
;Simple as that.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HEADER
LOROM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Defines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!FreeSpace 	= $1D8000	;Point this to some free space in the ROM, preferably below bank $40
	!FreeRAM	= $010B		;<-- It is recommended that you do NOT edit this.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Levelnum.ips disassembly
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ORG $05D8B9
	JSR Levelnummain

ORG $05DC46
Levelnummain:	LDA $0E  		;Load level number
		STA !FreeRAM		;Store it in free stack RAM
		ASL A    		;Multiply A by 2, probably to recover old code?
		STZ $0DC4
		STZ $0F3A
		RTS      		;Return from subroutine


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Hijack routine for LevelASM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ORG $00A242|$800000
	JML MAIN : NOP

org $00A5EE|$800000
        JML INITStart ;Hijack INIT stuff

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;New main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ORG !FreeSpace|$800000

db "STAR"				;\
dw MainEnd-MainStart-$01		; |Write RATS tag #1 to protect main code
dw MainEnd-MainStart-$01^$FFFF		;/

MainStart:
MAIN:
	PHB				;\
	PHK				; |Setup program bank
	PLB				;/
	REP #$30			;16-bit AXY
	LDA !FreeRAM			;Load Level Number from stack
	ASL A				;x2
	TAX				;Transfer level # x2 into X
	LDA LevelASMTable,x		;Load table, indexed with X
	STA $00				;Store in $00 and $01
	SEP #$30			;8-bit AXY
	LDX #$00			;Clear X
	JSR ($0000,x)			;And jump to subroutine in pointers
JSR RunsEachFrame
	PLB				;Recover program bank
	LDA $13D4
	BEQ +
	JML $00A25B|$800000
+	JML $00A28A|$800000
INITStart:
        LDA !FreeRAM
        ASL A
        TAX
        LDA LevelInitTable,x
        STA $00
        SEP #$30
        LDX #$00
	PHB
	PHK
	PLB
        JSR ($0000,x)
	PLB
        PHK
        PEA.w RETURNADDR-1
        PEA $84CE
        JML $00919B

RETURNADDR:
	JML $00A5F3



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Pointers and Code. Don't touch this.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LevelASMTable:
dw level0	;Level 000
dw level1	;Level 001
dw level2	;Level 002
dw level3	;etc...
dw level4
dw level5
dw level6
dw level7
dw level8
dw level9
dw levelA
dw levelB
dw levelC
dw levelD
dw levelE
dw levelF
dw level10
dw level11
dw level12
dw level13
dw level14
dw level15
dw level16
dw level17
dw level18
dw level19
dw level1A
dw level1B
dw level1C
dw level1D
dw level1E
dw level1F
dw level20
dw level21
dw level22
dw level23
dw level24
dw level25
dw level26
dw level27
dw level28
dw level29
dw level2A
dw level2B
dw level2C
dw level2D
dw level2E
dw level2F
dw level30
dw level31
dw level32
dw level33
dw level34
dw level35
dw level36
dw level37
dw level38
dw level39
dw level3A
dw level3B
dw level3C
dw level3D
dw level3E
dw level3F
dw level40
dw level41
dw level42
dw level43
dw level44
dw level45
dw level46
dw level47
dw level48
dw level49
dw level4A
dw level4B
dw level4C
dw level4D
dw level4E
dw level4F
dw level50
dw level51
dw level52
dw level53
dw level54
dw level55
dw level56
dw level57
dw level58
dw level59
dw level5A
dw level5B
dw level5C
dw level5D
dw level5E
dw level5F
dw level60
dw level61
dw level62
dw level63
dw level64
dw level65
dw level66
dw level67
dw level68
dw level69
dw level6A
dw level6B
dw level6C
dw level6D
dw level6E
dw level6F
dw level70
dw level71
dw level72
dw level73
dw level74
dw level75
dw level76
dw level77
dw level78
dw level79
dw level7A
dw level7B
dw level7C
dw level7D
dw level7E
dw level7F
dw level80
dw level81
dw level82
dw level83
dw level84
dw level85
dw level86
dw level87
dw level88
dw level89
dw level8A
dw level8B
dw level8C
dw level8D
dw level8E
dw level8F
dw level90
dw level91
dw level92
dw level93
dw level94
dw level95
dw level96
dw level97
dw level98
dw level99
dw level9A
dw level9B
dw level9C
dw level9D
dw level9E
dw level9F
dw levelA0
dw levelA1
dw levelA2
dw levelA3
dw levelA4
dw levelA5
dw levelA6
dw levelA7
dw levelA8
dw levelA9
dw levelAA
dw levelAB
dw levelAC
dw levelAD
dw levelAE
dw levelAF
dw levelB0
dw levelB1
dw levelB2
dw levelB3
dw levelB4
dw levelB5
dw levelB6
dw levelB7
dw levelB8
dw levelB9
dw levelBA
dw levelBB
dw levelBC
dw levelBD
dw levelBE
dw levelBF
dw levelC0
dw levelC1
dw levelC2
dw levelC3
dw levelC4
dw levelC5
dw levelC6
dw levelC7
dw levelC8
dw levelC9
dw levelCA
dw levelCB
dw levelCC
dw levelCD
dw levelCE
dw levelCF
dw levelD0
dw levelD1
dw levelD2
dw levelD3
dw levelD4
dw levelD5
dw levelD6
dw levelD7
dw levelD8
dw levelD9
dw levelDA
dw levelDB
dw levelDC
dw levelDD
dw levelDE
dw levelDF
dw levelE0
dw levelE1
dw levelE2
dw levelE3
dw levelE4
dw levelE5
dw levelE6
dw levelE7
dw levelE8
dw levelE9
dw levelEA
dw levelEB
dw levelEC
dw levelED
dw levelEE
dw levelEF
dw levelF0
dw levelF1
dw levelF2
dw levelF3
dw levelF4
dw levelF5
dw levelF6
dw levelF7
dw levelF8
dw levelF9
dw levelFA
dw levelFF
dw levelFC
dw levelFD
dw levelFE
dw levelFF
dw level100
dw level101
dw level102
dw level103
dw level104
dw level105
dw level106
dw level107
dw level108
dw level109
dw level10A
dw level10B
dw level10C
dw level10D
dw level10E
dw level10F
dw level110
dw level111
dw level112
dw level113
dw level114
dw level115
dw level116
dw level117
dw level118
dw level119
dw level11A
dw level11B
dw level11C
dw level11D
dw level11E
dw level11F
dw level120
dw level121
dw level122
dw level123
dw level124
dw level125
dw level126
dw level127
dw level128
dw level129
dw level12A
dw level12B
dw level12C
dw level12D
dw level12E
dw level12F
dw level130
dw level131
dw level132
dw level133
dw level134
dw level135
dw level136
dw level137
dw level138
dw level139
dw level13A
dw level13B
dw level13C
dw level13D
dw level13E
dw level13F
dw level140
dw level141
dw level142
dw level143
dw level144
dw level145
dw level146
dw level147
dw level148
dw level149
dw level14A
dw level14B
dw level14C
dw level14D
dw level14E
dw level14F
dw level150
dw level151
dw level152
dw level153
dw level154
dw level155
dw level156
dw level157
dw level158
dw level159
dw level15A
dw level15B
dw level15C
dw level15D
dw level15E
dw level15F
dw level160
dw level161
dw level162
dw level163
dw level164
dw level165
dw level166
dw level167
dw level168
dw level169
dw level16A
dw level16B
dw level16C
dw level16D
dw level16E
dw level16F
dw level170
dw level171
dw level172
dw level173
dw level174
dw level175
dw level176
dw level177
dw level178
dw level179
dw level17A
dw level17B
dw level17C
dw level17D
dw level17E
dw level17F
dw level180
dw level181
dw level182
dw level183
dw level184
dw level185
dw level186
dw level187
dw level188
dw level189
dw level18A
dw level18B
dw level18C
dw level18D
dw level18E
dw level18F
dw level190
dw level191
dw level192
dw level193
dw level194
dw level195
dw level196
dw level197
dw level198
dw level199
dw level19A
dw level19B
dw level19C
dw level19D
dw level19E
dw level19F
dw level1A0
dw level1A1
dw level1A2
dw level1A3
dw level1A4
dw level1A5
dw level1A6
dw level1A7
dw level1A8
dw level1A9
dw level1AA
dw level1AB
dw level1AC
dw level1AD
dw level1AE
dw level1AF
dw level1B0
dw level1B1
dw level1B2
dw level1B3
dw level1B4
dw level1B5
dw level1B6
dw level1B7
dw level1B8
dw level1B9
dw level1BA
dw level1BB
dw level1BC
dw level1BD
dw level1BE
dw level1BF
dw level1C0
dw level1C1
dw level1C2
dw level1C3
dw level1C4
dw level1C5
dw level1C6
dw level1C7
dw level1C8
dw level1C9
dw level1CA
dw level1CB
dw level1CC
dw level1CD
dw level1CE
dw level1CF
dw level1D0
dw level1D1
dw level1D2
dw level1D3
dw level1D4
dw level1D5
dw level1D6
dw level1D7
dw level1D8
dw level1D9
dw level1DA
dw level1DB
dw level1DC
dw level1DD
dw level1DE
dw level1DF
dw level1E0
dw level1E1
dw level1E2
dw level1E3
dw level1E4
dw level1E5
dw level1E6
dw level1E7
dw level1E8
dw level1E9
dw level1EA
dw level1EB
dw level1EC
dw level1ED
dw level1EE
dw level1EF
dw level1F0
dw level1F1
dw level1F2
dw level1F3
dw level1F4
dw level1F5
dw level1F6
dw level1F7
dw level1F8
dw level1F9
dw level1FA
dw level1FB
dw level1FC
dw level1FD
dw level1FE
dw level1FF

LevelInitTable:
dw levelinit0	;Level 000
dw levelinit1	;Level 001
dw levelinit2	;Level 002
dw levelinit3	;etc...
dw levelinit4
dw levelinit5
dw levelinit6
dw levelinit7
dw levelinit8
dw levelinit9
dw levelinitA
dw levelinitB
dw levelinitC
dw levelinitD
dw levelinitE
dw levelinitF
dw levelinit10
dw levelinit11
dw levelinit12
dw levelinit13
dw levelinit14
dw levelinit15
dw levelinit16
dw levelinit17
dw levelinit18
dw levelinit19
dw levelinit1A
dw levelinit1B
dw levelinit1C
dw levelinit1D
dw levelinit1E
dw levelinit1F
dw levelinit20
dw levelinit21
dw levelinit22
dw levelinit23
dw levelinit24
dw levelinit25
dw levelinit26
dw levelinit27
dw levelinit28
dw levelinit29
dw levelinit2A
dw levelinit2B
dw levelinit2C
dw levelinit2D
dw levelinit2E
dw levelinit2F
dw levelinit30
dw levelinit31
dw levelinit32
dw levelinit33
dw levelinit34
dw levelinit35
dw levelinit36
dw levelinit37
dw levelinit38
dw levelinit39
dw levelinit3A
dw levelinit3B
dw levelinit3C
dw levelinit3D
dw levelinit3E
dw levelinit3F
dw levelinit40
dw levelinit41
dw levelinit42
dw levelinit43
dw levelinit44
dw levelinit45
dw levelinit46
dw levelinit47
dw levelinit48
dw levelinit49
dw levelinit4A
dw levelinit4B
dw levelinit4C
dw levelinit4D
dw levelinit4E
dw levelinit4F
dw levelinit50
dw levelinit51
dw levelinit52
dw levelinit53
dw levelinit54
dw levelinit55
dw levelinit56
dw levelinit57
dw levelinit58
dw levelinit59
dw levelinit5A
dw levelinit5B
dw levelinit5C
dw levelinit5D
dw levelinit5E
dw levelinit5F
dw levelinit60
dw levelinit61
dw levelinit62
dw levelinit63
dw levelinit64
dw levelinit65
dw levelinit66
dw levelinit67
dw levelinit68
dw levelinit69
dw levelinit6A
dw levelinit6B
dw levelinit6C
dw levelinit6D
dw levelinit6E
dw levelinit6F
dw levelinit70
dw levelinit71
dw levelinit72
dw levelinit73
dw levelinit74
dw levelinit75
dw levelinit76
dw levelinit77
dw levelinit78
dw levelinit79
dw levelinit7A
dw levelinit7B
dw levelinit7C
dw levelinit7D
dw levelinit7E
dw levelinit7F
dw levelinit80
dw levelinit81
dw levelinit82
dw levelinit83
dw levelinit84
dw levelinit85
dw levelinit86
dw levelinit87
dw levelinit88
dw levelinit89
dw levelinit8A
dw levelinit8B
dw levelinit8C
dw levelinit8D
dw levelinit8E
dw levelinit8F
dw levelinit90
dw levelinit91
dw levelinit92
dw levelinit93
dw levelinit94
dw levelinit95
dw levelinit96
dw levelinit97
dw levelinit98
dw levelinit99
dw levelinit9A
dw levelinit9B
dw levelinit9C
dw levelinit9D
dw levelinit9E
dw levelinit9F
dw levelinitA0
dw levelinitA1
dw levelinitA2
dw levelinitA3
dw levelinitA4
dw levelinitA5
dw levelinitA6
dw levelinitA7
dw levelinitA8
dw levelinitA9
dw levelinitAA
dw levelinitAB
dw levelinitAC
dw levelinitAD
dw levelinitAE
dw levelinitAF
dw levelinitB0
dw levelinitB1
dw levelinitB2
dw levelinitB3
dw levelinitB4
dw levelinitB5
dw levelinitB6
dw levelinitB7
dw levelinitB8
dw levelinitB9
dw levelinitBA
dw levelinitBB
dw levelinitBC
dw levelinitBD
dw levelinitBE
dw levelinitBF
dw levelinitC0
dw levelinitC1
dw levelinitC2
dw levelinitC3
dw levelinitC4
dw levelinitC5
dw levelinitC6
dw levelinitC7
dw levelinitC8
dw levelinitC9
dw levelinitCA
dw levelinitCB
dw levelinitCC
dw levelinitCD
dw levelinitCE
dw levelinitCF
dw levelinitD0
dw levelinitD1
dw levelinitD2
dw levelinitD3
dw levelinitD4
dw levelinitD5
dw levelinitD6
dw levelinitD7
dw levelinitD8
dw levelinitD9
dw levelinitDA
dw levelinitDB
dw levelinitDC
dw levelinitDD
dw levelinitDE
dw levelinitDF
dw levelinitE0
dw levelinitE1
dw levelinitE2
dw levelinitE3
dw levelinitE4
dw levelinitE5
dw levelinitE6
dw levelinitE7
dw levelinitE8
dw levelinitE9
dw levelinitEA
dw levelinitEB
dw levelinitEC
dw levelinitED
dw levelinitEE
dw levelinitEF
dw levelinitF0
dw levelinitF1
dw levelinitF2
dw levelinitF3
dw levelinitF4
dw levelinitF5
dw levelinitF6
dw levelinitF7
dw levelinitF8
dw levelinitF9
dw levelinitFA
dw levelinitFF
dw levelinitFC
dw levelinitFD
dw levelinitFE
dw levelinitFF
dw levelinit100
dw levelinit101
dw levelinit102
dw levelinit103
dw levelinit104
dw levelinit105
dw levelinit106
dw levelinit107
dw levelinit108
dw levelinit109
dw levelinit10A
dw levelinit10B
dw levelinit10C
dw levelinit10D
dw levelinit10E
dw levelinit10F
dw levelinit110
dw levelinit111
dw levelinit112
dw levelinit113
dw levelinit114
dw levelinit115
dw levelinit116
dw levelinit117
dw levelinit118
dw levelinit119
dw levelinit11A
dw levelinit11B
dw levelinit11C
dw levelinit11D
dw levelinit11E
dw levelinit11F
dw levelinit120
dw levelinit121
dw levelinit122
dw levelinit123
dw levelinit124
dw levelinit125
dw levelinit126
dw levelinit127
dw levelinit128
dw levelinit129
dw levelinit12A
dw levelinit12B
dw levelinit12C
dw levelinit12D
dw levelinit12E
dw levelinit12F
dw levelinit130
dw levelinit131
dw levelinit132
dw levelinit133
dw levelinit134
dw levelinit135
dw levelinit136
dw levelinit137
dw levelinit138
dw levelinit139
dw levelinit13A
dw levelinit13B
dw levelinit13C
dw levelinit13D
dw levelinit13E
dw levelinit13F
dw levelinit140
dw levelinit141
dw levelinit142
dw levelinit143
dw levelinit144
dw levelinit145
dw levelinit146
dw levelinit147
dw levelinit148
dw levelinit149
dw levelinit14A
dw levelinit14B
dw levelinit14C
dw levelinit14D
dw levelinit14E
dw levelinit14F
dw levelinit150
dw levelinit151
dw levelinit152
dw levelinit153
dw levelinit154
dw levelinit155
dw levelinit156
dw levelinit157
dw levelinit158
dw levelinit159
dw levelinit15A
dw levelinit15B
dw levelinit15C
dw levelinit15D
dw levelinit15E
dw levelinit15F
dw levelinit160
dw levelinit161
dw levelinit162
dw levelinit163
dw levelinit164
dw levelinit165
dw levelinit166
dw levelinit167
dw levelinit168
dw levelinit169
dw levelinit16A
dw levelinit16B
dw levelinit16C
dw levelinit16D
dw levelinit16E
dw levelinit16F
dw levelinit170
dw levelinit171
dw levelinit172
dw levelinit173
dw levelinit174
dw levelinit175
dw levelinit176
dw levelinit177
dw levelinit178
dw levelinit179
dw levelinit17A
dw levelinit17B
dw levelinit17C
dw levelinit17D
dw levelinit17E
dw levelinit17F
dw levelinit180
dw levelinit181
dw levelinit182
dw levelinit183
dw levelinit184
dw levelinit185
dw levelinit186
dw levelinit187
dw levelinit188
dw levelinit189
dw levelinit18A
dw levelinit18B
dw levelinit18C
dw levelinit18D
dw levelinit18E
dw levelinit18F
dw levelinit190
dw levelinit191
dw levelinit192
dw levelinit193
dw levelinit194
dw levelinit195
dw levelinit196
dw levelinit197
dw levelinit198
dw levelinit199
dw levelinit19A
dw levelinit19B
dw levelinit19C
dw levelinit19D
dw levelinit19E
dw levelinit19F
dw levelinit1A0
dw levelinit1A1
dw levelinit1A2
dw levelinit1A3
dw levelinit1A4
dw levelinit1A5
dw levelinit1A6
dw levelinit1A7
dw levelinit1A8
dw levelinit1A9
dw levelinit1AA
dw levelinit1AB
dw levelinit1AC
dw levelinit1AD
dw levelinit1AE
dw levelinit1AF
dw levelinit1B0
dw levelinit1B1
dw levelinit1B2
dw levelinit1B3
dw levelinit1B4
dw levelinit1B5
dw levelinit1B6
dw levelinit1B7
dw levelinit1B8
dw levelinit1B9
dw levelinit1BA
dw levelinit1BB
dw levelinit1BC
dw levelinit1BD
dw levelinit1BE
dw levelinit1BF
dw levelinit1C0
dw levelinit1C1
dw levelinit1C2
dw levelinit1C3
dw levelinit1C4
dw levelinit1C5
dw levelinit1C6
dw levelinit1C7
dw levelinit1C8
dw levelinit1C9
dw levelinit1CA
dw levelinit1CB
dw levelinit1CC
dw levelinit1CD
dw levelinit1CE
dw levelinit1CF
dw levelinit1D0
dw levelinit1D1
dw levelinit1D2
dw levelinit1D3
dw levelinit1D4
dw levelinit1D5
dw levelinit1D6
dw levelinit1D7
dw levelinit1D8
dw levelinit1D9
dw levelinit1DA
dw levelinit1DB
dw levelinit1DC
dw levelinit1DD
dw levelinit1DE
dw levelinit1DF
dw levelinit1E0
dw levelinit1E1
dw levelinit1E2
dw levelinit1E3
dw levelinit1E4
dw levelinit1E5
dw levelinit1E6
dw levelinit1E7
dw levelinit1E8
dw levelinit1E9
dw levelinit1EA
dw levelinit1EB
dw levelinit1EC
dw levelinit1ED
dw levelinit1EE
dw levelinit1EF
dw levelinit1F0
dw levelinit1F1
dw levelinit1F2
dw levelinit1F3
dw levelinit1F4
dw levelinit1F5
dw levelinit1F6
dw levelinit1F7
dw levelinit1F8
dw levelinit1F9
dw levelinit1FA
dw levelinit1FB
dw levelinit1FC
dw levelinit1FD
dw levelinit1FE
dw levelinit1FF

CodeStart:
incsrc Levelcode.asm			;Level Codes.

InitCodeStart:
incsrc Levelinitcode.asm		;Level inits.

RunsEachFrame:
HDMACircle:
REP #$20
LDA $010B
CMP #$00C7
BEQ .endre
LDA $80
BMI .endnd
CMP #$0108
BCS .endr
.endnd
SEP #$20
LDA $0F3A		;radius
BEQ .endr
LDX $9D
BNE .endr
DEC
STA $0F3A
BNE .ende
.endr
SEP #$20
LDA #$00
STA $41
STA $42
STA $43
.endre
JMP .end		;branch out of range :<
.ende
STA $4202
STA $4203		;put radius in squaring microwave... *beep beep beep mvrrrrrrrrrr...*
LDA $1337		;brkpoint
REP #$10
LDX #$0000
LDA $14
BIT #$01
BEQ .later
LDX #$0500
.later
STX $08
LDA $0F3A
ASL
CLC
ADC #$80
INC
STA $7FA008,x
REP #$20
TXA
CLC
ADC #$0009
TAX
LDA $0F3A
AND #$00FF
TAY
LDA $4216		;remove radius squared from microwave... jeez it's already cold
STA $04			;$04 = radius squared
SEP #$20
DEY
LDA $7E
CLC
ADC #$08
STA $06

-			;loop starts here
REP #$20
PHY
TYA
ASL
TAY
LDA $04
SEC
SBC.w SQUARE_DATA,y
PLY
STA $00			;ROOT takes 16-bit $00 as an arguement.
JSR ROOT		;$02 = X, since r-squared = x-squared + y-squared, then x = root(r-squared - y-squared)
SEP #$20
LDA $06
SEC
SBC $02
BCS .okaydokay
LDA #$00		;if less than 0, 0
.okaydokay
STA $7FA000,x
LDA $06
CLC
ADC $02
BCC .okaysidetwo
LDA #$FF
.okaysidetwo		;if greater than FF, FF
STA $7FA001,x
INX
INX
DEY
BPL -			;loop until Y hits 0

LDA #$02
STA $00			;halfway done
STZ $01
REP #$20
INY

-			;this loop goes backwards through the old data and stores it to the new data place
INY
TXA
SEC
SBC $00			;$00 = how far back/forward to go for corresponding values
TAX
LDA $7FA000,x
STA $02
TXA
CLC
ADC $00
TAX
LDA $02
STA $7FA000,x
INX
INX
LDA $00
CLC
ADC #$0004
STA $00
CPY $0F3A
BNE -

LDA #$0001			;wrap up
STA $7FA000,x
SEP #$20
DEC
STA $7FA002,x

LDX $08
LDA $0F3A
STA $0A
STZ $0B
REP #$20
LDA $80
CLC
ADC #$0018
SEC
SBC $0A
STA $0C				;$0C = windowstart ex: $FFF2
BMI .thatotherspecialcase	;ex: branch
CMP #$0080
BCS .thethirdspecialcase
LDA $0C
BNE .rarrrt
INC
.rarrrt
SEP #$20
STA $7FA005,x
LDA #$01
STA $7FA006,x
DEC
STA $7FA007,x
REP #$20
LDA #$A005
CLC
ADC $08
STA $4332
BRA .superunexpectedend
.thethirdspecialcase
LDA #$A002
CLC
ADC $08
STA $4332
SEP #$20
LDA #$7F
STA $7FA002,x
LDA #$01
STA $7FA003,x
DEC
STA $7FA004,x
LDA $0C
SEC
SBC #$7F
BPL .rarrrg
DEC
.rarrrg
STA $7FA005,x
LDA #$01
STA $7FA006,x
DEC
STA $7FA007,x
BRA .superunexpectedend
.donteventry
JMP .endr
.thatotherspecialcase
EOR #$FFFF			;NPAS	ex: 000D
INC				;ex: 0E
STA $0E				;$0E = NPAS
SEP #$20
LDA $0F3A
ASL				;diameter	ex: $4A
CMP $0E
BEQ .donteventry
BCC .donteventry		;if higher above screen than diameter	;ex: no branch
SEC				;don't try
SBC $0E
CLC
ADC #$81
STA $0A				;0A = diameter - NPAS. essentially number of pixels to show, store to tablestart
ASL $0E
REP #$20
TXA
CLC
ADC $0E
CLC
ADC #$0008
TAX
SEP #$20
LDA $0A
STA $7FA000,x
REP #$20
TXA
LDX $08
CLC
ADC #$A000
STA $4332
.superunexpectedend
;HDMA
LDA $1338
REP #$20
LDA #$2601		;$2126, two regs write once
STA $4330
;LDX #$0000
;LDA $14
;BIT #$0001
;BNE +
;LDX #$0500
;+
;STX $08
;LDA $7FA000,x
;CLC
;ADC #$A000
;CLC
;ADC $08
;STA $4332
SEP #$10
LDY #$7F
STY $4334
SEP #$20
LDA #$08

TSB $0D9F
LDA #$20		;windowing enable
STA $41
LDA $0F3B
BEQ .end
LDA #$33		;disregard
STA $41
STA $42
STA $43
.end
SEP #$30
STZ $0F3B
STZ $0F3C
RTS	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;
;$00$0116bit̐lẴTu[`Ăяo
;
;$02ɂ̕Ԃ܂B
;$03rvZŒׂ܂B
;
;Ȃ݂ɌvZ@̓j[g@QłB
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ROOT:
	PHX
	PHP
	SEP #$30
	LDA $01
BIT #$F0
	BNE ROOT_NOTS
REP #$10
	LDX $00
	LDA ROOT_DATA,x
	STA $02
	PLP
	PLX
	RTS

ROOT_NOTS:
	LDA $01
	STA $4205
	LDA $00
	STA $4204	;폜Zbg

	LDX $01
	LDA ROOT_DATA,x
	ASL
	ASL
	ASL
	ASL
	ORA #$07
	STA $4206	;ZbgB̌16TCN҂
	REP #$20	;3TCN
	AND #$00FF	;2TCN
	CLC		;2TCN
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP		;Ô10TCN
	ADC $4214
	LSR
	STA $02
	SEP #$30
	STA $4206	;ZbgB
	LDA $03		;3TCN	
	BEQ SKIP_FF	;2TCN
	LDA #$FF
	STA $02
	PLX
	RTS

SKIP_FF:
	REP #$30	;3TCN
	LDA $02		;3TCN
	CLC		;2TCN
	NOP
	NOP
	NOP
	NOP		;Ô4TCN
	ADC $4214
	LSR
	STA $02
	SEP #$30
	PLX
	RTS


ROOT_DATA:
	
db $00,$01,$01,$02,$02,$02,$02,$03,$03,$03,$03,$03,$03,$04,$04,$04
db $04,$04,$04,$04,$04,$05,$05,$05,$05,$05,$05,$05,$05,$05,$05,$06
db $06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$07,$07,$07,$07,$07
db $07,$07,$07,$07,$07,$07,$07,$07,$07,$08,$08,$08,$08,$08,$08,$08
db $08,$08,$08,$08,$08,$08,$08,$08,$08,$09,$09,$09,$09,$09,$09,$09
db $09,$09,$09,$09,$09,$09,$09,$09,$09,$09,$09,$0A,$0A,$0A,$0A,$0A
db $0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0B
db $0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B
db $0B,$0B,$0B,$0B,$0B,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C
db $0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0D,$0D,$0D
db $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
db $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E
db $0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E
db $0E,$0E,$0E,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
db $0F,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10
db $10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10
db $10,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11
db $11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11
db $11,$11,$11,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12
db $12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12
db $12,$12,$12,$12,$12,$12,$12,$13,$13,$13,$13,$13,$13,$13,$13,$13
db $13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13
db $13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$14,$14,$14
db $14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14
db $14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14
db $14,$14,$14,$14,$14,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15
db $15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15
db $15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$16
db $16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16
db $16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16
db $16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$16,$17,$17,$17,$17,$17
db $17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17
db $17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17,$17
db $17,$17,$17,$17,$17,$17,$17,$17,$17,$18,$18,$18,$18,$18,$18,$18
db $18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18
db $18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18,$18
db $18,$18,$18,$18,$18,$18,$18,$18,$18,$19,$19,$19,$19,$19,$19,$19
db $19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19
db $19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19
db $19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$1A,$1A,$1A,$1A,$1A
db $1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A
db $1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A
db $1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1B
db $1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B
db $1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B
db $1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B
db $1B,$1B,$1B,$1B,$1B,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C
db $1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C
db $1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C
db $1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1D,$1D,$1D
db $1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D
db $1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D
db $1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D
db $1D,$1D,$1D,$1D,$1D,$1D,$1D,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E
db $1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E
db $1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E
db $1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E
db $1E,$1E,$1E,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
db $1F,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
db $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
db $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
db $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
db $20,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21
db $21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21
db $21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21
db $21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21
db $21,$21,$21,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22
db $22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22
db $22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22
db $22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22
db $22,$22,$22,$22,$22,$22,$22,$23,$23,$23,$23,$23,$23,$23,$23,$23
db $23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23
db $23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23
db $23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23
db $23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$24,$24,$24
db $24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24
db $24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24
db $24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24
db $24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24
db $24,$24,$24,$24,$24,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25
db $25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25
db $25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25
db $25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25
db $25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$26
db $26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26
db $26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26
db $26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26
db $26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26
db $26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$26,$27,$27,$27,$27,$27
db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
db $27,$27,$27,$27,$27,$27,$27,$27,$27,$28,$28,$28,$28,$28,$28,$28
db $28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28
db $28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28
db $28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28
db $28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28,$28
db $28,$28,$28,$28,$28,$28,$28,$28,$28,$29,$29,$29,$29,$29,$29,$29
db $29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29
db $29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29
db $29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29
db $29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29
db $29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$29,$2A,$2A,$2A,$2A,$2A
db $2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A
db $2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A
db $2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A
db $2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A
db $2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2B
db $2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
db $2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
db $2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
db $2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
db $2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
db $2B,$2B,$2B,$2B,$2B,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
db $2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
db $2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
db $2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
db $2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
db $2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
db $2D,$2D,$2D,$2D,$2D,$2D,$2D,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
db $2E,$2E,$2E,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
db $2F,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30
db $30,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31
db $31,$31,$31,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32
db $32,$32,$32,$32,$32,$32,$32,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33
db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $34,$34,$34,$34,$34,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35
db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36
db $36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$36,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37,$37
db $37,$37,$37,$37,$37,$37,$37,$37,$37,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38,$38
db $38,$38,$38,$38,$38,$38,$38,$38,$38,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39
db $39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$39,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
db $3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
db $3B,$3B,$3B,$3B,$3B,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
db $3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
db $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
db $3E,$3E,$3E,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
db $3F,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40
db $40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40
db $40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40
db $40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40

SQUARE_DATA:
dw $0000,$0001,$0004,$0009,$0010,$0019,$0024,$0031
dw $0040,$0051,$0064,$0079,$0090,$00A9,$00C4,$00E1
dw $0100,$0121,$0144,$0169,$0190,$01B9,$01E4,$0211
dw $0240,$0271,$02A4,$02D9,$0310,$0349,$0384,$03C1
dw $0400,$0441,$0484,$04C9,$0510,$0559,$05A4,$05F1
dw $0640,$0691,$06E4,$0739,$0790,$07E9,$0844,$08A1
dw $0900,$0961,$09C4,$0A29,$0A90,$0AF9,$0B64,$0BD1
dw $0C40,$0CB1,$0D24,$0D99,$0E10,$0E89,$0F04,$0F81
dw $1000,$1081,$1104,$1189,$1210,$1299,$1324,$13B1
dw $1440,$14D1,$1564,$15F9,$1690,$1729,$17C4,$1861
dw $1900,$19A1,$1A44,$1AE9,$1B90,$1C39,$1CE4,$1D91
dw $1E40,$1EF1,$1FA4,$2059,$2110,$21C9,$2284,$2341
dw $2400,$24C1,$2584,$2649,$2710,$27D9,$28A4,$2971
dw $2A40,$2B11,$2BE4,$2CB9,$2D90,$2E69,$2F44,$3021
dw $3100,$31E1,$32C4,$33A9,$3490,$3579,$3664,$3751
dw $3840,$3931,$3A24,$3B19,$3C10,$3D09,$3E04,$3F01
dw $4000,$4101,$4204,$4309,$4410,$4519,$4624,$4731
dw $4840,$4951,$4A64,$4B79,$4C90,$4DA9,$4EC4,$4FE1
dw $5100,$5221,$5344,$5469,$5590,$56B9,$57E4,$5911
dw $5A40,$5B71,$5CA4,$5DD9,$5F10,$6049,$6184,$62C1
dw $6400,$6541,$6684,$67C9,$6910,$6A59,$6BA4,$6CF1
dw $6E40,$6F91,$70E4,$7239,$7390,$74E9,$7644,$77A1
dw $7900,$7A61,$7BC4,$7D29,$7E90,$7FF9,$8164,$82D1
dw $8440,$85B1,$8724,$8899,$8A10,$8B89,$8D04,$8E81
dw $9000,$9181,$9304,$9489,$9610,$9799,$9924,$9AB1
dw $9C40,$9DD1,$9F64,$A0F9,$A290,$A429,$A5C4,$A761
dw $A900,$AAA1,$AC44,$ADE9,$AF90,$B139,$B2E4,$B491
dw $B640,$B7F1,$B9A4,$BB59,$BD10,$BEC9,$C084,$C241
dw $C400,$C5C1,$C784,$C949,$CB10,$CCD9,$CEA4,$D071
dw $D240,$D411,$D5E4,$D7B9,$D990,$DB69,$DD44,$DF21
dw $E100,$E2E1,$E4C4,$E6A9,$E890,$EA79,$EC64,$EE51
dw $F040,$F231,$F424,$F619,$F810,$FA09,$FC04,$FE01


MainEnd: