;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Smoke Sprite High Bytes
;
; This patch adds a high byte for the X and Y positions of smoke sprites. This
; will allow you to easily utilize smoke sprites without having them possibly
; appear onscreen when the sprites are actually supposed to be offscreen. No
; freespace is required for this patch, but two sets of 4 consecutive bytes of
; free RAM are required.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!high_x			= $18C9			; 4 consecutive bytes
!high_y			= $18C5			; 4 consecutive bytes

header
lorom

org $00FD7E
		LDA $9B
		STA !high_x,y
		LDA $99
		STA !high_y,y
		LDA $1933
		BEQ +
		LDA $9A
		SEC
		SBC $26
		AND #$F0
		STA $17C8,y
		LDA $9B
		SBC $27
		STA !high_x,y
		LDA $98
		SEC
		SBC $28
		AND #$F0
		JSL mod_00FD7E
	+	LDA #$10
		STA $17CC,y
		RTS
		
org $00FE8E
		JSL mod_00FE8E
		RTS
		
org $018078
		LDA $E4,x
		CLC
		ADC $00
		STA $17C8,y
		LDA $14E0,x
		ADC #$00
		JSL mod_018078
		RTS
		
org $01AB8D
		LDA #$08
		JSL mod_01AB8D
		PLY
		RTL
		
org $01ABC5
		JSL mod_01ABC5
		PLY
		RTL
		
org $01BDAD
		LDA #$1B
		JSL mod_01BDAD
		RTS
		
org $01C505
		LDA #$10
		JSL mod_01C505
		RTS
		
org $01C5E6
		JSL mod_01C5E6
		RTL
		
org $029AEA
		LDA $17E0,x
		STA $17C8,y
		LDA $17EC,x
		STA !high_x,y
		LDA $17D4,x
		STA $17C4,y
		LDA $17E8,x
		STA !high_y,y
		JSL mod_029AEA
		RTS
		
org $02A438
		JSL mod_02A438
		NOP
		
org $02B513
		JSL mod_02B513
		RTS
		
org $02B969	
		LDA #$01
		STA $17C0,y
		LDA #$0F
		STA $17CC,y
		LDA $E4,x
		PHY
		LDY $157C,x
		CLC
		ADC $B94E,y
		PLY
		STA $17C8,y
		LDA $14E0,x
		PHY
		LDY $157C,x
		ADC $B950,y
		PLY
		STA !high_x,y
		
		LDA $D8,x
		STA $17C4,y
		LDA $14D4,x
		STA !high_y,y
		RTS
		
org $038A2E
		LDA $14E0,x
		ADC #$00
		STA !high_x,y
		JSL mod_038A2E
		RTS
		
org $0296C0
		STZ $0F
		REP #$30
		TXA
		AND #$00FF
		TAX
		SEP #$30
		
		LDA $17CC,x
		BEQ kill
		LDA $17C0,x
		BMI dec
		LDA $9D
		BNE no_dec
	dec:	DEC $17CC,x
		
	no_dec:	LDA $17C0,x
		AND #$7F
		JSL $0086DF
		
		dw return
		dw puff
		dw contact
		dw skid
		dw return
		dw glitter
		
oam_1:
		dw $0020,$0024,$0028,$002C
oam_2:
		dw $0090,$0094,$0098,$009C
oam_3:
		dw $0190,$0194,$0198,$019C

kill:		STZ $17C0,x
return:		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

puff_tile:
		db $66,$66,$64,$62,$60,$62,$60

puff:		REP #$30
		PHX
		TXA
		ASL
		TAX
		SEP #$20
		LDY oam_1,x
		LDA $A5
		CMP #$A9
		BEQ +
		BIT $0D9B
		BVC +
		LDY oam_3,x
		
	+	PLX
		
		JSR check_y
		BCS .return
		STA $0201,y
		
		LDA $17CC,x
		PHX
		LSR
		LSR
		TAX
		LDA puff_tile,x
		STA $0202,y
		PLX
		
		LDA $64
		STA $0203,y
		
		JSR check_x
		STA $0200,y
		BCC .on_scr
		INC $0F
		
	.on_scr	REP #$20
		TYA
		LSR
		LSR
		TAY
		SEP #$20
		LDA $0F
		ORA #$02
		STA $0420,y
		
	.return	SEP #$10
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

contact_tiles:
		db $7C,$7D,$7D,$7C

contact:	REP #$10
		LDY #$00F0
		BIT $0D9B
		BVC +
		LDA $0D9B
		DEC
		BEQ +
		LDY #$0190
		
	+	JSR check_y
		BCS .return
		STA $0201,y
		STA $0205,y
		CLC
		ADC #$08
		STA $0209,y
		STA $020D,y
		
		LDA $17CC,x
		AND #$02
		PHX
		TAX
		LDA contact_tiles,x
		STA $0202,y
		STA $020E,y
		INX
		LDA contact_tiles,x
		STA $0206,y
		STA $020A,y
		PLX
		
		LDA $17CC,x
		ASL
		ASL
		ASL
		ASL
		ASL
		AND #$40
		ORA $64
		STA $0203,y
		STA $0207,y
		EOR #$C0
		STA $020B,y
		STA $020F,y
		
		JSR check_x
		STA $0200,y
		STA $0208,y
		PHP
		CLC
		ADC #$08
		PLP
		STA $0204,y
		STA $020C,y
		BCC .on_scr
		INC $0F
		
	.on_scr	REP #$20
		TYA
		LSR
		LSR
		TAY
		SEP #$20
		LDA $0F
		STA $0420,y
		STA $0421,y
		STA $0422,y
		STA $0423,y
		
	.return	SEP #$10
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
skid_tile:
		db $66,$66,$64,$62,$62
		
skid:		LDA $9D
		BNE .no_dec
		LDA $17CC,x
		AND #$07
		BNE .no_dec
		LDA $17C4,x
		SEC
		SBC #$01
		STA $17C4,x
		BCS .no_dec
		DEC !high_y,x
	
	.no_dec	REP #$30
		PHX
		TXA
		ASL
		TAX
		SEP #$20
		LDY oam_1,x
		LDA $A5
		CMP #$A9
		BEQ .use_1
		LDA $140F
		BNE .use_1
		LDA $0D9B
		BPL .use_1
		CMP #$C1
		BEQ .use_2
		AND #$40
		BNE .use_3
	.use_2	LDY oam_2,x
		BRA .use_1
	.use_3	LDY oam_3,x
	.use_1	PLX
		
		JSR check_y
		BCS .return
		STA $0201,y
		
		LDA $17CC,x
		LSR
		LSR
		PHX
		TAX
		LDA skid_tile,x
		STA $0202,y
		PLX
		
		LDA $64
		STA $0203,y
		
		JSR check_x
		STA $0200,y
		BCC .on_scr
		INC $0F
		
	.on_scr	REP #$20
		TYA
		LSR
		LSR
		TAY
		SEP #$20
		LDA $0F
		STA $0420,y
		
	.return	SEP #$10
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
glitter_x:
		db $04,$08,$04,$00
		
glitter_y:
		db $FC,$04,$0C,$04
		
glitter:	LDA $17CC,x
		AND #$03
		BNE .return
		
		JSR check_x
		BCS .return
		JSR check_y
		BCS .return
		
		LDY #$0B
	.loop	LDA $17F0,y
		BEQ .found
		DEY
		BPL .loop
		LDY $185D
		DEY
		BPL .found
		LDY #$0B
		
	.found	LDA #$02
		STA $17F0,y
		
		LDA $17C8,x
		STA $00
		LDA $17C4,x
		STA $01
		
		LDA $17CC,x
		LSR
		LSR
		AND #$03
		PHX
		TAX
		LDA $00
		CLC
		ADC glitter_x,x
		STA $1808,y
		LDA $01
		CLC
		ADC glitter_y,x
		STA $17FC,y
		PLX
		
		LDA #$17
		STA $1850,y
		
	.return	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

check_x:	LDA !high_x,x
		XBA
		LDA $17C8,x
		REP #$20
		SEC
		SBC $1A
		CMP #$0100
		SEP #$20
		RTS
		
check_y:	LDA !high_y,x
		XBA
		LDA $17C4,x
		REP #$20
		SEC
		SBC $1C
		CMP #$00E0
		SEP #$20
		RTS
		
mod_00FD7E:	STA $17C4,y
		LDA $99
		SBC $29
		STA !high_y,y
		RTL
		
mod_00FE8E:	LDA $97
		ADC #$00
		STA !high_y,y
		
		LDA $94
		CLC
		ADC #$04
		LDA $95
		ADC #$00
		STA !high_x,y
		
		LDA #$13
		STA $17CC,y
		RTL
		
mod_018078:
mod_038A2E:	STA !high_x,y
		
		LDA $D8,x
		CLC
		ADC $01
		STA $17C4,y
		LDA $14D4,x
		ADC #$00
		STA !high_y,y
		
		LDA #$13
		STA $17CC,y
		RTL

mod_01ABC5:	LDA #$00
		ADC $97
		STA !high_y,y
		LDA #$08
		STA $17CC,y
		
		LDA $95
		STA !high_x,y
		RTL
		
mod_01AB8D:
mod_01BDAD:	
mod_01C505:	STA $17CC,y
		
		LDA $D8,x
		STA $17C4,y
		LDA $14E0,x
		STA !high_x,y
		LDA $14D4,x
		STA !high_y,y
		RTL
		
mod_01C5E6:	LDA $97
		ADC #$00
		STA !high_y,y
		
		LDA $94
		STA $17C8,y
		LDA $95
		STA !high_x,y
		RTL
		
mod_029AEA:	LDA $17E4,x
		LSR
		BCC .return
		
		LDA $17E0,x
		SEC
		SBC $26
		STA $17C8,y
		LDA $17EC,x
		SBC $27
		STA !high_x,y
		
		LDA $17D4,x
		SEC
		SBC $28
		STA $17C4,y
		LDA $17E8,x
		SBC $29
		STA !high_y,y
		
	.return	LDA #$10
		STA $17CC,y
		RTL
		
mod_02A438:	LDA $1733,x
		STA !high_x,y
		LDA $1729,x
		STA !high_y,y
		
		LDA #$0A
		STA $17CC,y
		RTL
		
table_02B513:
		db $FF,$00
		
mod_02B513:	STA $17C8,y
		LDA table_02B513,x
		LDX $15E9
		ADC $17A3,x
		STA !high_x,y
		LDA $1793,x
		STA !high_y,y
		RTL
		
warnpc $0299D2