;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Tile-Generation Routine Expander, by imamelia
;;
;; This patch enables you to use more values of $9C.  Normally, only 00-1B are valid,
;; but with this, you can use all 256 possible values, 00-FF.  You simply have to set
;; the tile numbers for values 20-FF in the table at the end of the patch, and values
;; 1C-1F use RAM to determine which tiles to create.  (This is optional; you don't even
;; have to use 1C-1F if you don't want to.)
;;
;; Notes:
;;
;; -This is incompatible with Kaijyuu's Custom Bounce Sprites patch.  (His patch allows you
;; to generate any tile number anyway, albeit with a slightly longer code.)
;; -The actual tile generation routine is still at $00BEB0.
;; -The default freespace addresses are kind of weird...I put part of the code in bank 00,
;; in the extra space created by making part of the tile setup routine into a JSR, and part of
;; it in bank 03 (by default; you may change it).  The data for the tile numbers is in bank 05
;; by default.  You'll need 448, or 0x1C0, bytes of free space for the tile number data and
;; about 43, or 0x2B, bytes for the extra code.  If you change either of the default freespace
;; addresses to an address outside of banks 00-0F, you need to uncomment the corresponding
;; RATS tag.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

!Freespace1 = $03D6B0		; 43/0x2B bytes of free space
!Freespace2 = $058E40		; 448/0x1C0 bytes of free space

!RAM_TileNumber = $7F9C82		; 8 bytes of optional free RAM; you can use these to set any tile number
; The first pair of bytes is for value 1C, the second pair is for value 1D,
; the third pair is for value 1E, and the fourth pair is for value 1F.

org $00BFBE
JMP Main

org $00BFD9
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C099
dw $C096
dw $C096

org $00C077	; new routine takes up 31 bytes
JSR Setup
AND.w #$00FF
TAX
SEP #$20
LDA [$6E],y
AND #$FE
STA [$6E],y
LDA $00C06B,x
STA [$6B],y
REP #$20
AND #$00FF
ASL
TAY
JMP $C0FB

org $00C096	; was $00C0C1; shifted and reduced to 41 bytes
JSR $C00D	;
JSR Setup
SEC
SBC.w #$0009
AND.w #$00FF
TAX
SEP #$20
LDA [$6E],y
ORA #$01
STA [$6E],y
LDA OriginalPage1Tiles,x
STA [$6B],y
REP #$20
AND.w #$00FF
ORA #$0100
ASL
TAY
JMP $C0FB

org $00C0BF	; code that is identical in the two variants of the tile-generation routine
Setup:		; requires 24 bytes
REP #$30
LDA $98
AND #$01F0
STA $04
LDA $9A
LSR #4
AND.w #$000F
ORA $04
TAY
LDA $9C
RTS

org $00C0D7	; 37 bytes to $00C0FB
Main:		;
LDA $9C		; check the tile generation number
CMP #$1C	; if 1C or greater...
BCS CustomTile	; then the tile number is a custom one
DEC A		; if not...
JMP $BFC1	; then just continue with the normal code

CustomTile:	;
JSR Setup		; set up the tile position
CMP.w #$0020	; if $9C is 1D-1F...
BCC Indirect	; get the tile number from RAM
SEC		;
SBC.w #$0020	;
ASL		; x2 because the tile numbers are 16-bit
TAX		;
JML LoadCustomNumber

Indirect:		;
SEC		;
SBC.w #$001C	;
JML IndirectTileRt	;

org !Freespace1

;db "STAR"		; the RATS tag is not necessary if you don't change the default freespace setting
;dw EndCode-StartCode-$01
;dw EndCode-StartCode-$01

StartCode:

LoadCustomNumber:

LDA.l CustomTiles,x	;
BRA GenerateNewTile	;

IndirectTileRt:

ASL			;
TAX			;
LDA !RAM_TileNumber,x	;

GenerateNewTile:

SEP #$20			;
STA [$6B],y		; store the low byte of the tile number
XBA			;
STA [$6E],y		; store the high byte
XBA			;
REP #$20			;
ASL			;
TAY			; get an index for the stripe image routine so that the graphics show up correctly
JML $00C0FB		;

OriginalPage1Tiles:		;
db $52,$1B,$23,$1E,$32,$13,$15,$16,$2B,$2C,$12,$68,$69,$32,$5E

EndCode:			;

org !Freespace2		;

;db "STAR"		; the RATS tag is not necessary if you don't change the default freespace setting
;dw EndData-StartData-$01
;dw EndData-StartData-$01

StartData:

CustomTiles:					;

dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 20-27
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 28-2F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 30-37
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 38-3F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 40-47
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 48-4F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 50-57
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 58-5F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 60-67
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 68-6F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 70-77
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 78-7F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 80-87
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 88-8F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 90-97
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values 98-9F
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values A0-A7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values A8-AF
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values B0-B7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values B8-BF
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values C0-C7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values C8-CF
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values D0-D7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values D8-DF
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values E0-E7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values E8-EF
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values F0-F7
dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000	; values F8-FF

EndData:

