;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Timer Patch 1.3, by imamelia
;;
;; This patch will install a timer on your status bar, one that
;; displays minutes, seconds, and frames rather than the simple
;; unit timer in the original SMW.  You can set, on a per-level basis,
;; how much time should be on the timer at the start of the level,
;; whether the timer should go up or down, whether it should reset,
;; and more.  You can also make it activate customizable subroutines
;; when the time reaches a certain point.
;;
;; See the note below "Settings:", the first table at the end; this will
;; give you information about how to set up the timer.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
lorom

!Freespace = $1D8000	; free space where the main code will be placed

!ShowClock = 1		; show the clock tile or not?
!ShowFrames = 1		; show the frames or not?

!SBPos = $0EF9				; the first tile on the status bar that the timer uses

!RAM_Minutes = $7FA116		; used for the minutes*
!RAM_Seconds = $7FA117		; used for the seconds*
!RAM_Frames = $7FA118		; used for the frames*
!RAM_CustPtrLo = $7FA119		; low byte of an optional 24-bit pointer
!RAM_CustPtrHi = $7FA11A		; high byte of an optional 24-bit pointer
!RAM_CustPtrBnk = $7FA11B		; bank byte of an optional 24-bit pointer
!RAM_CriticalM = $7FA11C		; when the minutes reach this value...
!RAM_CriticalS = $7FA11D		; and the seconds reach this value...
						; a subroutine will be activated
!RAM_TFlags = $7FA11E			; if bit 0 of this is set, the timer is frozen (i.e. it will
						; not increment or decrement), if bit 1 of this
						; is set, a subroutine has already been activated
						; and need not be again, and if bit 2 is set, the timer
						; has already been reset at the beginning of a level
						; and need not be again.  Bits 3-7 are unused,
						; but may be used in the future.
!RAM_TeleLevel = $7FA11F		; the level to teleport to, when set (16-bit)
;
; This is the format for a custom subroutine.  It is commented out, obviously,
; because it is not necessary to have it taking up extra space in the patch.
;
; LDA.b #DoStuff/$10000
; STA !RAM_CustPtrBnk
; LDA.b #DoStuff/$100
; STA !RAM_CustPtrHi
; LDA.b #DoStuff
; STA !RAM_CustPtrLo
;
; DoStuff:
; LDA #$13
; STA $9C
; JSL $00BEB0
; RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; level number patch (the same one used by LevelASM etc.)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

namespace "Timer"

org $05D8B9
JSR LevelNumMain

org $05DC46
LevelNumMain:
LDA $0E
STA $010B
ASL
RTS

org $00A5E8
JSL TimerInit
NOP
NOP

org $00A2E6
JSL TimerMain

org !Freespace

reset bytes

db "STAR"
dw End-Start-$01
dw End-Start-$01^$FFFF

Start:

macro ShowClock()
rep !ShowClock-1 : LDA #$76		; load the clock tile
rep !ShowClock-1 : STA !SBPos,x	; and store it to the first status bar slot
rep !ShowClock-1 : INX			; increment X once so we can get to the next status bar slot
endmacro

macro ShowFrames()
rep !ShowFrames-1 : STA !SBPos+5,x	; store the semicolon to the second slot
rep !ShowFrames-1 : LDA !RAM_Frames	; load the frames
rep !ShowFrames-1 : JSL HexToDecShort	; hexadecimal-to-decimal conversion subroutine
rep !ShowFrames-1 : STA !SBPos+7,x	; store the ones digit one slot down
rep !ShowFrames-1 : TYA			; transfer Y to A
rep !ShowFrames-1 : STA !SBPos+6,x	; store the tens digit
endmacro

macro CheckZero()
LDA !RAM_Minutes				; if the minutes,
ORA !RAM_Seconds				; seconds,
rep !ShowFrames-1 : ORA !RAM_Frames	; and frames are all zero...
endmacro

macro CheckZero2()
rep !ShowFrames-1 : LDA !RAM_Frames	; check the frames
rep !ShowFrames-1 : BNE EndTimeSet	; they have to be 00 for the subroutine to activate
endmacro

; These are the subroutines that will be activated when the timer reaches a
; critical number.  Subroutine 0 simply returns, subroutines 1-6 are common
; and well-known subroutines, and subroutine 7 is determined by a 24-bit
; pointer beginning at !RAM_CustPtrLo.  In fact, by manipulating this pointer,
; you can even string together several subroutines and critical numbers.  You
; can also change subroutines 1-6 if you don't like the defaults.  If you are
; REALLY feeling tricky, you can even change all bit checks for the settings
; bytes and use bits 3 and 4 for subroutines as well.  Since bits 6 and 7 are
; unused, you could potentially move bits 3-5 up two bits and have a total
; of 32 subroutines instead of just 8.

SubroutinePointers:

dw EndTimeSet		; 00: do nothing
dw KillPlayer		; 01: kill the player
dw HurtPlayer		; 02: hurt the player
dw Teleport00		; 03: teleport to the level set by the screen exit of level 00
dw TeleportCust		; 04: teleport to the level set by !RAM_TeleLevel
dw BlueSwitch		; 05: activate the blue P-switch
dw GraySwitch		; 06: activate the gray P-switch
dw CustomRoutine	; 07: activate a custom routine set by !RAM_CustPtr

FreezeActions:		; which player actions freeze the timer (00 = let it run, 01-FF - pause it)
db $00,$00,$00,$00,$00,$01,$01,$00,$00,$01,$01,$01,$01,$01

TimerMain:

JSL $828AB1	; restore hijacked code
PHB
PHK
PLB
JSR TimerMainRt	; jump to the timer routine
PLB
RTL			; and return
; You can actually add more JSRs before the second PLB for more routines
; that you want to run every frame in a level, if you like.

TimerInit:

LDX $0701		; restore
STX $0903		; hijacked code
SEP #$30		; restore A, X, and Y to 8-bit
PHB
PHK
PLB
JSR TimerInitRt	;
REP #$10		; 16-bit XY
LDY $010B		; load the level number into Y
LDA Settings,y	; check the current level settings
AND #$10		; if bit 4 is not set...
BEQ NoTimer2	; the current level does not have a timer
JSR StatusShow	;
NoTimer2:
PLB
REP #$30		; we had 16-bit AXY before, so we need to keep it that way
RTL
; You can actually add more JSRs before the second PLB for more routines
; that you want to run every frame right before the start of a level, if you like.

TimerInitRt:

REP #$10			; 16-bit XY
LDY $010B			; get the level number into Y
LDA Settings,y		; check the settings
AND #$60			; check bits 5 and 6
CMP #$20			; if set to 01, reset the timer
BEQ .Reset			;
CMP #$40			; if set to 10, check if any sublevels have been entered
BNE .NoReset		; if not, don't reset the timer
LDA $141A			;
BNE .NoReset		;
.Reset			;
LDA TblMinutes,y		; load the minutes
STA !RAM_Minutes		; and store to the timer RAM
LDA TblSeconds,y		; load the seconds
STA !RAM_Seconds	; and store to the timer RAM
.NoReset			;
LDA ICVM,y			; load the initial critical values for minutes
STA !RAM_CriticalM	; and store to RAM
LDA ICVS,y			; load the initial critical values for seconds
STA !RAM_CriticalS	; and store to RAM
LDA #$00			;
STA !RAM_Frames		; set the frames to zero
STA !RAM_TFlags		; clear all flags
RTS				; end timer initialization

TimerMainRt:

LDA $1493			; if the level is ending...
BNE .Freeze			;
LDY $71			;
LDA FreezeActions,y	; or the player is performing certain actions...
BNE .Freeze			; then freeze the timer
.NoFreeze			;
LDA !RAM_TFlags		;
AND #$FE			;
STA !RAM_TFlags		;
BRA .Skip			;
.Freeze			;
LDA !RAM_TFlags		;
ORA #$01			;
STA !RAM_TFlags		;
.Skip				;

REP #$10			; 16-bit XY
LDY $010B			; load the level number into Y
LDA Settings,y		; check the current level settings
AND #$10			; if bit 4 is not set...
BEQ .NoTimer		; the current level does not have a timer
JSR SetTime			; set the time on the timer
JSR StatusShow		; show the timer on the status bar
.NoTimer			;
SEP #$10			;
RTS				;

StatusShow:			;

SEP #$10			; no need for X or Y to be 16-bit here

PHX				; preserve X
PHY				; and Y

LDX #$00			; start X at zero
%ShowClock()		; this macro will not be run if the clock is not supposed to be shown

SEP #$10			; 8-bit AXY temporarily

LDA !RAM_Minutes	; load the minutes
JSL HexToDecShort	; hexadecimal-to-decimal conversion subroutine
STA !SBPos+1,x		; store the ones digit one slot down
TYA				; transfer Y to A
STA !SBPos,x		; store the tens digit
LDA !RAM_Seconds	; load the seconds
JSL HexToDecShort	; hexadecimal-to-decimal conversion subroutine
STA !SBPos+4,x		; store the ones digit one slot down
TYA				; transfer Y to A
STA !SBPos+3,x		; store the tens digit

LDA #$78			; store the semicolon tile
STA !SBPos+2,x		; to the first position
%ShowFrames()		;

PLY
PLX
RTS

SubroutineSetJ:			;
JMP SetTime_SubroutineSet	;

SetTime:

LDA !RAM_TFlags		; check the flag RAM
AND #$01			; if bit 0 is set...
BNE SubroutineSetJ	; don't increment or decrement the timer

REP #$10			; 16-bit XY
LDY $010B			; load the level number
LDA Settings,y		; check the settings
AND #$08			; check bit 3 to determine whether the timer should go up or down
BNE .Increasing		;

%CheckZero()		; if the minutes, seconds, and frames (if shown) are all zero...
BEQ .SubroutineSet	; don't decrement anything and go directly to the subroutines

LDA !RAM_Frames		;
DEC				;
STA !RAM_Frames		; decrement the frames
BMI .ResetFrames		; if the frames have overflowed below zero...
BRA .SubroutineSet	;

.ResetFrames		; then set them back to 59
LDA #$3B			;
STA !RAM_Frames		;

LDA !RAM_Seconds	;
DEC				;
STA !RAM_Seconds	; and decrement the seconds
BMI .ResetSeconds		; if the seconds have overflowed below zero...
BRA .SubroutineSet	;

.ResetSeconds		; then set them back to 59
LDA #$3B			;
STA !RAM_Seconds	;

LDA !RAM_Minutes	;
DEC				;
STA !RAM_Minutes		; and decrement the minutes
BRA .SubroutineSet	; no need to check for overflow here

.Increasing

LDA #$3B			;
CMP !RAM_Frames	; if the frames,
BNE .IncFrames		;
CMP !RAM_Seconds	; seconds,
BNE .IncFrames		;
CMP !RAM_Minutes	; and minutes are all 59...
BNE .IncFrames		;
BRA .SubroutineSet	; then don't increment them

.IncFrames

LDA !RAM_Frames		;
INC				; increment the frames
CMP #$3C			; if the frames have reached 60...
BEQ .IncSeconds		; then reset the frames and increment the seconds
STA !RAM_Frames		; if not, just increment the frames
BRA .SubroutineSet	;

.IncSeconds			;

LDA #$00			;
STA !RAM_Frames		; reset the frames to zero
LDA !RAM_Seconds	;
INC				; increment the seconds
CMP #$3C			; if the seconds have reached 60...
BEQ .IncMinutes		; then reset the seconds and increment the minutes
STA !RAM_Seconds	; if not, just increment the seconds
BRA .SubroutineSet	;

.IncMinutes			;

LDA #$00			;
STA !RAM_Seconds	; reset the seconds to zero
LDA !RAM_Minutes	;
INC				;
STA !RAM_Minutes		; increment the minutes

.SubroutineSet		;

LDA !RAM_TFlags		; check the flag RAM
AND #$02			; if bit 1 is set...
BNE EndTimeSet		; then a subroutine has already been activated

%CheckZero2()		;
LDA !RAM_Seconds	; check the seconds
CMP !RAM_CriticalS	; if the seconds have not reached their "critical point"...
BNE EndTimeSet		; don't activate a subroutine
LDA !RAM_Minutes	; check the minutes
CMP !RAM_CriticalM	; if the minutes have not reached their "critical point"...
BNE EndTimeSet		; don't activate a subroutine

LDA !RAM_TFlags		;
ORA #$02			; set bit 1 of the flag RAM,
STA !RAM_TFlags		; indicating that the subroutine has already been activated

LDA Settings,y		; Y still has the level number
SEP #$10			; set Y back to 8-bit
AND #$07			; get bits 0-2, which determine which subroutine to activate

PHX				; 16-bit pointer subroutine
ASL A
TAX
REP #$20
LDA SubroutinePointers,x
STA $0A
SEP #$20
PLX
JMP ($000A)

EndTimeSet:		;
SEP #$10			; set Y back to 8-bit if it hasn't been done already
RTS

HexToDecShort:

LDY #$00
StartCompare1:
CMP #$0A
BCC ReturnHTD1
SBC #$0A
INY
BRA StartCompare1
ReturnHTD1:
RTL

KillPlayer:
JSL $80F606		; kill the player
LDA !RAM_TFlags	;
ORA #$01		; freeze
STA !RAM_TFlags	; the timer
RTS			;

HurtPlayer:		;
JSL $80F5B7
RTS

Teleport00:

LDA $19B8			; screen exit of screen 00, low byte
STA $00			; store for later
LDA $19D8			; screen exit of screen 00, high byte
STA $01			; store for later
BRA TeleportMain		;

TeleportCust:		;
REP #$20			; set A to 16-bit
LDA !RAM_TeleLevel	; load the free RAM that determines which level to teleport to
SEP #$20			; set A back to 8-bit
STA $00			; store the low byte
XBA				; switch the bytes of A...
STA $01			; and store the high byte

TeleportMain:	;

PHX
LDA $5B		; check whether the level is horizontal or vertical
BNE .Vertical	;
LDX $95		; if the level is horizontal, use the player's X position
BRA .ExitIndex	;
.Vertical		;
LDX $97		; if the level is horizontal, use the player's X position
.ExitIndex		;
LDA $00		; screen exit low byte
STA $19B8,x	; set the current screen exit to that value
LDA $01		; screen exit low byte
ORA #$04		; exit present flag
STA $19D8,x	; set the current screen exit to that value
LDA #$06		;
STA $71		; set teleporting action
STZ $88		; some settings
STZ $89		; related to motion and entering a pipe
LDA !RAM_TFlags	;
ORA #$01		; freeze
STA !RAM_TFlags	; the timer
PLX
RTS

BlueSwitch:		;

LDA #$B0		; blue switch timer, default value
STA $14AD		; RAM address for the blue P-switch timer
BRA SwitchMusic	;

GraySwitch:		;

LDA #$B0		; gray switch timer, default value
STA $14AE		; RAM address for the gray P-switch timer

SwitchMusic:	;

LDA $0DDA		;
BMI .NoSetMusic	; check to see if the music is "abnormal"
LDA #$0E		;
STA $1DFB		; set the P-switch music
.NoSetMusic	;
RTS			;

CustomRoutine:		; if the code gets here, then we want a custom subroutine

LDA !RAM_CustPtrLo	;
STA $00			; set the low byte of the pointer
LDA !RAM_CustPtrHi	;
STA $01			; set the high byte of the pointer
LDA !RAM_CustPtrBnk	;
STA $02			; set the bank byte of the pointer
PHK				; push a 24-bit
PEA.w .Return-$01	; return address
JML [$0000]			; and jump to the address loaded into the pointer
.Return			;
RTS

Settings:

; These are in bitwise format.
; Bits 0-2: These determine which subroutine to activate.
;	000 - nothing
;	001 - kill the player
;	010 - hurt the player
;	011 - teleport to the screen exit of level 00
;	100 - teleport to the level set by !RAM_TeleLevel
;	101 - activate the blue P-switch
;	110 - activate the silver P-switch
;	111 - jump to a custom subroutine set by !RAM_CustPtr
; Bit 3: This determines whether the timer should go up or down.
;	0 - down
;	1 - up
; Bit 4: This determines whether the timer is actually present or not.
;	0 - absent
;	1 - present (must be set in all levels that use the timer)
; Bits 5-6: These determine whether the timer resets at the beginning of the level.
;	00 - don't reset
;	01 - reset
;	10 - reset only if no screen exits have been taken
;	11 - unused (currently does the same thing as 00)
; Bit 7: Currently unused.

db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 0-F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 10-1F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 20-2F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 30-3F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 40-4F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 50-5F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 60-6F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 70-7F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 80-8F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 90-9F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; A0-AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; B0-BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; C0-CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; D0-DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; E0-EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; F0-FF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 100-10F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 110-11F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 120-12F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 130-13F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 140-14F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 150-15F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 160-16F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 170-17F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 180-18F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 190-19F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1A0-1AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1B0-1BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1C0-1CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1D0-1DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1E0-1EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1F0-1FF

TblMinutes:	; the number of minutes on the timer at the start of each level

db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 0-F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 10-1F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 20-2F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 30-3F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 40-4F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 50-5F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 60-6F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 70-7F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 80-8F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 90-9F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; A0-AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; B0-BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; C0-CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; D0-DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; E0-EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; F0-FF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 100-10F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 110-11F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 120-12F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 130-13F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 140-14F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 150-15F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 160-16F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 170-17F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 180-18F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 190-19F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1A0-1AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1B0-1BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1C0-1CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1D0-1DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1E0-1EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1F0-1FF

TblSeconds:

db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 0-F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 10-1F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 20-2F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 30-3F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 40-4F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 50-5F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 60-6F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 70-7F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 80-8F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 90-9F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; A0-AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; B0-BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; C0-CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; D0-DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; E0-EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; F0-FF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 100-10F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 110-11F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 120-12F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 130-13F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 140-14F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 150-15F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 160-16F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 170-17F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 180-18F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 190-19F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1A0-1AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1B0-1BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1C0-1CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1D0-1DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1E0-1EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1F0-1FF

ICVM:	; "Initial Critical Values, Minutes" - set at the start of the level

db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 0-F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 10-1F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 20-2F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 30-3F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 40-4F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 50-5F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 60-6F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 70-7F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 80-8F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 90-9F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; A0-AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; B0-BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; C0-CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; D0-DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; E0-EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; F0-FF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 100-10F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 110-11F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 120-12F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 130-13F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 140-14F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 150-15F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 160-16F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 170-17F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 180-18F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 190-19F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1A0-1AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1B0-1BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1C0-1CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1D0-1DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1E0-1EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1F0-1FF

ICVS:	; "Initial Critical Values, Seconds" - set at the start of the level

db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 0-F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 10-1F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 20-2F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 30-3F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 40-4F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 50-5F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 60-6F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 70-7F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 80-8F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 90-9F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; A0-AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; B0-BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; C0-CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; D0-DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; E0-EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; F0-FF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 100-10F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 110-11F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 120-12F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 130-13F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 140-14F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 150-15F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 160-16F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 170-17F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 180-18F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 190-19F
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1A0-1AF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1B0-1BF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1C0-1CF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1D0-1DF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1E0-1EF
db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 1F0-1FF

print "Freespace used: ",bytes," bytes."
print "Next address: $",pc,".

End:







