incsrc SpriteXkasDefines.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; YI Egg Sprites
; By Sonikku and Jimmy52905
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; YIEgg.cfg -
;
; 	Description: This is the sprite that is thrown be Yoshi. It bounces off
;	walls and changes color like it does in YI. After a while though, it
;	will crack. If sprite buoyancy is enabled, the sprite will move slower
;	through water and skid across the top of it. It will also open
;	? blocks, turn blocks, and the YI cloud sprite when it hits them. It
;	hurts enemies and will generate either a coin, 2 stars that give Yoshi
;	health, or nothing depending on what color it is when it hits them.
;	These can also be grabbed with Yoshi's tongue.
;
; YIEgg_Collectable.cfg -
;
; 	Description: This is the sprite that can actually be collected by Yoshi.
;	Depending on the X position it's placed in LM, it will either be: Green,
;	Green, Yellow, or Red, respectively. Yoshi can only carry up to 6 of
;	these, as in YI, and these can also be grabbed with Yoshi's tongue.
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!YCOIN   =	$21	; sprite number of yellow coin
!STARNUM =	$01	; Number of stars to generate when red egg hits enemy - 1 (default is 2 stars)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT and MAIN JSL targets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
	PHB
	PHK				
	PLB				
	LDA $E4,x	; Sprite X position
	LSR A
	LSR A
	LSR A
	LSR A		; Divide by $10 to get tile coordinate
	AND #$03	; Read from only 4 table values
	TAY
	LDA INITPALETTE,y
	STA $C2,x	; Egg palette status address
	PLB
	RTL         

INITPALETTE:	db $00,$00,$01,$02	; Changes depending on X coordinate placed in LM: green, green, yellow, red

print "MAIN ",pc			
	PHB
	PHK				
	PLB				
	JSR SPRITE_ROUTINE	
	PLB
	RTL     

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPRITE_ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PALETTE:	db $0A,$04,$08,$04,$08,$04,$08,$04,$04
STATE:		db $08,$08,$08,$08,$08,$08,$08,$08,$02
EggPropertyTBL: db $00,$01,$02,$01,$02,$01,$02,$01,$01
RETURN:		RTS
SPRITE_ROUTINE:
LDA $14D4,x
BPL notoffscreen:
LDA $D8,x
CMP #$80
BCS notoffscreen:
STZ $14C8,x
notoffscreen:
	JSR SUB_OFF_SCREEN_X1
	JSR SUB_GFX
	LDA $9D
	BNE RETURN
	LDA $7FAB28,x
	AND #$01
	BNE NO_NON_COLLECT

	JMP NON_COLLECT

NO_NON_COLLECT:
	JSL $01802A

	JSR EDIBLE_TIMER

	LDA $151C,x
	BNE NO_MOVEMENT

	LDA $1588,x
	AND #$03
	BNE NO_X_SPEED
	LDA $1588,x
	AND #$0C
	BEQ NO_VERT
NO_MOVEMENT:
	STZ $AA,x
NO_X_SPEED:
	STZ $B6,x
NO_VERT:
	JSL $01A7DC
	BCC RETURN_COLLECT
	STZ $14C8,x
	LDA #$0E
	STA $1DF9

	PHX
	LDA #$05
	SEC
	SBC !FREERAM1
	TAX
	LDA !FREERAM4,x
	TAX
	LDA PALETTE,x
	STA $00
	PLX

	LDA !FREERAM4+$4
	STA !FREERAM4+$5
	LDA !FREERAM4+$3
	STA !FREERAM4+$4
	LDA !FREERAM4+$2
	STA !FREERAM4+$3
	LDA !FREERAM4+$1
	STA !FREERAM4+$2
	LDA !FREERAM4
	STA !FREERAM4+$1
	PHX
	LDA $C2,x
	TAX
	LDA EggPropertyTBL,x
	PLX
	STA !FREERAM4	; Shift ram table

	REP #$20
	LDA #$0018
	STA !FREERAM9	; Egg rotating timer.
	SEP #$20

	;LDA !FREERAM2
	;STA $0EF9
	LDA #$05
	CMP !FREERAM2
	BCC NO_RETURN_COLLECT
	INC !FREERAM2
	BRA RETURN_COLLECT

NO_RETURN_COLLECT:
	PHX
	JSR NoAddEgg5
	PLX

RETURN_COLLECT:
	RTS
NON_COLLECT:
	LDA $14C8,x
	BNE NOTRETURN

	RTS

NOTRETURN:
	LDA $14C8,x
	CMP #$08
	BCS NORMAL
	STZ $1510,x
	STZ $14C8,x
	JSR SUB_SMOKE
	JSR SUB_SHELL_PIECE
	RTS
NORMAL:
	LDA $14
	AND #$03
	BNE NO_SMOKE
	JSR SUB_SMOKE
NO_SMOKE:
	LDA $1510,x		; Check if egg has hit water.
	BEQ NO_GRAVITY
	JSL $01802A
	BRA GRAVITY		; Sprite has gravity.
NO_GRAVITY:
	JSL $01801A
	JSL $018022
GRAVITY:
	LDA $5B
	PHA

	LDA $5B
	AND #$7F
	STA $5B

	JSL $019138

	STZ $01

	JSR CLOUDCHECKLOOP

	PLA
	STA $5B

	LDA $1588,x
	PHA
	LDA $15B8,x
	STA $00
	PHA
	LDA $01
	PHA

	JSL $019138

	PLA
	STA $01
	PLA
	STA $15B8,x
	PLA
	ORA $1588,x
	STA $1588,x

	JSR CLOUDCHECKLOOP
	JSR SLOPE_DETECTION

HORZ:
	LDA $1588,x
	AND #$03
	BEQ NO_HORZ

	LDY #$00
	LDA $18A7
	JSL $00F160	; ? block routine

	LDA #$01
	STA $1DF9
	STZ $1510,x		; Egg is no longer touching water.
	LDA $1594,x
	EOR #$7F
	INC A
	STA $1594,x		; Flip Angle horizontally
	JSR GET_ANGLE_SPEEDS

	LDA $1588,x
	AND #$0C
	BEQ HIT_STUFF

NO_HORZ:
	LDA $1588,x
	AND #$0C
	BEQ NO_VERT2
	AND #$08
	BEQ VERT3

	LDY #$00
	LDA $1868
	JSL $00F160	; ? block routine

VERT3:
	LDA #$01
	STA $1DF9
	STZ $1510,x		; Egg is no longer touching water.

	LDA $1594,x
	CLC
	ADC #$40
	EOR #$7F
	INC A
	SEC
	SBC #$40
	STA $1594,x		; Flip Angle vertically
	JSR GET_ANGLE_SPEEDS
HIT_STUFF:
	INC $C2,x
	LDY $C2,x
	LDA STATE,y
	STA $14C8,x
NO_VERT2:
	LDY #$0B
LOOP:
	CPY #$00
	BEQ NO_SPR_CON
	DEY
	STX $06
	CPY $06
	BEQ LOOP
	LDA $14C8,y
	CMP #$08
	BCC LOOP
	LDA $167A,y
	AND #$02
	BNE LOOP
	JSL $03B69F
	PHX
	TYX
	JSL $03B6E5
	PLX
	JSL $03B72B
	BCC LOOP
	LDA $1632,x
	CMP $1632,y
	BNE LOOP
	LDA #$02
	STA $14C8,x
	JSR SUB_SPRITE
	LDA #$01
	STA $1DF9
	LDA $166E,y
	AND #$10
	BNE NO_SPR_CON
	LDA $190F,y
	AND #$08
	BEQ FIRE_KILL
	LDA $1528,y
	CLC
	ADC #$01
	STA $1528,y
	LDA $1528,y
	CMP #$05
	BCC NO_SPR_CON
	LDA #$03
	BRA KILL_SPR
FIRE_KILL:
	LDA #$00
KILL_SPR:
	JSL $02ACE5
	LDA #$02
	STA $14C8,y
	LDA #$E0
	STA $00AA,y
NO_SPR_CON:
	RTS
SUB_SPRITE:
	LDA $C2,x
	JSL $0086DF
	dw GREEN
	dw YELLOW
	dw RED
	dw YELLOW
	dw RED
	dw YELLOW
	dw RED
	dw YELLOW
	dw GREEN
GREEN:	RTS
YELLOW:	LDA #!YCOIN
	STA $9E,x
	STA $7FAB9E,x
	JSL $07F7D2
	LDA #$08
	STA $14C8,x
	LDA $1632,y
	STA $1632,x
	PHX
	BRA SET_Y_SPEED
RED:	LDA #!StarSprite
	STA $7FAB9E,x
	LDA #$01
	STA $14C8,x
	JSL $07F7D2
	JSL $0187A7
	LDA #$08
	STA $7FAB10,x
	LDA #!STARNUM
	STA $151C,x
	LDA $1632,y
	STA $1632,x
	PHX
SET_Y_SPEED:
	LDA #$E0
	STA $AA,x
	JSR SUB_SMOKE
	PLX
RETURN_RED:
	RTS

XSPEEDTABLE:
db $E0,$20

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NoAddEgg5:
	LDY #$07
GenerateEggLoop:
	LDA $170B,y
	BEQ InitEggGen

	DEY
	BPL GenerateEggLoop

InitEggGenReturn:
	RTS

InitEggGen:
	LDA !FREERAM1
	CMP #$06
	BEQ InitEggGenReturn
	LDA $00
	ORA #$C0
	STA $1765,y	; Egg properties + egg sprite flag.
	LDA #$09
	STA $170B,y	; Egg sprite number.
	LDX $76
	LDA Egg_XSpeed,x
	STA $1747,y
	LDA #$D0
	STA $173D,y
	LDA #$00
	STA $176F,y

	LDA #$05
	SEC
	SBC !FREERAM1
	ASL A
	TAX

	REP #$20
	LDA table,x
	CLC
	ADC !FREERAM19	; Index offset
	CMP #$00C0
	BCC GetEggCoordIndex
	SEC
	SBC #$00C0
GetEggCoordIndex:
	TAX
	SEP #$20

	LDA !FREERAM6,x		;\
	STA $171F,y		; |
	LDA !FREERAM6+$1,x	; |
	STA $1733,y		; |
	LDA !FREERAM6+$2,x	; | Generate egg at the proper coordinates.
	STA $1715,y		; |
	LDA !FREERAM6+$3,x	; |
	STA $1729,y		;/
	RTS

Egg_XSpeed:
	db $10,$F0
table:
	dw $00A0,$0080,$0060,$0040,$0020,$0000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

EDIBLE_TIMER:
	LDA $1528,x
	BEQ EGG_IS_EDIBLE

	LDA $1686,x
	ORA #$01	; Make sprite temporarily inedible
	STA $1686,x

	DEC $1528,x

	BRA EGG_IS_UNEDIBLE

EGG_IS_EDIBLE:
	LDA $1686,x
	AND #$FE
	STA $1686,x	; Egg is edible again

EGG_IS_UNEDIBLE:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CLOUDCHECKLOOP:
	LDA $1862		; Check map16 high byte.
	BNE YES_TOUCHINGCLOUD
	LDA $18D7		; Check map16 high byte.
	BEQ NOT_TOUCHINGCLOUD

YES_TOUCHINGCLOUD:
	LDA $1693
	CMP #$11
	BCC TOUCHINGCLOUD
	CMP #$EB
	BEQ TOUCHINGCLOUD
	SEC
	SBC #$6E
	CMP #$85
	BCS NOT_TOUCHINGCLOUD

	TAY
	LDA TYPE_OF_SLOPE,y
	STA $15B8,x
	RTS

TOUCHINGCLOUD:
	LDA $AA,x
	BMI NOT_TOUCHINGCLOUD

	LDA $1588,x
	ORA #$04	; Sprite is blocked from bottom
	STA $1588,x

	RTS

NOT_TOUCHINGCLOUD:
	LDA $01
	BEQ ALSO_NOT_TOUCHINGCLOUD

	RTS

ALSO_NOT_TOUCHINGCLOUD:
	LDA $164A,x	; Check sprite follower table
	BEQ HORZ_2
	LDA $1693
	BEQ WATERCHECK
	LDA $1693
	CMP #$01
	BEQ WATERCHECK
	LDA $85
	BNE WATERCHECK2	; "Water level" flag
	LDA $1693
	CMP #$03
	BEQ WATERCHECK2
	CMP #$02
	BNE HORZ_2	; Check if tile acts like 00 or 02, if it does, go to the water routine.

WATERCHECK2:
	LDA $1504,x	; In water flag
	BNE WATER_RETURN

	LDA #$01
	STA $1504,x	; Set "In water" flag

	INC $01

	JSR SLOWDOWN_IN_WATER

	RTS

WATERCHECK:
	LDA $1504,x
	BNE WATER_RETURN

	LDA $AA,x
	BMI WATERCHECK2
	CMP #$60
	BCS WATERCHECK2

	LDA #$01
	STA $1510,x	; Set skidding across water flag

	LDA #$0E
	STA $1DF9

	LDA $1594,x
	CLC
	ADC #$40
	EOR #$7F
	INC A
	SEC
	SBC #$40
	STA $1594,x		; Flip Angle vertically
	JSR GET_ANGLE_SPEEDS

HORZ_2:
	LDA $1504,x
	BEQ WATER_RETURN

	STZ $1504,x

	JSR SPEEDUP_IN_WATER

WATER_RETURN:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SLOWDOWN_IN_WATER:
	LDA $AA,x
	BMI YSLOWDOWN_NEG

	LSR A
YSLOWDOWN_POS:
	STA $AA,x

	LDA $B6,x
	BMI XSLOWDOWN_NEG

	LSR A
XSLOWDOWN_POS:
	STA $B6,x
	RTS

YSLOWDOWN_NEG:
	EOR #$FF
	INC A
	LSR A
	EOR #$FF
	INC A
	BRA YSLOWDOWN_POS

XSLOWDOWN_NEG:
	EOR #$FF
	INC A
	LSR A
	EOR #$FF
	INC A
	BRA XSLOWDOWN_POS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPEEDUP_IN_WATER:
	LDA $AA,x
	BMI YSPEEDUP_NEG

	ASL A
YSPEEDUP_POS:
	STA $AA,x

	LDA $B6,x
	BMI XSPEEDUP_NEG

	ASL A
XSPEEDUP_POS:
	STA $B6,x
	RTS

YSPEEDUP_NEG:
	EOR #$FF
	INC A
	ASL A
	EOR #$FF
	INC A
	BRA YSPEEDUP_POS

XSPEEDUP_NEG:
	EOR #$FF
	INC A
	ASL A
	EOR #$FF
	INC A
	BRA XSPEEDUP_POS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SLOPE_DETECTION:
	PHY
	LDA $15B8,x
	BEQ SLOPE_RETURN
	CLC
	ADC #$04
	TAY
	BMI UPSIDEDOWN_SLOPE

	LDA SLOPE_ANGLE,y
	EOR #$7F
	INC A
	STA $00

	LDA $1594,x
	SEC
	SBC $00
	SEC
	SBC #$40
	BMI SLOPE_RETURN

UPSIDEDOWN_SLOPE:
	TYA
	AND #$7F
	TAY

	LDA $1594,x
	CLC
	ADC SLOPE_ANGLE,y
	EOR #$7F
	INC A
	SEC
	SBC SLOPE_ANGLE,y
	STA $1594,x		; Flip Angle according to slope.
	JSR GET_ANGLE_SPEEDS
	STZ $1588,x
	INC $C2,x
	LDY $C2,x
	LDA STATE,y
	STA $14C8,x
	LDA #$01
	STA $1DF9

SLOPE_RETURN:
	LDA $15B8,x
	CLC
	ADC #$04
	AND #$7F
	SEC
	SBC #$04
	STA $15B8,x
	PLY
	RTS

CosTBL:
	db $7C,$7C,$7C,$7C,$7C,$7C,$7A,$7A,$7A,$78,$78,$78,$76,$76,$74,$74
	db $74,$72,$70,$70,$6E,$6C,$6C,$68,$66,$66,$64,$60,$60,$5E,$5A,$5A
	db $56,$54,$54,$50,$4C,$4C,$48,$44,$44,$42,$3E,$3E,$3A,$36,$36,$32
	db $2C,$2C,$28,$24,$24,$20,$1A,$1A,$16,$12,$12,$0E,$0A,$0A,$04,$00
SinTBL:	db $00,$FC,$F6,$F6,$F2,$EE,$EE,$EA,$E6,$E6,$E0,$DC,$DC,$D8,$D4,$D4
	db $CE,$CA,$CA,$C6,$C2,$C2,$BE,$BC,$BC,$B8,$B4,$B4,$B0,$AC,$AC,$AA
	db $A6,$A6,$A2,$A0,$A0,$9C,$9A,$9A,$98,$94,$94,$92,$90,$90,$8E,$8C
	db $8C,$8C,$8A,$8A,$88,$88,$88,$86,$86,$86,$84,$84,$84,$84,$84,$84
	db $84,$84,$84,$84,$84,$84,$86,$86,$86,$88,$88,$88,$8A,$8A,$8C,$8C
	db $8C,$8E,$90,$90,$92,$94,$94,$98,$9A,$9A,$9C,$A0,$A0,$A2,$A6,$A6
	db $AA,$AC,$AC,$B0,$B4,$B4,$B8,$BC,$BC,$BE,$C2,$C2,$C6,$CA,$CA,$CE
	db $D4,$D4,$D8,$DC,$DC,$E0,$E6,$E6,$EA,$EE,$EE,$F2,$F6,$F6,$FC,$00
	db $00,$04,$0A,$0A,$0E,$12,$12,$16,$1A,$1A,$20,$24,$24,$28,$2C,$2C
	db $32,$36,$36,$3A,$3E,$3E,$42,$44,$44,$48,$4C,$4C,$50,$54,$54,$56
	db $5A,$5A,$5E,$60,$60,$64,$66,$66,$68,$6C,$6C,$6E,$70,$70,$72,$74
	db $74,$74,$76,$76,$78,$78,$78,$7A,$7A,$7A,$7C,$7C,$7C,$7C,$7C,$7C
	db $7C,$7C,$7C,$7C,$7C,$7C,$7A,$7A,$7A,$78,$78,$78,$76,$76,$74,$74
	db $74,$72,$70,$70,$6E,$6C,$6C,$68,$66,$66,$64,$60,$60,$5E,$5A,$5A
	db $56,$54,$54,$50,$4C,$4C,$48,$44,$44,$42,$3E,$3E,$3A,$36,$36,$32
	db $2C,$2C,$28,$24,$24,$20,$1A,$1A,$16,$12,$12,$0E,$0A,$0A,$04,$00

SLOPE_ANGLE:
	db $10,$20,$30,$38,$40,$48,$50,$60,$70

TYPE_OF_SLOPE:
	db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
	db $FF,$FF,$FF,$FF,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
	db $01,$01,$01,$01,$01,$01,$01,$01,$FE,$FE,$FE,$FE,$FE,$FE,$FE,$FE
	db $FE,$FE,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$FD,$FD,$FD,$FD
	db $FD,$03,$03,$03,$03,$03,$FD,$03,$FC,$04,$FF,$FF,$01,$01,$FE,$FE
	db $02,$02,$FD,$FD,$03,$03,$83,$7D,$82,$82,$7E,$7E,$FC,$FC,$04,$04
	db $FD,$FD,$03,$03,$FE,$FE,$02,$02,$FD,$03,$FF,$FF,$FF,$01,$01,$01
	db $FE,$FE,$02,$02,$FD,$FD,$03,$03,$00,$00,$00,$FD,$03,$00,$83,$7D
	db $82,$7E,$00,$FC,$04

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GET_ANGLE_SPEEDS:
	PHY
	LDA $1594,x	; Angle of egg.
	TAY
	LDA CosTBL,y
	STA $B6,x	; X speed.
	LDA SinTBL,y
	STA $AA,x	; Y speed.
	STZ $1504,x	; Clear in water flag
	PLY
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; smoke routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SUB_SMOKE:
	JSR SUB_IS_OFF_SCREEN
	BNE RETURN_SMOKE

	LDA $14
	LSR A
	LSR A
	AND #$03
	TAY
FOUND_ONE:
	LDA #$01
	STA $17C0,y
	LDA #$14
	STA $17CC,y
	LDA $D8,x
	STA $17C4,y
	LDA $E4,x
	STA $17C8,y
RETURN_SMOKE:
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; spawn yoshi's egg shell pieces
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SHYPOS:	db $00,$00,$08,$08
SHXPOS:	db $00,$08,$00,$08
SHYSPD:	db $E8,$E8,$F4,$F4
SHXSPD:	db $FA,$06,$FD,$03
SHTIMR:	db $2A,$2A,$20,$20
SUB_SHELL_PIECE:
	LDA $E4,x
	STA $00
	LDA $D8,x
	STA $01
	LDA $14D4,x
	STA $02
	PHX
	LDY #$03
	LDX #$0B
LOOP1:	LDA $17F0,x
	BEQ SPAWN
LOOP2:	DEX
	BPL LOOP1
	DEC $185D
	BPL LOOP3
	LDA #$0B
	STA $185D
LOOP3:	LDX $185D
SPAWN:	LDA #$03
	STA $17F0,x
	LDA $00
	CLC
	ADC SHXPOS,y
	STA $1808,x
	LDA $01
	CLC
	ADC SHYPOS,y
	STA $17FC,x
	LDA $02
	STA $1814,x
	LDA SHYSPD,y
	STA $1820,x
	LDA SHXSPD,y
	STA $182C,x
	TYA
	LDA SHTIMR,y
	STA $1850,x
	DEY
	BPL LOOP2
	PLX
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SUB_GFX:	    JSR GET_DRAW_INFO       ; sets y = OAM offset

		    LDA $C2,x
		    STA $03

		    PHX		     ; /
                    LDA $00		 ; \ tile x position = sprite x location ($00)
		    STA $0300,y	     ; /

		    LDA $01		 ; \ tile y position = sprite y location ($01)
		    STA $0301,y	     ; /

		    LDA $1632,x      ; Sprite goes behind layers flag
		    EOR #$01
		    ASL #5
		    LDX $03
		    ORA PALETTE,x
		    STA $0303,y	     ; store tile properties

		    LDA #!EggTILEGFXTHING
		    STA $0302,y	     ; /

		    INY		     ; \ increase index to sprite tile map ($300)...
		    INY		     ;  |    ...we wrote 1 16x16 tile...
		    INY		     ;  |    ...sprite OAM is 8x8...
		    INY		     ; /    ...so increment 4 times

		    PLX		     ; pull, X = sprite index
		    LDY #$02		; \ 460 = 2 (all 16x16 tiles)
		    LDA #$00		;  | A = (number of tiles drawn - 1)
		    JSL $01B7B3	     ; / don't draw if offscreen
		    RTS		     ; return



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040            ;  |
                    CMP #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:             db $40,$B0
SPR_T13:             db $01,$FF
SPR_T14:             db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:             db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:   LDA #$0E                ;  |
STORE_03:			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0:   STZ $03					; /

START_SUB:           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_35:           RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return