header
lorom

;Block Duplication Fix v1.4 by lolcats439
;Fixes the block duplication glitch

;The root cause of the glitch is that the sprite used for the glitch gets moved to the left or right by the code at 0191D0, which is the code that moves a sprite in case it gets stuck in a block, to try to get it unstuck; unfortunately, simply NOPing it out makes the sprite disappear when it is inside a block, and for some reason can crash the rom. After this code runs, $9A and $98 are set using the moved position instead of the correct position. The high nybble of the sprite's new x (+ 8) position is one more or one less than the block's actual x position, and then is used to spawn the bounce sprite. The block is then respawned using the coordinates of the bounce sprite, causing the block to be duplicated.

;Duplicating a block vertically is a little different, and is not caused by the stuck-sprite code. It happens when a throwable sprite clips through a block so much that it's y position is up to 2 pixels above the block's y position, but somehow still counts as a hit and triggers the bounce sprite. Since the high nybble of the sprite y position is then one less than the block's y position high nybble, the block is duplicated vertically.

;The fix is to store the sprite's x position to FreeRAM before the stuck sprite code runs, then get it back afterwards. For vertical duplication, the fix is to add 02 to the sprite y position in A to increment the high nybble if necessary before clearing the low nybble.


;Changelog:
;1.0 - initial release
;1.1 - fixed embarrasing side effect that made blocks always duplicate when hit from the side with an object.
;1.2 - fix new glitch explained above
;1.3 - moved the STZ !FreeRAM to a different hijack, so it always runs
;1.4 - changed default freespace from $0660 to $0695, and simplified the patch to remove a possible glitch

!FREESPACE = $15FC4B  ; change to freespace
!FreeRAM = $1F48      ; change to free ram that no other patch is using


org $0192FC           ;\ Hijack code and JSL to custom code
JSL CodeStart         ;/
NOP #2


org $0195A6
JSL GetUnmodifiedSpriteXPos
NOP


org $0195B4
JSL Add2toSpriteYPos


org !FREESPACE
!CodeSize = CodeEnd-CodeStart ;\ 
RATS:                 ;| Write RATS tag to
db "STAR"             ;| prevent other programs
dw !CodeSize-$01      ;| from overwriting this 
dw !CodeSize-$01^$FFFF;/ code!

CodeStart:
LDA.w $1693           ;\ restore overwritten code
STA.w $1868           ;/
LDA $E4,x             ;\ Store x position of sprite to FreeRAM so the original value can be used 
STA !FreeRAM          ;/ after the stuck-sprite code runs
Return:
RTL


GetUnmodifiedSpriteXPos:
LDA !FreeRAM          ;\ Get the sprite x pos 
CLC                   ;| restore overwritten code
ADC #$08              ;|
STZ !FreeRAM          ;/
RTL


Add2toSpriteYPos:
LDA $D8,x             ;\ y position of block
CLC                   ;| add 02 to prevent duplicating blocks vertically,
ADC #$02              ;| by making the high nybble increment if necessary
AND.b #$F0            ;/
RTL
CodeEnd:





