;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSMBWii Inventory
;
; This adds an inventory similar to NSMBWii's.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!inv_gfx_loc		= $0100			; The beginning of where the inventory graphics are written to (in the 8x8 editor).
!inv_palette		= $04			; The inventory palette.
!num_items		= $04			; The number of item types.
!sp2_gfx_file		= $FF			; The modified OW SP2 gfx file. CHANGE THIS IF NEEDED!
!sram_buff		= $1F49			; The inventory item SRAM buffer, !num_items bytes. CHANGE THIS IF NEEDED!
!star_power		= $1DEF			; The star power flag, 1 byte. CHANGE THIS IF NEEDED!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; These don't actually need to be changed... This is all only used in levels,
; so it is safe to use.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!inv_cursor		= $57			; The inventory cursor index, 1 byte.
!inv_dir		= $59			; The inventory direction, 1 byte.
!inv_timer		= $5A			; The inventory timer, 1 byte.
!inv_y			= $5B			; The inventory Y position + 32, 1 byte.
!vram_backup		= $7ED800		; Backup for overwritten layer 1 tilemap, 512 bytes.

org $00827C
		autoclean JSL nmi_main
		NOP
		
org $008385
		JML irq_main
		NOP
		
org $00A660
		JSL star_power
		NOP
		NOP
		
org $00A908
		db !sp2_gfx_file
		
org $00FFD8
		db $07
		
org $0485F3
	-	LDA $00
		CLC
		ADC x_offsets,y
		STA $0200,x
		LDA $01
		ADC y_offsets,y
		STA $0201,x
		LDA #$E8
		STA $0202,x
		LDA #$36
		STA $0203,x
		LDA #$02
		STA $0420,y
		INY
		INX
		INX
		INX
		INX
		CPY #$04
		BNE -
		RTS
		
x_offsets:	db $00,$00,$10,$10
		
y_offsets:	db $00,$10,$00,$10
		
org $0485A7
		JSL ow_main
		NOP

org $04FB0A
		STA $0242,y
		XBA
		AND #$EF			; Disable priority bit 1 on OW sprites.
		STA $0243,y
		LDA $01
		ORA $03
		BNE +
		LDA $00
		STA $0240,y
		LDA $02
		STA $0241,y
		PHP
		TYA
		LSR
		LSR
		PLP
		PHY
		TAY
		ROL
		ASL
		AND #$03
		STA $0430,y
		PLY
		DEY
		DEY
		DEY
		DEY
	+	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

freecode
prot inv_gfx
reset bytes

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Add Item Subroutine
;
; Load the item number into A and then call this subroutine.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	

print "add_item is located at: $", pc

add_item:	PHX
		TAX
		LDA !sram_buff,x		; If the cap isn't reached,
		CMP.l item_caps,x
		BEQ +
		INC				; increase the number of items.
		STA !sram_buff,x
	+	PLX
		RTL
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Items
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

item_actions:	dw mushroom			; The actions to take when using an item.
		dw flower
		dw cape
		dw star

item_caps:	db $63,$63,$63,$63		; The maximum amount of items that can be carried.
		
item_tiles:	db $80,$84,$88,$8C		; The item tiles.
		
item_props:	db $32,$32,$32,$32		; The item yxppccct properties.
		
mushroom:	LDA #$01
		CMP $19
		BCS powerup_shared
		PLA
		PLA
		RTS
		
flower:		LDA #$03
		CMP $19
		BNE powerup_shared
		PLA
		PLA
		RTS
		
cape:		LDA #$02
		CMP $19
		BNE powerup_shared
		PLA
		PLA
		RTS
		
star:		LDA $1490
		BNE +
		LDA #$FF
		STA $1490
		STA !star_power
		BRA sound_shared
		
	+	PLA
		PLA
		RTS

powerup_shared:	LDX $0DB3
		STA $19
		STA $0DB8,x
sound_shared:	LDA #$0A
		STA $1DF9
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; IRQ Hijack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;		
		
layer_1_offsets:
		db $FF,$1F
		
irq_main:	LDA $0100
		CMP #$0E
		BNE .return			; If we're on the overworld,

		LDY $0DD6
		LDX $1F1A,y
		LDA.l layer_1_offsets,x
		SBC !inv_y
		LDX #$01
		
	-	BIT $4212
		BVS -
	-	BIT $4212
		BVC -				; wait for H-blank,
		
		STZ $210D			; set layer 1 x to 0,
		STZ $210D
		STA $210E			; set layer 1 y to show the inventory,
		STX $2105			; disable layer 3 priority,
		STX $210E
		STZ $212E			; windowing,
		STZ $2131			; and color math.
		JML $0083B2
		
	.return	LDA #$81
		LDY $0D9B
		JML $00838A
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NMI Hijack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

inv_end:	db $00,$FC
		
inv_y_speeds:	db $04,$FC
		
nmi_main:	LDY $0100
		CPY #$0E
		BNE .return			; If we're on the overworld,
		
		PHB
		PHK
		PLB
		LDA #$15
		STA $212E			; restore windowing settings
		LDA $40
		STA $2131			; and color math settings.
		
		LDA !inv_timer
		BEQ +
		
		DEC !inv_timer			; Decrement inventory timer.
		LDX !inv_dir
		LDA !inv_y
		CLC
		ADC inv_y_speeds,x
		STA !inv_y			; Move inventory Y position.
		CMP inv_end,x
		BNE +				; When the inventory first opens or fully closes,
		
		JSR dma_vram			; DMA the VRAM backup, inventory graphics, and tilemap.
		
	+	PLB
		
		LDA #$81
		LDY !inv_y
		BEQ .return
		TYA
		SEC
		SBC #$20
		STA $4209			; Set IRQ at VC = inventory Y position.
		STZ $420A
		
		LDA #$A1			; Enable IRQ.
	.return	LDY $13C6
		CPY #$08
		RTL

vram_addr:	dw $2B80,$2000
		
vram_tilemap:	dw !inv_palette<<10|$2000|!inv_gfx_loc
		dw !inv_palette<<10|$6000|(!inv_gfx_loc+1)
		dw !inv_palette<<10|$6000|!inv_gfx_loc
		dw !inv_palette<<10|$2000|(!inv_gfx_loc+2)
		dw !inv_palette<<10|$2000|(!inv_gfx_loc+3)
		dw !inv_palette<<10|$6000|(!inv_gfx_loc+2)
		dw !inv_palette<<10|$2000|(!inv_gfx_loc+2)
		dw !inv_palette<<10|$2000|(!inv_gfx_loc+3)
		dw !inv_palette<<10|$6000|(!inv_gfx_loc+2)
		dw !inv_palette<<10|$A000|!inv_gfx_loc
		dw !inv_palette<<10|$E000|(!inv_gfx_loc+1)
		dw !inv_palette<<10|$E000|!inv_gfx_loc
		
dma_vram:	LDA #$80
		STA $2115			; VRAM addr auto-increment on high byte.
		
		LDY $0DD6
		LDA $1F1A,y
		ASL
		TAX
		REP #$20
		LDA vram_addr,x
		LDX #$02
		STA $00
		STA $2116
		LDA #$1801
		LDY !inv_dir
		BEQ .write
		LDA #$3981
	.write	STA $4310
		LDA #!vram_backup
		STA $4312
		STX $4316
		LDY #!vram_backup>>16
		STY $4314
		STX $420B			; DMA the VRAM backup ($0200 bytes).
		
		LDY !inv_dir
		BEQ .return			; If the inventory is opening,
		
		LDA #!inv_gfx_loc*16
		STA $2116			; Set the VRAM address.
		LDA #$1801
		STA $4310
		LDA #inv_gfx
		STA $4312
		LDY #$80
		STY $4315
		LDY #inv_gfx>>16
		STY $4314
		STX $420B			; DMA the inventory GFX ($80 bytes).

		INX				; Overwrite part of the layer 1 tilemap with the inventory tilemap.
	.loop	PHX
		TXA
		XBA
		LSR
		LSR
		LSR
		ADC $00
		STA $2116			; Set VRAM address.
		
		TXA
		STA $02
		ASL
		ADC $02
		ASL
		TAX				; X = 6 * X
		LDA vram_tilemap,x
		STA $2118			; Set the first tile.
		LDA vram_tilemap+2,x
		LDY #$1D
	-	STA $2118			; Set the next 30 tiles.
		DEY
		BPL -
		LDA vram_tilemap+4,x
		STA $2118			; Set the last tile.
		PLX
		DEX
		BPL .loop			; Go to the next row.
		
	.return	SEP #$20
		RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Star Power Hijack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
star_power:	LDA !star_power
		BEQ +
		
		JSL $01C580
		LDA #$00
		STA !star_power
		
	+	LDA $13F4
		ORA $13F5
		RTL
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; OW Hijack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

digits:		db $E0,$E1,$E2,$E3,$E4,$E5,$F0,$F1
		db $F2,$F3

ow_main:	LDA !star_power
		BEQ +
		STA $1490
		
	+	LDA $13D9
		CMP #$03
		BNE .return			; If we're on a level tile,
		LDA $13D4
		BNE .return			; and we're not looking around,
		
		LDA !inv_timer
		BNE +
		LDA $16
		AND #$20
		BEQ +				; open/close the inventory if select is pressed.
		
		LDA #$1C
		STA $1DFC
		LDA !inv_dir
		EOR #$01
		STA !inv_dir			; Switch the inventory direction,
		LDA #$08
		STA !inv_timer			; and set the timer.
		
	+	LDA !inv_y
		BEQ .return

		LDA !inv_cursor
		STA $4204
		STZ $4205
		LDA #$0A
		STA $4206			; Divide the cursor position by 10.
		
		PHB
		PHK
		PLB
		JSR update_inv			; Update the inventory.
		
		LDA !inv_y
		LDX !inv_timer
		BEQ +
		LDX !inv_dir
		CLC
		ADC inv_y_speeds,x
	+	STA $0C				; Offset the inventory graphics to counteract a one-frame delay.
		JSR draw_inv			; Draw the inventory.
		
		STZ $16
		STZ $17
		STZ $18				; Disable controls.
		PLB
		
	.return	REP #$20
		LDA #$001E
		RTL
		
draw_inv:
if !num_items > 10
		LDA $14
		ASL
		ASL
		ASL
		ROL
		LDA #$00
		PHP
		ADC $0C
		SBC #$0F
		PLP
		STA $0215
		
		LDA #$00
		SBC $0C
		EOR #$FF
		SBC #$17
		STA $0211			; Set the arrows' Y positions.
		
		LDA #$F4
		STA $0210
		STA $0214			; Set the arrows' X positions.
		
		LDY $0E
		BNE +
		LDA #$FF
	+	STA $0212
	
		LDA #$F5
		LDX $00
		CPY.b #!num_items-0.5/10
		BNE +
		LDA #$FF
	+	STA $0216			; Set the arrows' tiles.
		
		LDA #$34
		STA $0213
		STA $0217			; Set the arrows' properties.
		
		STZ $0424			; Set the arrows' size.
		STZ $0425			
endif
		LDX #$00
		TXY
	-	STX $4202
		LDA #$18
		STA $4203
		
		STX $00
		LDA $0E
		ASL
		ASL
		ADC $0E
		ASL
		ADC $00
		STA $00				; Compute the powerup index.
		CPX $0F
		BNE .no_digits			; If the current item is selected,
		
		PHY
		TYA
		LSR
		LSR
		TAY
		LDA #$00
		STA $0426,y
		STA $0427,y			; Mark the digits as 8x8 tiles.
		PLY
		LDX $00
		LDA !sram_buff,x
		
		PHX
		JSL $00974C
		PHA
		LDA #$FF			; Hide the tens digit if it is 0.
		CPX #$00
		BEQ +
		LDA digits,x
	+	STA $021A,y			; Set the tens digit's tile.
		PLA
		TAX
		LDA digits,x
		STA $021E,y			; Set the ones digit's tile.
		PLX
		
		LDA #$30
		STA $021B,y
		STA $021F,y			; Set the digits' properties
		
		LDA $4216
		ADC #$12
		STA $0218,y
		ADC #$06
		STA $021C,y			; Set the digits' x positions.
		
		LDA $0C
		SBC #$0B
		STA $0219,y
		STA $021D,y			; Set the digits' y positions.
		
		TYA
		CLC
		ADC #$08
		TAY
		
	.no_digits
		LDA $0C
		SEC
		SBC #$18
		STA $0219,y			; Set the item's Y position.
		LDA $4216
		ADC #$0B
		STA $0218,y			; Set the item's X position.
		
		PHX
		LDX $00
		LDA item_props,x
		STA $021B,y			; Set the item's properties.
		LDA item_tiles,x
		PLX
		CPX $0F
		BEQ +				; If the item is not selected, use the next tile.
		INC
		INC
	+	STA $021A,y			; Set the item's tiles.
		
		PHY
		TYA
		LSR
		LSR
		TAY
		LDA #$02
		STA $0426,y			; Mark the item as a 16x16 tile.
		PLY
		
		LDA $0E
		CMP.b #!num_items-0.5/10
		BNE +
		CPX #!num_items%10-1
		BRA ++
	+	CPX #$09
	++	BEQ .return
		INX
		INY
		INY
		INY
		INY
		JMP -				; Go to the next item.
	.return	RTS
	
update_inv:	LDA $4214
		STA $0E				; Set the cursor's page number.
		LDA $4216
		STA $0F				; Set the cursor's index.
		
		LSR $16
		BCC .no_r			; If right is pressed,
		LDA !inv_cursor
		CMP #!num_items-1
		BEQ .no_r			; and the cursor isn't at the rightmost item,
		INC $0F				; move the cursor right.
		BRA +
		
	.no_r	LSR $16
		BCC .no_l			; If left is pressed,
		LDA !inv_cursor
		BEQ .no_l			; and the cursor isn't at the leftmost item,
		DEC $0F				; move the cursor left.
		BRA +

	.no_l	LSR $16
		BCC .no_d			; If down is pressed,
		LDA $0E
		CMP.b #!num_items-0.5/10
		BEQ .no_d			; and the cursor isn't at the bottommost page,
		INC $0E				; move the page down.
		BRA +
		
	.no_d	LSR $16
		BCC .no_u			; If up is pressed,
		LDA $0E
		BEQ .no_u			; and the cursor isn't at the first page,
		DEC $0E				; move the page up.
	+	LDA #$23
		STA $1DFC
		BRA .return
		
	.no_u	LDA $16
		ORA $18
		AND #$CC
		BEQ .return			; If A, B, X, or Y is pressed,
		
		LDX !inv_cursor
		LDA !sram_buff,x
		BEQ .return			; and the item stack isn't empty,
		TXA
		ASL
		TAX
		JSR (item_actions,x)		; run the item action.
		
		LDX !inv_cursor
		LDA !sram_buff,x
		DEC
		STA !sram_buff,x
		RTS
		
	.return	LDA $0F
		BPL +				; If the index is negative,
		DEC $0E
		LDA #$09
		STA $0F				; move a page back.
	+	CMP #$0A
		BCC +				; If the index is too large,
		INC $0E
		STZ $0F				; move a page forward.
	+	LDA $0E
		ASL
		ASL
		ADC $0E
		ASL
		ADC $0F				; Set the cursor to 10 * page + index
		CMP #!num_items
		BCC .no_max
		LDA #!num_items-1		; Set the cursor <= !num_items-1.
	.no_max	STA !inv_cursor
		RTS
		
freedata
inv_gfx:
		incbin "inventory_gfx.bin"
		
print "Bytes inserted: ", bytes