;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SMB3 Treasure Chest
;
; When touched, this sprite will immediately turn into smoke and put an item
; into the player's inventory. The item inside of it depends on the sprite's X
; position within the level. To programmatically change the item, modify $1504.
;
; If the extra bit is set, then this sprite will end the level after the item
; is collected.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!add_item		= $90F391		; The location of the add_item subroutine. CHANGE THIS!
!chest_tile		= $C2			; The treasure chest's tile.
!music			= $0C			; Which music to use when ending a level.
!item_types		= $04			; The number of item types. CHANGE THIS!

item_tiles:	db $24,$26,$0E,$48		; The item tiles.
		
item_props:	db $08,$0A,$04,$04		; The item properties.

		print "INIT ", pc
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$20
		LSR #4
		STA $4204
		SEP #$20
		LDA #!item_types
		STA $4206
		LDA $7FAB10,x
		AND #$04
		STA $1510,x
		LDA $1504,x
		BNE +				; If the item was modified by something else, don't set it again.
		LDA $4216
		STA $1504,x			; Set the item based on X position modulo !item_types.
	+	RTL
		
		print "MAIN ", pc
		PHB
		PHK
		PLB
		JSR main
		PLB
		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
main:		JSR gfx
		
		LDA $9D
		BNE .return
		
		LDA $C2,x
		BNE .item
		
		JSL $01802A			; Update gravity.
		JSL $01A7DC
		BCC .return			; If the player touched the sprite,
		
		LDA #$D0
		STA $AA,x			; set the item's Y speed,
		INC $C2,x			; turn the chest into the item,
		LDA #$1E
		STA $1DF9
	.puff	LDY #$03
	-	LDA $17C0,y
		BNE +
		INC
		STA $17C0,y			; and show a puff of smoke.
		LDA $D8,x
		STA $17C4,y
		LDA $E4,x
		STA $17C8,y
		LDA #$1B
		STA $17CC,y
		RTS
		
	+	DEY
		BPL -
	.return	RTS

	.item	LDA $154C,x
		BEQ .rise
		CMP #$01
		BNE .return			; If the item timer is $01,
		
		LDA $1504,x
		JSL !add_item			; give the item
		STZ $14C8,x			; and kill the sprite.
		LDA $1510,x
		BEQ .return			; If the extra bit is set,
		LDA #$FF
		STA $1493
		STA $13C6
		LDA #!music
		STA $1DFB			; end the level.
		RTS
		
	.rise	JSL $01801A
		INC $AA,x			; Update Y speeds.
		BMI .return
		LDA #$70
		STA $154C,x			; Set the item timer.
		LDA #$15
		STA $1DFC
		RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
gfx:		LDA $154C,x
		BEQ +
		AND #$0C
		BEQ .return
		
	+	JSR get_draw_info
		
		LDA $00
		STA $0300,y			; Set X position.
		LDA $01
		STA $0301,y			; Set Y position.
		LDA $C2,x
		BEQ .chest			; If the sprite is an item,
		PHX
		LDA $1504,x
		TAX
		LDA.l item_tiles,x		; use the item tile
		STA $0302,y
		LDA.l item_props,x		; and the item properties.
		PLX
		BRA +
	.chest	LDA #!chest_tile	
		STA $0302,y			; Otherwise, use the chest tile and properties.
		LDA $15F6,x
	+	ORA $64
		STA $0303,y
		
		LDA #$00
		LDY #$02
		JSL $01B7B3
	.return	RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
spr_t1:		db $0C,$1C
spr_t2:		db $01,$02

get_draw_info:	STZ $15A0,x
		STZ $186C,x
		
		LDA $E4,x
		CMP $1A
		LDA $14E0,x
		SBC $1B
		BEQ .on_scr_x
		INC $15A0,x

	.on_scr_x
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$20
		SEC
		SBC $1A
		CLC
		ADC #$0040
		CMP #$0180
		SEP #$20
		ROL
		AND #$01
		STA $15C4,x
		BNE .invalid
		
		LDY #$00
		LDA $1662,x
		AND #$20
		BEQ .loop
		
		INY 
	.loop	LDA $D8,x
		CLC
		ADC spr_t1,y
		PHP
		CMP $1C
		ROL $00
		PLP
		LDA $14D4,x
		ADC #$00
		LSR $00
		SBC $1D
		BEQ .on_scr_y
		
		LDA $186C,x
		ORA spr_t2,y
		STA $186C,x
	.on_scr_y
		DEY
		BPL .loop

		LDY $15EA,x
		LDA $E4,x
		SEC
		SBC $1A
		STA $00
		LDA $D8,x 
		SEC
		SBC $1C
		STA $01
		RTS

	.invalid
		PLA
		PLA
		RTS
