;####################################################################
;
; 	RESETTING MADE EASY
; 	by WhiteYoshiEgg
; 	made for Cellcen, but everyone's free to use this, of course.
;
;####################################################################
;
; - What does this patch do?
;    This patch resets the level when L+R are pressed at the same time.
;    "Resetting" means you get teleported to the start of the level.

; - What do I have to do to make it work?
;    First you need to patch this to your ROM of course. But to actually
;    make it work, you have to set the screen exit of screen 0 to the
;    start of the level.
;    EXAMPLE: Screen exit of screen 0 in level 105 must be set to level 105,
;    Screen exit of screen 0 in level 106 must be set to level 106 and so on.
;    This is where you will be teleported when pressing L+R. Of course, you
;    can set it to anything you want, but you'll normally want to go back to
;    the start of a level when resetting.

; - Can I change the key combination I have to press?
;    Yes. Look for the lines AND #$10 and AND #$20 in the code. The 10 and 20
;    is what you have to change. Look at SMWC's RAM Map to see what value a
;    key has. This patch uses the values from address $17, but you can easily
;    make it read from $15 by changing both $17's to $15.

; - Is it possible to only press one key to reset?
;    Yes. The comments in the code tell you which lines to remove.

; - Anything else I should know?
;    Yes. For example, it probably won't work in vertical levels. I can't guarantee
;    that there are no other bugs either, so I highly recommend making a backup
;    before using this. Also, LM will probably tell you something about the checksum
;    being wrong. Don't worry about that - simply save something, and that message
;    will be gone.

; - Do I need to give credit?
;    You don't really have to, but I sure appreciate it. ;)

;####################################################################
;####################################################################

!FreeSpace = $1F8000 	;   POINT TO FREE SPACE!

header
lorom

org $8F89		; \ Hijack a routine to
JSL CustomCode		; / jump to our custom code

org !FreeSpace		; \
!CodeSize = End-Start	;  |
db "STAR"		;  | Write RATS Tag
dw !CodeSize-$01	;  |
dw !CodeSize-$01^$FFFF	; /

Start:
CustomCode:

STZ $01			; \ Restore hijacked code - Do not touch this
STZ $00			; /

PHA
PHY
LDA $71			; \
CMP #$09		;  | Disable resetting when Mario dies
BEQ Return		; /
LDA $17			; \
AND #$10		;  | First key to press
BEQ Return		; /
LDA $17			; \ 
AND #$20		;  | Second key to press - remove these 3 lines to only use 1 key
BEQ Return		; /
LDY $95			; \
LDA $19B8		;  |
STA $19B8,y		;  | Transfer exit from screen 0 to current screen
LDA $19D8		;  |
STA $19D8,y		; /
LDA #$06		; \
STA $71			;  | Teleport
STZ $88			;  |
STZ $89			; /

Return:			; \
PLY			;  |
PLA			;  |
RTL			;  | The End
End:			; /